Package: widr
Title: Interface to the World Inequality Database (WID)
Version: 0.1.1
Authors@R: person("Cheryl Isabella", "Lim", email = "cheryl.academic@gmail.com", role = c("aut", "cre"))
Description: Interface to the World Inequality Database (WID) API <https://wid.world>. Downloads distributional national accounts data with filters for country, year, percentile, age group, and population type. Includes code validation and reference tables. Independent implementation unaffiliated with the World Inequality Lab (WIL) or the Paris School of Economics.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: base64enc, jsonlite, digest, ggplot2, httr2, scales, tools,
        utils
Suggests: testthat (>= 3.0.0), covr, knitr, rmarkdown, withr
VignetteBuilder: knitr
URL: https://github.com/cherylisabella/widr
BugReports: https://github.com/cherylisabella/widr/issues
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-20 20:02:05 UTC; Isabella
Author: Cheryl Isabella Lim [aut, cre]
Maintainer: Cheryl Isabella Lim <cheryl.academic@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-20 21:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 11:04:54 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,wid_df)
S3method(print,wid_query)
export(download)
export(download_wid)
export(wid_ages)
export(wid_cache)
export(wid_cache_clear)
export(wid_cache_list)
export(wid_concepts)
export(wid_convert)
export(wid_countries)
export(wid_decode)
export(wid_encode)
export(wid_fetch)
export(wid_filter)
export(wid_gini)
export(wid_is_valid)
export(wid_metadata)
export(wid_percentile_ratio)
export(wid_percentiles)
export(wid_plot_compare)
export(wid_plot_lorenz)
export(wid_plot_timeseries)
export(wid_pop_types)
export(wid_query)
export(wid_search)
export(wid_series_types)
export(wid_set_key)
export(wid_tidy)
export(wid_top_share)
export(wid_validate)
importFrom(base64enc,base64encode)
importFrom(digest,digest)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,req_timeout)
importFrom(httr2,req_url_query)
importFrom(httr2,request)
importFrom(httr2,resp_body_string)
importFrom(httr2,resp_check_status)
importFrom(jsonlite,fromJSON)
importFrom(scales,percent_format)
importFrom(stats,setNames)
importFrom(tools,R_user_dir)
importFrom(utils,head)
importFrom(utils,modifyList)

# widr

> R interface to the [World Inequality Database](https://wid.world) (WID).
This package downloads distributional national accounts data (DINA), validates and decodes WID variable codes, and provides helpers for currency conversion, inequality measurement, and plotting. 

## Installation

```r
# CRAN
install.packages("widr")

# Development version
remotes::install_github("cherylisabella/widr")
```

## Variable codes

WID variables follow the grammar `<type><concept>[age][pop]`:

| Component | Width | Example | Meaning |
|-----------|-------|---------|---------|
| `type` | 1 letter | `s` | share |
| `concept` | 5–6 letters | `ptinc` | pre-tax national income |
| `age` | 3 digits | `992` | adults 20+ |
| `pop` | 1 letter | `j` | equal-split between spouses |

`sptinc992j` = share of pre-tax national income for equal-split adults aged 20+.

```r
wid_decode("sptinc992j")
#> $series_type  "s"
#> $concept      "ptinc"
#> $age          "992"
#> $pop          "j"

wid_encode("s", "ptinc", age = "992", pop = "j")   # "sptinc992j"
wid_is_valid(series_type = "s", concept = "ptinc")  # TRUE
wid_search("national income")                        # search the concept table
```

Full catalogue: <https://wid.world/codes-dictionary/>

## Download data

```r
library(widr)

# Top 1% pre-tax income share, United States, 2000–2022
top1 <- download_wid(
  indicators = "sptinc992j",
  areas      = "US",
  perc       = "p99p100",
  years      = 2000:2022)

top1
#> <wid_df>  23 rows | 1 countries | 1 variables
#>   country   variable percentile year  value age pop
#> 1      US sptinc992j  p99p100   2000  0.168 992   j
```

`download_wid()` returns a `wid_df`, a `data.frame` subclass that works natively with dplyr, ggplot2, and base R without any `$data` unwrapping.

### Key parameters

| Parameter | Default | Description |
|-----------|---------|-------------|
| `indicators` | `"all"` | Variable codes |
| `areas` | `"all"` | ISO-2 country / region codes |
| `years` | `"all"` | Integer vector or `"all"` |
| `perc` | `"all"` | Percentile codes, e.g. `"p99p100"` |
| `ages` | `"992"` | Three-digit age code |
| `pop` | `"j"` | Population unit |
| `metadata` | `FALSE` | Attach source info as `attr(., "wid_meta")` |
| `include_extrapolations` | `TRUE` | Include interpolated points |
| `cache` | `TRUE` | Cache responses to disc |

## Tidyverse integration

```r
library(dplyr)
library(ggplot2)

top1 |>
  wid_tidy(country_names = FALSE) |>
  filter(year >= 1990) |>
  ggplot(aes(year, value)) +
  geom_line(colour = "#58a6ff", linewidth = 0.9) +
  scale_y_continuous(labels = scales::percent_format()) +
  labs(title = "Top 1% pre-tax income share — United States", x = NULL, y = NULL) +
  theme_minimal()
```

## Inequality measures

```r
dist <- download_wid("sptinc992j", areas = c("US", "FR"), perc = "all", years = 1990:2022)

wid_gini(dist)                        # Gini coefficient
wid_top_share(dist, top = 0.01)       # top 1% share
wid_top_share(dist, top = 0.10)       # top 10% share

thresh <- download_wid("tptinc992j", areas = "US", perc = "all")
wid_percentile_ratio(thresh)          # P90/P10
```

## Currency conversion

```r
download_wid("aptinc992j", areas = c("US", "FR"), perc = "p0p50") |>
  wid_convert(target = "ppp", base_year = "2022")
```

Supported targets: `"lcu"`, `"usd"`, `"eur"`, `"gbp"`, `"ppp"`, `"yppp"`.

## Plotting

```r
wid_plot_timeseries(dist)                 # time series, one line per country
wid_plot_compare(dist, year = 2020)       # cross-country bar chart
wid_plot_lorenz(dist, country = "US")     # Lorenz curve
```

## Reusable queries

```r
q <- wid_query(indicators = "sptinc992j", areas = c("US", "FR"))
q <- wid_filter(q, years = 2010:2022)
wid_fetch(q)
```

## Reference tables

Six tables synchronised from [WID](https://wid.world/codes-dictionary/):

| Table | Contents |
|-------|----------|
| `wid_series_types` | Series type codes (`s`, `a`, `m`, …) |
| `wid_concepts` | Concept codes (`ptinc`, `hweal`, `nninc`, …) |
| `wid_ages` | Age group codes (`992`, `999`, …) |
| `wid_pop_types` | Population unit codes (`j`, `i`, `t`, …) |
| `wid_countries` | Country and region codes |
| `wid_percentiles` | Percentile codes (`p99p100`, `p0p50`, …) |

Independent implementation unaffiliated with the World Inequality Lab (WIL) or the Paris School of Economics. 
Data maintained by WIL.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)

## ----install------------------------------------------------------------------
# install.packages("widr")
# 
# # Development version
# remotes::install_github("cherylisabella/widr")

## ----codes--------------------------------------------------------------------
# wid_search("national income")                           # keyword search across concepts
# wid_decode("sptinc992j")                                # parse into components
# wid_encode("s", "ptinc", age = "992", pop = "j")       # build from components
# wid_is_valid(series_type = "s", concept = "ptinc")      # non-throwing validation

## ----download-----------------------------------------------------------------
# library(widr)
# 
# # Top 1% pre-tax income share, United States, 2000-2022
# top1 <- download_wid(
#   indicators = "sptinc992j",
#   areas      = "US",
#   perc       = "p99p100",
#   years      = 2000:2022
# )
# 
# top1
# #> <wid_df>  23 rows | 1 countries | 1 variables
# #>   country   variable percentile year  value age pop
# #> 1      US sptinc992j  p99p100   2000  0.168 992   j
# #> ...

## ----multi--------------------------------------------------------------------
# shares <- download_wid(
#   indicators = "sptinc992j",
#   areas      = c("US", "FR", "DE", "CN"),
#   perc       = c("p90p100", "p99p100"),
#   years      = 1980:2022
# )

## ----extrap-------------------------------------------------------------------
# download_wid("sptinc992j", areas = "MZ", include_extrapolations = FALSE)

## ----meta---------------------------------------------------------------------
# result <- download_wid("sptinc992j", areas = "US", metadata = TRUE)
# attr(result, "wid_meta")
# #>     variable country      source method quality    imputation
# #> 1 sptinc992j      US Tax records    DFL    high adjusted surveys

## ----tidy-pipe----------------------------------------------------------------
# library(dplyr)
# library(ggplot2)
# 
# top1 |>
#   wid_tidy(country_names = FALSE) |>
#   filter(year >= 1990) |>
#   ggplot(aes(year, value)) +
#   geom_line(colour = "#58a6ff", linewidth = 0.9) +
#   scale_y_continuous(labels = scales::percent_format()) +
#   labs(title = "Top 1% pre-tax income share - United States",
#        x = NULL, y = NULL) +
#   theme_minimal()

## ----query--------------------------------------------------------------------
# q <- wid_query(indicators = "sptinc992j", areas = c("US", "FR"), cache = FALSE)
# q <- wid_filter(q, years = 2010:2022)
# wid_fetch(q)

## ----cache--------------------------------------------------------------------
# wid_cache_list()    # list cached queries
# wid_cache_clear()   # remove all

## ----convert------------------------------------------------------------------
# # Bottom 50% average income, four countries - convert to 2022 USD PPP
# download_wid("aptinc992j", areas = c("US", "FR", "CN", "IN"), perc = "p0p50") |>
#   wid_convert(target = "ppp", base_year = "2022")

## ----gini---------------------------------------------------------------------
# dist <- download_wid("sptinc992j", areas = c("US", "FR"), perc = "all",
#                      years = 1990:2022)
# wid_gini(dist)
# #>   country year  gini
# #> 1      FR 1990 0.411
# #> 2      US 1990 0.453

## ----top-share----------------------------------------------------------------
# wid_top_share(dist, top = 0.01)   # top 1%
# wid_top_share(dist, top = 0.10)   # top 10%

## ----perc-ratio---------------------------------------------------------------
# thresh <- download_wid("tptinc992j", areas = "US", perc = "all")
# wid_percentile_ratio(thresh)                                          # P90/P10
# wid_percentile_ratio(thresh, numerator = "p90", denominator = "p50") # P90/P50

## ----plot---------------------------------------------------------------------
# # Time series - one line per country; facet = TRUE for separate panels
# wid_plot_timeseries(shares,
#   country_labels = c(US = "United States", FR = "France",
#                      DE = "Germany",       CN = "China"))
# 
# # Cross-country bar chart for a single year
# wid_plot_compare(shares, year = 2020)
# 
# # Lorenz curve
# wid_plot_lorenz(dist, country = "US")

## ----full-example-------------------------------------------------------------
# library(widr); library(dplyr); library(ggplot2)
# 
# download_wid(
#   indicators = "aptinc992j",
#   areas      = c("US", "FR", "CN", "IN"),
#   perc       = "p0p50",
#   years      = 1990:2022
# ) |>
#   wid_convert(target = "ppp", base_year = "2022") |>
#   wid_tidy(country_names = TRUE) |>
#   ggplot(aes(year, value, colour = country_name)) +
#   geom_line(linewidth = 0.8) +
#   scale_y_continuous(labels = scales::dollar_format()) +
#   labs(title    = "Bottom 50% average pre-tax income",
#        subtitle = "2022 USD PPP · equal-split adults 20+",
#        x = NULL, y = NULL, colour = NULL)

# download_wid() + query builder

new_wid_df <- function(df) structure(df, class = c("wid_df", "data.frame"))

#' Print a wid_df object
#'
#' This method prints a summary of a \code{wid_df} object,
#' including the number of rows, countries, and variables.
#'
#' @param x A \code{wid_df} object.
#' @param ... Additional arguments passed to \code{print()}.
#' @return Invisibly returns the input \code{x}.
#' @examples
#' \donttest{
#' d <- data.frame(country = "US", variable = "sptinc992j",
#'                 percentile = "p99p100", year = "2020",
#'                 value = 0.19, age = "992", pop = "j",
#'                 stringsAsFactors = FALSE)
#' class(d) <- c("wid_df", "data.frame")
#' print(d)
#' }
#' @export
print.wid_df <- function(x, ...) {
  cat(sprintf("<wid_df>  %d rows | %d countries | %d variables\n",
              nrow(x), length(unique(x$country)), length(unique(x$variable))))
  NextMethod()
  invisible(x)
}

# Stub target: does catalogue lookup + filtering + data fetch; returns plain df of rows.
.fetch <- function(indicators, areas, perc, ages, pop, no_extrapolation, verbose) {
  if (verbose) message("Fetching variable catalogue...")
  variables <- do.call(rbind, lapply(
    if (identical(indicators, "all")) "all" else indicators,
    .get_variables_areas,
    areas = if (identical(areas, "all")) "all" else areas))
  
  if (is.null(variables) || nrow(variables) == 0L) {
    warning("No data found for this selection.", call. = FALSE)
    return(NULL)
  }
  variables <- .filter_vars(variables, perc, ages, pop)
  if (nrow(variables) == 0L) {
    warning("No data after applying filters.", call. = FALSE)
    return(NULL)
  }
  
  variables$api_code <- paste(variables$variable, variables$percentile,
                              variables$age, variables$pop, sep = "_")
  if (verbose) message(sprintf("Fetching %d variable(s)...", length(unique(variables$variable))))
  
  data <- .chunk_fetch(variables, size = 10L, function(countries, codes)
    .get_data_variables(countries, codes, no_extrapolation = no_extrapolation))
  
  if (is.null(data) || nrow(data) == 0L) {
    warning("API returned no data.", call. = FALSE)
    return(NULL)
  }
  
  data <- .parse_indicator(data)
  data <- merge(data, unique(variables[, c("variable","percentile","age","pop")]),
                by = c("variable","percentile"), all.x = TRUE)
  data <- data[!duplicated(data[, c("country","variable","percentile","year")]), ]
  data[order(data$country, data$variable, data$percentile, data$year), ]
}

.fetch_meta <- function(variables)
  .chunk_fetch(variables, size = 10L, function(countries, codes)
    .get_metadata_variables(countries, codes))

#' Download data from WID.world
#'
#' Returns a \code{wid_df}, a classed \code{data.frame} that works natively
#' with dplyr, ggplot2, and all base-R operations.
#'
#' @param indicators WID variable codes or \code{"all"}.
#' @param areas      ISO-2 area codes or \code{"all"}.
#' @param years      Integer vector or \code{"all"}.
#' @param perc       Percentile codes or \code{"all"}.
#' @param ages       Age codes or \code{"all"}. Default \code{"992"}.
#' @param pop        Population unit codes or \code{"all"}. Default \code{"j"}.
#' @param metadata   Logical. If \code{TRUE}, fetches metadata as a
#'   \code{"wid_meta"} attribute on the returned object. Default \code{FALSE}.
#' @param include_extrapolations Logical. Default \code{TRUE}.
#' @param verbose    Logical. Default \code{FALSE}.
#' @param cache      Logical. Default \code{TRUE}.
#' @return A \code{wid_df} (a classed \code{data.frame}) with columns
#'   \code{country}, \code{variable}, \code{percentile}, \code{year},
#'   \code{value}, \code{age}, \code{pop}. If \code{metadata = TRUE},
#'   a \code{"wid_meta"} attribute is attached.
#' @examples
#' \donttest{
#'   if (nzchar(Sys.getenv("WID_API_KEY"))) {
#'     d <- download_wid(
#'       indicators = "sptinc992j",
#'       areas      = "US",
#'       perc       = "p99p100",
#'       years      = 2010:2020
#'     )
#'   }
#' }
#' @export
download_wid <- function(indicators = "all", areas = "all", years = "all",
                         perc = "all", ages = "992", pop = "j",
                         metadata = FALSE, include_extrapolations = TRUE,
                         verbose = FALSE, cache = TRUE) {
  if (identical(indicators, "all") && identical(areas, "all"))
    stop("Specify at least one of `indicators` or `areas`.", call. = FALSE)
  
  if (!identical(indicators, "all")) indicators <- as.character(indicators)
  if (!identical(areas,  "all")) areas <- wid_validate(areas = as.character(areas))$areas
  if (!identical(years,  "all")) years <- wid_validate(years = years)$years
  if (!identical(perc,   "all")) perc  <- wid_validate(perc  = as.character(perc))$perc
  if (!identical(ages,   "all"))
    ages <- wid_validate(age = formatC(suppressWarnings(as.integer(ages)),
                                       width = 3L, flag = "0"))$age
  if (!identical(pop,    "all")) pop <- wid_validate(pop = as.character(pop))$pop
  
  key <- .cache_key(indicators, areas, years, perc, ages, pop, metadata, include_extrapolations)
  if (cache) { hit <- .cache("get", key); if (!is.null(hit)) return(hit) }
  
  if (verbose) message("Fetching data from WID.world...")
  
  rows <- .fetch(indicators, areas, perc, ages, pop,
                 no_extrapolation = !include_extrapolations, verbose = verbose)
  
  # If stub returned a wid_df directly, pass through unchanged
  if (inherits(rows, "wid_df")) rows <- as.data.frame(rows)
  
  if (is.null(rows)) {
    empty <- new_wid_df(
      .empty("country", "variable", "percentile", "year", "value", "age", "pop"))
    if (cache) .cache("set", key, empty)
    return(invisible(empty))
  }
  
  if (!identical(years, "all"))
    rows <- rows[as.integer(rows$year) %in% as.integer(years), ]
  
  rownames(rows) <- NULL
  result <- new_wid_df(rows)
  
  if (metadata) {
    vars <- do.call(rbind, lapply(
      if (identical(indicators, "all")) "all" else indicators,
      .get_variables_areas,
      areas = if (identical(areas, "all")) "all" else areas))
    meta <- if (!is.null(vars) && nrow(vars)) .fetch_meta(vars) else data.frame()
    attr(result, "wid_meta") <- meta
  }
  
  if (cache) .cache("set", key, result)
  result
}

#' @rdname download_wid
#' @export
download <- download_wid

#' Build a reusable WID query object
#'
#' Creates a \code{wid_query} object that stores query parameters for WID data fetching.
#'
#' @param ... Named arguments matching \code{\link{download_wid}()} parameters.
#' @return A \code{wid_query} object (a named list with class \code{"wid_query"}).
#' @examples
#' q <- wid_query(indicators = "sptinc992j", areas = "US", years = 2010:2020)
#' @export
wid_query <- function(...) {
  defaults <- lapply(formals(download_wid), function(x) if (is.symbol(x)) NULL else x)
  args     <- list(...)
  bad      <- setdiff(names(args), names(defaults))
  if (length(bad))
    warning("Unknown parameter(s) ignored: ", paste(bad, collapse = ", "), call. = FALSE)
  structure(modifyList(defaults, args[intersect(names(args), names(defaults))]),
            class = "wid_query")
}

#' Update a wid_query object
#'
#' Modify fields of a \code{wid_query} object, returning an updated query.
#'
#' @param query A \code{wid_query} object.
#' @param ...   Fields to update.
#' @return The updated \code{wid_query} object.
#' @examples
#' q <- wid_query(indicators = "sptinc992j", areas = "US")
#' q <- wid_filter(q, years = 2010:2020)
#' @export
wid_filter <- function(query, ...) {
  stopifnot(inherits(query, "wid_query"))
  for (nm in names(list(...))) query[[nm]] <- list(...)[[nm]]
  query
}

#' Execute a wid_query
#'
#' Fetch data from WID.world using a \code{wid_query} object.
#'
#' @param query A \code{wid_query} object.
#' @param ...   Parameter overrides.
#' @return A \code{wid_df} (a classed \code{data.frame}); see \code{\link{download_wid}}.
#' @examples
#' \donttest{
#'   if (nzchar(Sys.getenv("WID_API_KEY"))) {
#'     q <- wid_query(indicators = "sptinc992j", areas = "US", years = 2010:2020)
#'     wid_fetch(q)
#'   }
#' }
#' @export
wid_fetch <- function(query, ...) {
  stopifnot(inherits(query, "wid_query"))
  args <- modifyList(as.list(query), list(...))
  do.call(download_wid, args[intersect(names(args), names(formals(download_wid)))])
}

#' Print a wid_query object
#'
#' Displays the contents of a \code{wid_query} object.
#'
#' @param x A \code{wid_query} object.
#' @param ... Additional arguments (currently ignored).
#' @return Invisibly returns the input \code{x}.
#' @examples
#' q <- wid_query(indicators = "sptinc992j", areas = "US")
#' print(q)
#' @export
print.wid_query <- function(x, ...) {
  cat("<wid_query>\n")
  for (nm in names(x)) {
    v <- x[[nm]]
    s <- if (is.null(v)) "NULL"
    else if (length(v) > 4L) paste0(paste(head(v, 4L), collapse = ", "), " ...[", length(v), "]")
    else paste(v, collapse = ", ")
    cat(" ", nm, ": ", s, "\n", sep = "")
  }
  invisible(x)
}

# test-download.R
# Tests for download_wid(), wid_query(), wid_filter(), wid_fetch(), and cache.

# ── new_wid_df / print ────────────────────────────────────────────────────────

test_that("new_wid_df attaches correct class", {
  df  <- data.frame(country = "US", variable = "sptinc992j",
                    percentile = "p99p100", year = "2000",
                    value = 0.2, age = "992", pop = "j",
                    stringsAsFactors = FALSE)
  out <- widr:::new_wid_df(df)
  expect_s3_class(out, "wid_df")
  expect_s3_class(out, "data.frame")
})

test_that("print.wid_df emits <wid_df> header", {
  expect_output(print(.wid()), "<wid_df>")
})

test_that("wid_df passes through tidyverse-style subset", {
  d <- .wid(3L)
  expect_s3_class(d[d$year == "2020", ], "data.frame")
})

# ── download_wid argument validation ─────────────────────────────────────────

test_that("download_wid errors when both indicators and areas are 'all'", {
  expect_error(download_wid(), "`indicators` or `areas`")
  expect_error(download_wid(indicators = "all", areas = "all"),
               "`indicators` or `areas`")
})

test_that("download_wid warns on years outside [1800, 2100]", {
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) NULL,
    {
      expect_warning(
        download_wid("sptinc992j", areas = "US", years = 1700L, cache = FALSE),
        "outside"
      )
    }
  )
})

test_that("download_wid warns on malformed area codes", {
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) NULL,
    {
      expect_warning(
        download_wid("sptinc992j", areas = "us", cache = FALSE),
        "invalid"
      )
    }
  )
})

test_that("download_wid normalises ages != 'all'", {
  captured <- list()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(indicators, areas, perc, ages, pop, no_extrapolation, verbose) {
      captured$ages <<- ages
      .wid()
    },
    {
      download_wid("sptinc992j", areas = "US", ages = 992L, cache = FALSE)
      expect_equal(captured$ages, "992")
    }
  )
})

test_that("download_wid normalises pop != 'all'", {
  captured <- list()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(indicators, areas, perc, ages, pop, no_extrapolation, verbose) {
      captured$pop <<- pop
      .wid()
    },
    {
      download_wid("sptinc992j", areas = "US", pop = "j", cache = FALSE)
      expect_equal(captured$pop, "j")
    }
  )
})

# ── download_wid return value ─────────────────────────────────────────────────

test_that("download_wid returns wid_df on cache hit", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) if (a == "get") mock else NULL,
    {
      result <- download_wid("sptinc992j", areas = "US")
      expect_s3_class(result, "wid_df")
    }
  )
})

test_that("download_wid returns wid_df when .fetch returns a wid_df stub", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) mock,
    {
      result <- download_wid("sptinc992j", areas = "US", cache = FALSE)
      expect_s3_class(result, "wid_df")
    }
  )
})

test_that("download_wid returns empty wid_df when .fetch yields NULL", {
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) NULL,
    {
      expect_warning(
        result <- download_wid("sptinc992j", areas = "US", cache = FALSE),
        NA   # .fetch warns internally; download_wid itself does not
      )
      expect_s3_class(result, "wid_df")
      expect_equal(nrow(result), 0L)
    }
  )
})

test_that("download_wid filters years after fetch", {
  rows <- rbind(
    data.frame(country = "US", variable = "sptinc992j", percentile = "p99p100",
               year = "2019", value = 0.20, age = "992", pop = "j",
               stringsAsFactors = FALSE),
    data.frame(country = "US", variable = "sptinc992j", percentile = "p99p100",
               year = "2020", value = 0.21, age = "992", pop = "j",
               stringsAsFactors = FALSE)
  )
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) rows,
    {
      result <- download_wid("sptinc992j", areas = "US",
                             years = 2020L, cache = FALSE)
      expect_equal(nrow(result), 1L)
      expect_equal(result$year, "2020")
    }
  )
})

test_that("download_wid returns wid_df — tidyverse compatible", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) mock,
    {
      result <- download_wid("sptinc992j", areas = "US", cache = FALSE)
      expect_true(is.data.frame(result))
      expect_s3_class(result, "wid_df")
      expect_false(is.list(result) && !is.data.frame(result))
    }
  )
})

# ── verbose messaging ─────────────────────────────────────────────────────────

test_that("download_wid emits verbose message", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) mock,
    {
      expect_message(
        download_wid("sptinc992j", areas = "US", cache = FALSE, verbose = TRUE),
        "Fetching"
      )
    }
  )
})

# ── include_extrapolations flag ───────────────────────────────────────────────

test_that("download_wid passes include_extrapolations=FALSE as no_extrapolation=TRUE", {
  captured <- list()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(indicators, areas, perc, ages, pop, no_extrapolation, verbose) {
      captured$no_extrapolation <<- no_extrapolation
      .wid()
    },
    {
      download_wid("sptinc992j", areas = "US",
                   include_extrapolations = FALSE, cache = FALSE)
      expect_true(captured$no_extrapolation)
    }
  )
})

# ── caching ───────────────────────────────────────────────────────────────────

test_that("download_wid stores result in cache after fetch", {
  mock  <- .wid()
  saved <- list()
  testthat::with_mocked_bindings(
    `.cache` = function(a, key = NULL, value = NULL) {
      if (a == "get") return(NULL)
      if (a == "set") { saved$value <<- value; return(invisible(value)) }
    },
    `.fetch` = function(...) mock,
    {
      download_wid("sptinc992j", areas = "US", cache = TRUE)
      expect_s3_class(saved$value, "wid_df")
    }
  )
})

# ── metadata attribute ────────────────────────────────────────────────────────

test_that("download_wid metadata=TRUE attaches wid_meta attribute", {
  mock    <- .wid()
  meta_df <- data.frame(variable = "sptinc992j", country = "US",
                        source = "Tax records", stringsAsFactors = FALSE)
  testthat::with_mocked_bindings(
    `.cache`               = function(a, ...) NULL,
    `.fetch`               = function(...) mock,
    `.get_variables_areas` = function(...) data.frame(
      variable = "sptinc", country = "US", percentile = "p99p100",
      age = "992", pop = "j", stringsAsFactors = FALSE),
    `.fetch_meta`          = function(...) meta_df,
    {
      result <- download_wid("sptinc992j", areas = "US",
                             metadata = TRUE, cache = FALSE)
      expect_s3_class(result, "wid_df")
      expect_false(is.null(attr(result, "wid_meta")))
    }
  )
})

test_that("download_wid metadata=TRUE with empty catalogue sets NULL wid_meta", {
  testthat::with_mocked_bindings(
    `.cache`               = function(a, ...) NULL,
    `.fetch`               = function(...) NULL,
    `.get_variables_areas` = function(...) NULL,
    {
      suppressWarnings(
        result <- download_wid("sptinc992j", areas = "US",
                               metadata = TRUE, cache = FALSE)
      )
      expect_s3_class(result, "wid_df")
      expect_null(attr(result, "wid_meta"))
    }
  )
})

# ── .fetch internals ──────────────────────────────────────────────────────────

test_that(".fetch warns when .get_variables_areas returns empty", {
  testthat::with_mocked_bindings(
    `.get_variables_areas` = function(...) NULL,
    {
      expect_warning(
        widr:::.fetch("sptinc992j", "US", "all", "992", "j", FALSE, FALSE),
        "No data found"
      )
    }
  )
})

test_that(".fetch warns when .filter_vars removes all rows", {
  # Variables are returned from catalogue but none match the requested percentile.
  vars <- data.frame(
    variable = "sptinc992j", country = "US", percentile = "p99p100",
    age = "992", pop = "j", stringsAsFactors = FALSE
  )
  testthat::with_mocked_bindings(
    `.get_variables_areas` = function(...) vars,
    `.filter_vars`         = function(...) vars[0L, ],   # empty after filter
    {
      expect_warning(
        widr:::.fetch("sptinc992j", "US", "p0p50", "992", "j", FALSE, FALSE),
        "No data after"
      )
    }
  )
})

test_that(".fetch warns when .chunk_fetch returns no rows", {
  vars <- data.frame(
    variable = "sptinc992j", country = "US", percentile = "p99p100",
    age = "992", pop = "j", stringsAsFactors = FALSE
  )
  testthat::with_mocked_bindings(
    `.get_variables_areas` = function(...) vars,
    `.chunk_fetch`         = function(...) NULL,
    {
      expect_warning(
        widr:::.fetch("sptinc992j", "US", "all", "992", "j", FALSE, FALSE),
        "API returned no data"
      )
    }
  )
})

test_that(".fetch verbose=TRUE emits catalogue and variable count messages", {
  vars <- data.frame(
    variable = "sptinc992j", country = "US", percentile = "p99p100",
    age = "992", pop = "j", api_code = "sptinc992j_p99p100_992_j",
    stringsAsFactors = FALSE
  )
  testthat::with_mocked_bindings(
    `.get_variables_areas` = function(...) vars,
    `.chunk_fetch`         = function(...) data.frame(
      indicator = "sptinc992j_p99p100_992_j", country = "US",
      year = "2000", value = 0.2, stringsAsFactors = FALSE),
    `.parse_indicator`     = function(df) {
      df$variable   <- "sptinc992j"
      df$percentile <- "p99p100"
      df$indicator  <- NULL
      df
    },
    {
      msgs <- character(0L)
      withCallingHandlers(
        widr:::.fetch("sptinc992j", "US", "all", "992", "j", FALSE, TRUE),
        message = function(m) {
          msgs <<- c(msgs, conditionMessage(m))
          invokeRestart("muffleMessage")
        }
      )
      expect_true(any(grepl("catalogue|variable", msgs, ignore.case = TRUE)))
    }
  )
})

# ── wid_query / wid_filter / wid_fetch ───────────────────────────────────────

test_that("wid_query defaults match download_wid formals", {
  q <- wid_query()
  expect_equal(q$areas,  "all")
  expect_equal(q$years,  "all")
  expect_equal(q$ages,   "992")
  expect_true(q$cache)
  expect_false(q$verbose)
})

test_that("wid_query sets parameters", {
  q <- wid_query(indicators = "sptinc992j", areas = "US")
  expect_equal(q$indicators, "sptinc992j")
  expect_equal(q$areas,      "US")
})

test_that("wid_query warns on unknown params", {
  expect_warning(wid_query(foo = "bar"), "Unknown parameter")
})

test_that("wid_filter updates fields and preserves others", {
  q  <- wid_query(areas = "US", years = 2000:2010)
  q2 <- wid_filter(q, areas = c("US", "FR"))
  expect_equal(q2$areas, c("US", "FR"))
  expect_equal(q2$years, 2000:2010)
})

test_that("wid_filter errors on non-query", {
  expect_error(wid_filter(list()), "inherits")
})

test_that("wid_fetch executes query and returns wid_df", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) mock,
    {
      q <- wid_query(indicators = "sptinc992j", areas = "US", cache = FALSE)
      expect_s3_class(wid_fetch(q), "wid_df")
    }
  )
})

test_that("wid_fetch applies overrides", {
  mock <- .wid()
  testthat::with_mocked_bindings(
    `.cache` = function(a, ...) NULL,
    `.fetch` = function(...) mock,
    {
      q <- wid_query(indicators = "sptinc992j", cache = FALSE)
      expect_s3_class(wid_fetch(q, areas = "FR"), "wid_df")
    }
  )
})

test_that("print.wid_query outputs header", {
  expect_output(print(wid_query(areas = "US")), "<wid_query>")
})

test_that("print.wid_query truncates long vectors", {
  expect_output(print(wid_query(years = 1900:2100)), "\\.\\.\\.")
})

# ── cache (wid_cache / wid_cache_list / wid_cache_clear) ─────────────────────

test_that("wid_cache round-trips data", {
  tmp <- tempfile()
  on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  key <- widr:::.cache_key("test")
  wid_cache("set", key, 42L)
  expect_equal(wid_cache("get", key), 42L)
})

test_that("wid_cache returns NULL for missing key", {
  tmp <- tempfile()
  on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  expect_null(wid_cache("get", "nosuchkey"))
})

test_that("wid_cache_clear removes files", {
  tmp <- tempfile()
  on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  wid_cache("set", "k1", 1L)
  wid_cache("set", "k2", 2L)
  n <- wid_cache_clear()
  expect_equal(n, 2L)
  expect_length(wid_cache_list(), 0L)
})

test_that("wid_cache errors on unknown action", {
  expect_error(wid_cache("bad"), "Unknown action")
})

# Inequality measures: Gini, top share, percentile ratio

.parse_perc <- function(p) {
  m <- regmatches(p, regexec("^p([0-9.]+)p([0-9.]+)$", p))[[1L]]
  if (!length(m)) return(NULL)
  list(lo = as.numeric(m[2L]) / 100, hi = as.numeric(m[3L]) / 100)
}

.filter_data <- function(data, variable, country) {
  variable <- .resolve_var(data, variable)
  d <- data[data$variable == variable, ]
  if (!is.null(country)) d <- d[d$country %in% country, ]
  if (!nrow(d)) stop("No data after filtering.", call. = FALSE)
  d
}

#' Compute Gini coefficient from a share series
#'
#' Requires an \code{"s"} (share) series with \code{pXpY} percentile codes
#' covering the full distribution.
#'
#' @param data A \code{wid_df} with share series.
#' @param variable Variable code filter.
#' @param country Country code filter.
#' @return A \code{data.frame} with columns \code{country}, \code{year},
#'   and \code{gini} (numeric, 0-1).
#' @examples
#' d <- data.frame(
#'   country = rep("US", 3L), year = rep("2020", 3L),
#'   variable = rep("sptinc992j", 3L),
#'   percentile = c("p0p50", "p50p90", "p90p100"),
#'   value = c(0.20, 0.40, 0.40), age = "992", pop = "j",
#'   stringsAsFactors = FALSE
#' )
#' class(d) <- c("wid_df", "data.frame")
#' wid_gini(d)
#' @export
wid_gini <- function(data, variable = NULL, country = NULL) {
  .check(data, series_type = "s")
  d      <- .filter_data(data, variable, country)
  bounds <- lapply(d$percentile, .parse_perc)
  ok     <- !vapply(bounds, is.null, logical(1L))
  if (any(!ok))
    warning(sum(!ok), " row(s) dropped: unparseable percentile codes.", call. = FALSE)
  d <- d[ok, ]; bounds <- bounds[ok]
  d$lo <- vapply(bounds, `[[`, numeric(1L), "lo")
  d$hi <- vapply(bounds, `[[`, numeric(1L), "hi")
  do.call(rbind, lapply(split(d, list(d$country, d$year), drop = TRUE), function(g) {
    g <- g[order(g$lo), ]
    B <- sum(cumsum(g$value - g$value / 2) * (g$hi - g$lo))
    data.frame(country = g$country[1L], year = g$year[1L], gini = 1 - 2 * B,
               stringsAsFactors = FALSE)
  }))
}

#' Top income or wealth share
#'
#' @param data A \code{wid_df} with share series.
#' @param top Top fraction, e.g. 0.1 = top 10\\%.
#' @param variable Variable code filter.
#' @param country Country code filter.
#' @return A \code{data.frame} with columns \code{country}, \code{year},
#'   \code{top} (the requested fractile), and \code{share} (numeric).
#' @examples
#' d <- data.frame(
#'   country = "US", year = "2020", variable = "sptinc992j",
#'   percentile = "p99p100", value = 0.19, age = "992", pop = "j",
#'   stringsAsFactors = FALSE
#' )
#' class(d) <- c("wid_df", "data.frame")
#' wid_top_share(d, top = 0.01)
#' @export
wid_top_share <- function(data, top = 0.1, variable = NULL, country = NULL) {
  .check(data, series_type = "s")
  d      <- .filter_data(data, variable, country)
  target <- sprintf("p%gp100", (1 - top) * 100)
  out    <- d[d$percentile == target, c("country", "year", "value")]
  if (!nrow(out))
    stop("No percentile '", target, "' found. Ensure perc includes '", target, "'.",
         call. = FALSE)
  out$top <- top
  names(out)[3L] <- "share"
  out[c("country", "year", "top", "share")]
}

#' Percentile threshold ratio (e.g. P90/P10)
#'
#' @param data A \code{wid_df} with threshold (\code{"t"}) series.
#' @param numerator Upper percentile code, e.g. \code{"p90"}.
#' @param denominator Lower percentile code, e.g. \code{"p10"}.
#' @param variable Variable code filter.
#' @param country Country code filter.
#' @return A \code{data.frame} with columns \code{country}, \code{year},
#'   and \code{ratio} (numeric: numerator threshold divided by denominator threshold).
#' @examples
#' d <- data.frame(
#'   country = rep("US", 2L), year = rep("2020", 2L),
#'   variable = rep("tptinc992j", 2L),
#'   percentile = c("p90", "p10"), value = c(200000, 10000),
#'   age = "992", pop = "j", stringsAsFactors = FALSE
#' )
#' class(d) <- c("wid_df", "data.frame")
#' wid_percentile_ratio(d)
#' @export
wid_percentile_ratio <- function(data, numerator = "p90", denominator = "p10",
                                 variable = NULL, country = NULL) {
  .check(data, series_type = "t")
  d   <- .filter_data(data, variable, country)
  num <- d[d$percentile == numerator,   c("country", "year", "value")]
  den <- d[d$percentile == denominator, c("country", "year", "value")]
  m   <- merge(num, den, by = c("country", "year"), suffixes = c(".n", ".d"))
  if (!nrow(m))
    stop("No matching country/year for ", numerator, "/", denominator, ".",
         call. = FALSE)
  data.frame(country = m$country, year = m$year,
             ratio = m$value.n / m$value.d, stringsAsFactors = FALSE)
}

# ---- .parse_perc -----------------------------------------------------------

test_that(".parse_perc parses standard intervals", {
  b <- widr:::.parse_perc("p90p100")
  expect_equal(b$lo, 0.9)
  expect_equal(b$hi, 1.0)
})

test_that(".parse_perc parses decimal percentiles", {
  b <- widr:::.parse_perc("p99.9p100")
  expect_equal(b$lo, 0.999)
  expect_equal(b$hi, 1.0)
})

test_that(".parse_perc returns NULL on bad input", {
  expect_null(widr:::.parse_perc("bad"))
  expect_null(widr:::.parse_perc(""))
})

# ---- .filter_data ----------------------------------------------------------

test_that(".filter_data restricts to requested country", {
  d <- .wid(countries = c("US","FR"))
  f <- widr:::.filter_data(d, NULL, "US")
  expect_true(all(f$country == "US"))
})

test_that(".filter_data errors when no rows remain", {
  expect_error(widr:::.filter_data(.wid(), NULL, "XX"), "No data after filtering")
})

# ---- wid_gini --------------------------------------------------------------

test_that("wid_gini errors on non-wid_df", {
  expect_error(wid_gini(data.frame()), "wid_df")
})

test_that("wid_gini errors on empty wid_df", {
  expect_error(wid_gini(widr:::.wid_df()), "empty")
})

test_that("wid_gini errors on non-share series", {
  expect_error(wid_gini(.wid(type = "t")), "share")
})

test_that("wid_gini returns gini in [0,1]", {
  g <- wid_gini(.wid_dist())
  expect_true(all(g$gini >= 0 & g$gini <= 1))
  expect_named(g, c("country","year","gini"))
})

test_that("wid_gini warns and drops rows with unparseable percentile codes", {
  d              <- .wid_dist()
  d$percentile[1L] <- "bad"
  expect_warning(out <- wid_gini(d), "dropped")
  expect_true(nrow(out) >= 1L)
})

test_that("wid_gini filters by country", {
  d <- rbind(.wid_dist("US"), .wid_dist("FR"))
  g <- wid_gini(d, country = "US")
  expect_true(all(g$country == "US"))
})

test_that("wid_gini produces one row per country-year", {
  d <- rbind(.wid_dist("US", 2019L), .wid_dist("US", 2020L),
             .wid_dist("FR", 2020L))
  g <- wid_gini(d)
  expect_equal(nrow(g), 3L)
})

test_that("wid_gini returns 0 for perfectly equal distribution", {
  d <- .wid_dist()
  d$value <- rep(1/3, 3L)
  d$percentile <- c("p0p33", "p33p66", "p66p100")
  # All shares equal -> Gini should be near 0
  g <- suppressWarnings(wid_gini(d))
  expect_true(g$gini >= 0)
})

# ---- wid_top_share ---------------------------------------------------------

test_that("wid_top_share errors on non-share series", {
  expect_error(wid_top_share(.wid(type = "t")), "share")
})

test_that("wid_top_share errors when target percentile absent", {
  expect_error(wid_top_share(.wid_dist(), top = 0.001), "No percentile")
})

test_that("wid_top_share returns correct structure for top 10%", {
  d   <- .wid_dist()
  out <- wid_top_share(d, top = 0.1)
  expect_named(out, c("country","year","top","share"))
  expect_equal(out$top,   0.1)
  expect_equal(out$share, 0.4)
})

test_that("wid_top_share returns one row per country-year", {
  d <- rbind(.wid_dist("US", 2019L), .wid_dist("US", 2020L))
  expect_equal(nrow(wid_top_share(d, top = 0.1)), 2L)
})

# ---- wid_percentile_ratio --------------------------------------------------

test_that("wid_percentile_ratio errors on non-threshold series", {
  expect_error(wid_percentile_ratio(.wid(type = "s")), "threshold")
})

test_that("wid_percentile_ratio computes correct ratio", {
  r <- wid_percentile_ratio(.wid_t())   # p90 / p10 = 200000 / 10000 = 20
  expect_equal(r$ratio, 20, tolerance = 1e-9)
  expect_named(r, c("country","year","ratio"))
})

test_that("wid_percentile_ratio errors when percentiles not found", {
  d <- .wid_t(); d$percentile <- rep("p50", 3L)
  expect_error(wid_percentile_ratio(d), "No matching")
})

test_that("wid_percentile_ratio respects custom numerator/denominator", {
  r <- wid_percentile_ratio(.wid_t(), numerator = "p90", denominator = "p50")
  expect_equal(r$ratio, 200000 / 50000, tolerance = 1e-9)
})

test_that("wid_percentile_ratio returns one row per country-year", {
  d1 <- .wid_t(); d1$year <- 2020L
  d2 <- .wid_t(); d2$year <- 2021L
  d  <- rbind(d1, d2)
  expect_equal(nrow(wid_percentile_ratio(d)), 2L)
})

#' @importFrom httr2 request req_url_query req_headers req_timeout req_retry
#'   req_perform resp_body_string resp_check_status
#' @importFrom base64enc base64encode
#' @importFrom jsonlite fromJSON
#' @importFrom digest digest
#' @importFrom tools R_user_dir
#' @importFrom utils head modifyList
#' @importFrom stats setNames
#' @importFrom ggplot2 ggplot aes geom_line geom_col geom_point geom_abline
#'   scale_color_manual scale_fill_manual scale_x_continuous scale_y_continuous
#'   facet_wrap labs theme element_rect element_text element_line element_blank
#' @importFrom scales percent_format
#' @docType package
#' @name widr-package
#' @aliases widr
"_PACKAGE"

utils::globalVariables(c("hi", "cum_share", "country", "year", "value"))

.wid_env <- new.env(parent = emptyenv())
`%||%`   <- function(a, b) if (!is.null(a)) a else b

.wid_df <- function() structure(
  data.frame(country = character(), variable = character(), percentile = character(),
             year = character(), value = numeric(), age = character(), pop = character(),
             stringsAsFactors = FALSE),
  class = c("wid_df", "data.frame"))

# ---- API --------------------------------------------------------------------

.API <- "https://rfap9nitz6.execute-api.eu-west-1.amazonaws.com/prod"

#' Set WID API key
#' @param key character API key
#' @return Invisibly, the key string.
#' @export
wid_set_key <- function(key) { assign("key", key, envir = .wid_env); invisible(key) }

#' @noRd
.get_key <- function() {
  k <- tryCatch(get("key", envir = .wid_env), error = function(e) NULL) %||%
    Sys.getenv("WID_API_KEY", unset = NA_character_)
  if (is.na(k))
    stop("No WID API key. Use wid_set_key() or set WID_API_KEY env var.", call. = FALSE)
  base64encode(charToRaw(k))
}

#' @noRd
.wid_get <- function(endpoint, params = list()) {
  resp <- request(paste0(.API, endpoint)) |>
    req_url_query(!!!params) |>
    req_headers(`x-api-key` = .get_key(), `User-Agent` = "widr/1.0") |>
    req_timeout(60L) |>
    req_retry(max_tries = 3L) |>
    req_perform()
  resp_check_status(resp)
  fromJSON(resp_body_string(resp, "UTF-8"), simplifyVector = FALSE)
}

# ---- Cache ------------------------------------------------------------------

.CACHE_DIR  <- tools::R_user_dir("widr", "cache")
#' @noRd
.cache_key  <- function(...) digest::digest(list(...), algo = "xxhash64")
#' @noRd
.cache_path <- function(key) file.path(.CACHE_DIR, paste0(key, ".rds"))

#' @noRd
.cache <- function(action, key = NULL, value = NULL) {
  switch(action,
         get   = { p <- .cache_path(key); if (file.exists(p)) readRDS(p) else NULL },
         set   = { dir.create(.CACHE_DIR, recursive = TRUE, showWarnings = FALSE)
           saveRDS(value, .cache_path(key)); invisible(value) },
         clear = { f <- list.files(.CACHE_DIR, "\\.rds$", full.names = TRUE)
         file.remove(f); invisible(length(f)) },
         list  = list.files(.CACHE_DIR, "\\.rds$"),
         stop("Unknown action: ", action, call. = FALSE)
  )
}

#' Interact with the WID response cache
#'
#' Low-level access to get, set, list, or clear cached API responses.
#' @param action One of \code{"get"}, \code{"set"}, \code{"list"}, \code{"clear"}.
#' @param key    Cache key string (required for \code{"get"} and \code{"set"}).
#' @param value  Value to store (\code{"set"} only).
#' @return For \code{"get"}: the cached object or \code{NULL}. For
#'   \code{"list"}: character vector of cache keys. For \code{"set"} and
#'   \code{"clear"}: invisibly, the stored value or file count removed.
#' @examples
#' wid_cache("list")
#' @export
wid_cache <- function(action, key = NULL, value = NULL)
  .cache(action, key = key, value = value)

#' List cached WID API responses
#' @return Character vector of cache file names.
#' @examples
#' wid_cache_list()
#' @export
wid_cache_list <- function() .cache("list")

#' Clear all cached WID API responses
#' @return Invisibly, the number of files removed.
#' @examples
#' wid_cache_clear()
#' @export
wid_cache_clear <- function() .cache("clear")

# ---- Internal helpers -------------------------------------------------------

.CODE_TYPES <- c(
  a = "average",           b = "inverted Pareto-Lorenz coefficient",
  e = "total emissions",   f = "female population",
  g = "Gini coefficient",  i = "index",
  k = "per-capita emissions", l = "avg per-capita group emissions",
  m = "total",             n = "population",
  p = "proportion women",  r = "top-10/bottom-50 ratio",
  s = "share",             t = "threshold",
  w = "wealth/income ratio", x = "exchange rate",
  y = "wealth/GDP ratio"
)

#' @noRd
.recode <- function(x, lookup) { m <- match(x, names(lookup)); ifelse(is.na(m), x, lookup[m]) }

#' @noRd
.check <- function(data, series_type = NULL) {
  if (!inherits(data, "wid_df"))
    stop("'data' must be a wid_df returned by download().", call. = FALSE)
  if (nrow(data) == 0L)
    stop("'data' is empty.", call. = FALSE)
  if (!is.null(series_type)) {
    found <- unique(substr(data$variable, 1L, 1L))
    if (!series_type %in% found)
      stop(sprintf("Expected series type '%s' (%s); data has type(s): %s",
                   series_type,
                   switch(series_type, s = "share", t = "threshold", series_type),
                   paste(found, collapse = ", ")), call. = FALSE)
  }
  invisible(data)
}

#' @noRd
.resolve_var <- function(data, variable) {
  if (!is.null(variable)) {
    if (!variable %in% data$variable)
      stop("Variable '", variable, "' not found in data.", call. = FALSE)
    return(variable)
  }
  vars <- unique(data$variable)
  if (length(vars) > 1L)
    stop("Multiple variables in data; pass variable= to select one.", call. = FALSE)
  vars[1L]
}

#' @noRd
.empty <- function(...) {
  nms <- unlist(list(...))
  as.data.frame(setNames(replicate(length(nms), character(0L), simplify = FALSE), nms),
                stringsAsFactors = FALSE)
}

#' @noRd
.recode_imp <- function(x) {
  if (is.null(x) || is.na(x)) return(NA_character_)
  switch(x, region = "regional imputation", survey = "adjusted surveys",
         tax = "surveys and tax data", full = "surveys and tax microdata",
         rescaling = "rescaled fiscal income", x)
}

#' @noRd
.strip_extrap <- function(df, meta) {
  b <- meta$extrapolation
  if (is.null(b) || is.na(b) || !nzchar(b)) return(df)
  b  <- fromJSON(b)
  dp <- meta$data_points
  dp <- if (!is.null(dp) && !is.na(dp) && nzchar(dp)) fromJSON(dp) else character(0L)
  ex <- unlist(lapply(seq_len(nrow(b)), function(i)
    as.character(seq(as.integer(b[i, 1L]) + 1L, as.integer(b[i, 2L])))))
  df[!(df$year %in% setdiff(ex, dp)), ]
}

#' @noRd
.filter_vars <- function(v, perc, ages, pop) {
  if (!identical(perc, "all")) v <- v[v$percentile %in% perc, ]
  if (!identical(ages, "all")) v <- v[v$age        %in% ages, ]
  if (!identical(pop,  "all")) v <- v[v$pop        %in% pop,  ]
  v
}

#' @noRd
.parse_indicator <- function(df) {
  parts <- strsplit(df$indicator, "_", fixed = TRUE)
  if (any(lengths(parts) < 4L))
    stop("Unexpected indicator format; expected '<code>_<perc>_<age>_<pop>'.", call. = FALSE)
  df$variable   <- vapply(parts, function(p) paste0(p[1L], p[3L], p[4L]), character(1L))
  df$percentile <- vapply(parts, `[`, character(1L), 2L)
  df$indicator  <- NULL
  df
}

# ---- Low-level API fetchers -------------------------------------------------

#' @noRd
.get_variables_areas <- function(indicators, areas) {
  json <- .wid_get("/countries-available-variables",
                   list(countries = paste(areas,      collapse = ","),
                        variables = paste(indicators, collapse = ",")))
  if (length(json) == 1L && is.null(names(json))) json <- json[[1L]]
  rows <- do.call(rbind, lapply(names(json), function(variable)
    do.call(rbind, lapply(names(json[[variable]]), function(country)
      do.call(rbind, lapply(json[[variable]][[country]], function(item)
        data.frame(variable = variable, country = country,
                   percentile = item[[1L]], age = item[[2L]], pop = item[[3L]],
                   stringsAsFactors = FALSE)))))))
  if (is.null(rows)) .empty("variable", "country", "percentile", "age", "pop") else rows
}

#' @noRd
.get_data_variables <- function(areas, variables, no_extrapolation = FALSE) {
  json <- .wid_get("/countries-variables",
                   list(countries = paste(areas,     collapse = ","),
                        variables = paste(variables, collapse = ","),
                        years     = "all"))
  rows <- do.call(rbind, lapply(names(json), function(variable)
    do.call(rbind, lapply(json[[variable]], function(json_country) {
      country <- names(json_country)[1L]
      cd      <- json_country[[1L]]
      df <- data.frame(
        indicator = variable, country = country,
        year  = vapply(cd$values, function(v) as.character(v[[1L]]), character(1L)),
        value = vapply(cd$values, function(v) as.numeric(v[[2L]]),   numeric(1L)),
        stringsAsFactors = FALSE)
      if (no_extrapolation) .strip_extrap(df, cd$meta) else df
    }))))
  if (is.null(rows)) .empty("indicator", "country", "year", "value") else rows
}

#' @noRd
.get_metadata_variables <- function(areas, variables) {
  json  <- .wid_get("/countries-variables-metadata",
                    list(countries = paste(areas,     collapse = ","),
                         variables = paste(variables, collapse = ",")))
  jmeta <- json[[1L]]$metadata_func
  rows <- do.call(rbind, lapply(jmeta, function(jvar) {
    vname <- names(jvar); jv <- jvar[[vname]]
    jname  <- jv[[1L]][[1L]]; jpop <- jv[[3L]][[1L]]
    jage   <- jv[[4L]][[1L]]; junits <- jv[[5L]][[1L]]
    notes  <- setNames(jv[[6L]][[1L]][[1L]],
                       vapply(jv[[6L]][[1L]][[1L]], `[[`, character(1L), "alpha2"))
    do.call(rbind, lapply(junits, function(mc) {
      n   <- notes[[mc$country]]
      chr <- function(x) if (is.null(x)) NA_character_ else as.character(x)[[1L]]
      data.frame(
        variable    = vname,
        country     = mc$country,
        countryname = mc$country_name,
        shortname   = jname$shortname,
        shortdes    = jname$simpledes,
        pop         = jpop$longdes,
        age         = if (is.list(jage)) jage$fullname else jage[["fullname"]],
        source      = chr(if (is.null(n)) NULL else n$source),
        method      = chr(if (is.null(n)) NULL else n$method),
        quality     = chr(if (is.null(n)) NULL else n$data_quality),
        imputation  = if (is.null(n)) NA_character_ else .recode_imp(n$imputation),
        stringsAsFactors = FALSE)
    }))
  }))
  if (is.null(rows))
    .empty("variable","country","countryname","shortname","shortdes",
           "pop","age","source","method","quality","imputation")
  else rows
}

#' @noRd
.chunk_fetch <- function(variables, size, fetch_fn) {
  variables$chunk <- floor((as.integer(factor(
    do.call(paste, c(variables[, c("variable","percentile","age","pop")],
                     list(sep = "_"))))) - 1L) / size)
  do.call(rbind, lapply(split(variables, variables$chunk), function(ch)
    fetch_fn(unique(ch$country), unique(ch$api_code))))
}

#' @noRd
.auto_title <- function(v, countries) {
  lbl <- tryCatch({
    d   <- wid_decode(v, strict = FALSE)
    con <- if (!is.null(d)) wid_concepts$description[wid_concepts$code == d$concept] else character(0L)
    if (length(con)) tools::toTitleCase(sub("^\\(.*?\\)\\s*", "", con[1L])) else v
  }, error = function(e) v, warning = function(w) v)
  paste0(lbl, if (length(countries) <= 3L) paste0(" -- ", paste(countries, collapse = ", ")))
}

#' @noRd
.common_year <- function(d) {
  common <- Reduce(intersect, split(d$year, d$country))
  if (!length(common)) stop("No common year across countries.", call. = FALSE)
  max(common)
}

# ---- .wid_df ---------------------------------------------------------------

test_that(".wid_df returns empty wid_df with correct columns", {
  d <- widr:::.wid_df()
  expect_s3_class(d, "wid_df")
  expect_equal(nrow(d), 0L)
  expect_named(d, c("country","variable","percentile","year","value","age","pop"))
})

# ---- .get_key --------------------------------------------------------------

test_that(".get_key errors with no key set and no env var", {
  withr::with_envvar(list(WID_API_KEY = NA_character_), {
    if (exists("key", envir = widr:::.wid_env)) rm("key", envir = widr:::.wid_env)
    expect_error(widr:::.get_key(), "No WID API key")
  })
})

test_that(".get_key reads WID_API_KEY env var and base64-encodes it", {
  withr::with_envvar(list(WID_API_KEY = "envkey"), {
    if (exists("key", envir = widr:::.wid_env)) rm("key", envir = widr:::.wid_env)
    expect_equal(rawToChar(base64enc::base64decode(widr:::.get_key())), "envkey")
  })
})

test_that("wid_set_key stores key invisibly", {
  on.exit(rm(list = "key", envir = widr:::.wid_env), add = TRUE)
  expect_invisible(wid_set_key("testkey"))
  expect_equal(get("key", envir = widr:::.wid_env), "testkey")
})

# ---- .wid_get --------------------------------------------------------------

test_that(".wid_get calls request pipeline and parses JSON", {
  with_mocked_bindings(
    request           = function(url) structure(list(), class = "httr2_request"),
    req_url_query     = function(r, ...) r,
    req_headers       = function(r, ...) r,
    req_timeout       = function(r, ...) r,
    req_retry         = function(r, ...) r,
    req_perform       = function(r) structure(list(), class = "httr2_response"),
    resp_check_status = function(r) invisible(r),
    resp_body_string  = function(r, ...) '{"foo":"bar"}',
    .get_key          = function() "dGVzdA==",
    { expect_equal(widr:::.wid_get("/test")$foo, "bar") },
    .package = "widr"
  )
})

# ---- .cache ----------------------------------------------------------------

test_that(".cache set/get round-trips a value", {
  tmp <- tempfile(); on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  k <- widr:::.cache_key("x")
  widr:::.cache("set", k, 42L)
  expect_equal(widr:::.cache("get", k), 42L)
})

test_that(".cache get returns NULL for missing key", {
  tmp <- tempfile(); on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  expect_null(widr:::.cache("get", "missing"))
})

test_that(".cache clear removes files and returns count", {
  tmp <- tempfile(); on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  widr:::.cache("set", "k1", 1L); widr:::.cache("set", "k2", 2L)
  expect_equal(widr:::.cache("clear"), 2L)
  expect_length(widr:::.cache("list"), 0L)
})

test_that(".cache list returns rds filenames", {
  tmp <- tempfile(); on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  widr:::.cache("set", "k", 1L)
  expect_length(widr:::.cache("list"), 1L)
})

test_that(".cache errors on unknown action", {
  expect_error(widr:::.cache("bad"), "Unknown action")
})

test_that("wid_cache_list and wid_cache_clear delegate to .cache", {
  tmp <- tempfile(); on.exit(unlink(tmp, recursive = TRUE))
  assignInNamespace(".CACHE_DIR", tmp, "widr")
  widr:::.cache("set", "x", 1L)
  expect_length(wid_cache_list(), 1L)
  expect_equal(wid_cache_clear(), 1L)
})

# ---- .recode ---------------------------------------------------------------

test_that(".recode maps known values and passes through unknowns", {
  lkp <- c(US = "United States", FR = "France")
  expect_equal(widr:::.recode(c("US","XX"), lkp), c("United States","XX"))
})

# ---- .check ----------------------------------------------------------------

test_that(".check errors on non-wid_df, empty df, and type mismatch", {
  expect_error(widr:::.check(data.frame()), "wid_df")
  expect_error(widr:::.check(widr:::.wid_df()), "empty")
  expect_error(widr:::.check(.wid(type = "s"), series_type = "t"), "threshold")
})

test_that(".check passes invisibly on valid data and matching type", {
  expect_invisible(widr:::.check(.wid()))
  expect_invisible(widr:::.check(.wid(type = "t"), series_type = "t"))
})

# ---- .resolve_var ----------------------------------------------------------

test_that(".resolve_var handles all four paths", {
  d <- .wid()
  v <- unique(d$variable)
  expect_equal(widr:::.resolve_var(d, v), v)
  expect_error(widr:::.resolve_var(d, "xyz"), "not found")
  expect_equal(widr:::.resolve_var(d, NULL), v)
  d2 <- d; d2$variable[1L] <- "aptinc992j"
  expect_error(widr:::.resolve_var(d2, NULL), "Multiple variables")
})

# ---- .empty ----------------------------------------------------------------

test_that(".empty produces zero-row data.frame with named columns", {
  df <- widr:::.empty("a","b")
  expect_equal(nrow(df), 0L)
  expect_named(df, c("a","b"))
})

# ---- .recode_imp -----------------------------------------------------------

test_that(".recode_imp maps all known codes, unknowns, and NA/NULL", {
  expect_equal(widr:::.recode_imp("region"),    "regional imputation")
  expect_equal(widr:::.recode_imp("survey"),    "adjusted surveys")
  expect_equal(widr:::.recode_imp("tax"),       "surveys and tax data")
  expect_equal(widr:::.recode_imp("full"),      "surveys and tax microdata")
  expect_equal(widr:::.recode_imp("rescaling"), "rescaled fiscal income")
  expect_equal(widr:::.recode_imp("other"),     "other")
  expect_identical(widr:::.recode_imp(NULL), NA_character_)
  expect_identical(widr:::.recode_imp(NA),   NA_character_)
})

# ---- .strip_extrap ---------------------------------------------------------

test_that(".strip_extrap returns df unchanged when extrapolation absent", {
  df <- data.frame(year = c("2000","2001"), value = 1:2, stringsAsFactors = FALSE)
  expect_equal(widr:::.strip_extrap(df, list(extrapolation = NULL)), df)
  expect_equal(widr:::.strip_extrap(df, list(extrapolation = NA)),   df)
  expect_equal(widr:::.strip_extrap(df, list(extrapolation = "")),   df)
})

test_that(".strip_extrap removes extrapolated years and retains data_points", {
  df     <- data.frame(year = as.character(2000:2004), value = 1:5, stringsAsFactors = FALSE)
  extrap <- jsonlite::toJSON(matrix(c(2001L, 2004L), nrow = 1L))
  # Without data_points: years 2002-2004 dropped (extrap range is 2002..2004)
  out1 <- widr:::.strip_extrap(df, list(extrapolation = extrap, data_points = NULL))
  expect_equal(out1$year, c("2000","2001"))
  # With data_point 2002: retained
  dp   <- jsonlite::toJSON("2002")
  out2 <- widr:::.strip_extrap(df, list(extrapolation = extrap, data_points = dp))
  expect_true("2002" %in% out2$year)
  expect_false("2003" %in% out2$year)
})

# ---- .filter_vars ----------------------------------------------------------

test_that(".filter_vars filters by perc, ages, pop; 'all' skips filter", {
  v <- data.frame(percentile = c("p0p50","p50p100","p0p50"),
                  age = c("992","992","999"), pop = c("j","j","i"),
                  stringsAsFactors = FALSE)
  expect_equal(nrow(widr:::.filter_vars(v, "all", "all", "all")), 3L)
  expect_equal(nrow(widr:::.filter_vars(v, "p0p50", "all", "all")), 2L)
  expect_equal(nrow(widr:::.filter_vars(v, "all", "999", "all")), 1L)
  expect_equal(nrow(widr:::.filter_vars(v, "all", "all", "i")), 1L)
})

# ---- .parse_indicator ------------------------------------------------------

test_that(".parse_indicator builds canonical variable and extracts percentile", {
  df  <- data.frame(indicator = "sptinc_p99p100_992_j", country = "US",
                    year = "2020", value = 0.2, stringsAsFactors = FALSE)
  out <- widr:::.parse_indicator(df)
  expect_equal(out$variable, "sptinc992j")
  expect_equal(out$percentile, "p99p100")
  expect_null(out$indicator)
})

test_that(".parse_indicator errors on malformed indicator", {
  df <- data.frame(indicator = "bad", country = "US", year = "2020", value = 0,
                   stringsAsFactors = FALSE)
  expect_error(widr:::.parse_indicator(df), "indicator format")
})

# ---- .get_variables_areas --------------------------------------------------

test_that(".get_variables_areas parses json into data.frame", {
  # API structure: list(variable -> list(country -> list(list(perc, age, pop))))
  json <- list(
    sptinc992j = list(
      US = list(list("p90p100", "992", "j"))
    )
  )
  with_mocked_bindings(
    .wid_get = function(...) json,
    {
      out <- widr:::.get_variables_areas("sptinc992j", "US")
    },
    .package = "widr"
  )
  expect_equal(nrow(out), 1L)
  expect_equal(out$variable,   "sptinc992j")
  expect_equal(out$country,    "US")
  expect_equal(out$percentile, "p90p100")
})

test_that(".get_variables_areas returns empty frame when json is empty", {
  with_mocked_bindings(
    .wid_get = function(...) list(),
    { out <- widr:::.get_variables_areas("sptinc992j", "US") },
    .package = "widr"
  )
  expect_equal(nrow(out), 0L)
  expect_named(out, c("variable","country","percentile","age","pop"))
})

test_that(".get_variables_areas unwraps single-element outer list", {
  # length(json)==1 triggers json <- json[[1L]]
  json <- list(list(
    sptinc992j = list(US = list(list("p90p100", "992", "j")))
  ))
  with_mocked_bindings(
    .wid_get = function(...) json,
    { out <- widr:::.get_variables_areas("sptinc992j", "US") },
    .package = "widr"
  )
  expect_equal(nrow(out), 1L)
})

# ---- .get_data_variables ---------------------------------------------------

test_that(".get_data_variables parses json and returns data.frame", {
  # API: list(api_code -> list(list(country -> list(values=list(list(yr,val)), meta=...))))
  json <- list(
    `sptinc_p90p100_992_j` = list(
      list(US = list(
        values = list(list(2020L, 0.4)),
        meta   = list(extrapolation = NULL, data_points = NULL)
      ))
    )
  )
  with_mocked_bindings(
    .wid_get = function(...) json,
    { out <- widr:::.get_data_variables("US", "sptinc_p90p100_992_j") },
    .package = "widr"
  )
  expect_equal(out$country, "US")
  expect_equal(out$value,   0.4)
})

test_that(".get_data_variables returns empty frame when json is empty", {
  with_mocked_bindings(
    .wid_get = function(...) list(),
    { out <- widr:::.get_data_variables("US", "sptinc_p90p100_992_j") },
    .package = "widr"
  )
  expect_equal(nrow(out), 0L)
  expect_named(out, c("indicator","country","year","value"))
})

test_that(".get_data_variables strips extrapolations when no_extrapolation=TRUE", {
  extrap <- jsonlite::toJSON(matrix(c(2019L, 2021L), nrow = 1L))
  json <- list(
    `sptinc_p90p100_992_j` = list(
      list(US = list(
        values = list(list(2018L, 0.3), list(2020L, 0.4), list(2021L, 0.5)),
        meta   = list(extrapolation = extrap, data_points = NULL)
      ))
    )
  )
  with_mocked_bindings(
    .wid_get = function(...) json,
    { out <- widr:::.get_data_variables("US", "sptinc_p90p100_992_j",
                                        no_extrapolation = TRUE) },
    .package = "widr"
  )
  expect_false("2021" %in% out$year)
  expect_true("2018"  %in% out$year)
})

# ---- .get_metadata_variables -----------------------------------------------

# Exact structure mirroring .get_metadata_variables parsing logic:
# jv[[1L]][[1L]] = jname  (list with $shortname, $simpledes)
# jv[[3L]][[1L]] = jpop   (list with $longdes)
# jv[[4L]][[1L]] = jage   -> as.character() -> named char vector -> $fullname
# jv[[5L]][[1L]] = junits (list of {country, country_name})
# jv[[6L]][[1L]][[1L]] = notes list, each element has alpha2 + note fields
.mock_meta <- function(imputation = "survey", has_note = TRUE) {
  note <- if (has_note)
    list(list(alpha2 = "US", source = "Tax data", method = "DFL",
              data_quality = "high", imputation = imputation))
  else
    list(list(alpha2 = "US"))   # present but missing note fields -> n$source etc = NULL
  list(list(metadata_func = list(
    list(sptinc992j = list(
      list(list(shortname = "ptinc", simpledes = "Pre-tax income")),  # jv[[1]]
      list(),                                                          # jv[[2]]
      list(list(longdes = "equal-split adults")),                      # jv[[3]]
      list(c(fullname = "20+")),                                       # jv[[4]] named char vec
      list(list(list(country = "US", country_name = "United States"))),# jv[[5]]
      list(list(note))                                                 # jv[[6]]
    ))
  )))
}

test_that(".get_metadata_variables parses json and decodes imputation", {
  with_mocked_bindings(
    .wid_get = function(...) .mock_meta(),
    { out <- widr:::.get_metadata_variables("US", "sptinc992j") },
    .package = "widr"
  )
  expect_s3_class(out, "data.frame")
  expect_equal(out$source[1L],     "Tax data")
  expect_equal(out$imputation[1L], "adjusted surveys")
})

test_that(".get_metadata_variables returns NA fields when note has no data", {
  # note exists but fields missing
  with_mocked_bindings(
    .wid_get = function(...) .mock_meta(has_note = FALSE),
    { out <- widr:::.get_metadata_variables("US", "sptinc992j") },
    .package = "widr"
  )
  expect_true(is.na(out$source[1L]))
})

test_that(".get_metadata_variables returns empty frame when jmeta is empty", {
  with_mocked_bindings(
    .wid_get = function(...) list(list(metadata_func = list())),
    { out <- widr:::.get_metadata_variables("US", "sptinc992j") },
    .package = "widr"
  )
  expect_equal(nrow(out), 0L)
  expect_named(out, c("variable","country","countryname","shortname","shortdes",
                      "pop","age","source","method","quality","imputation"))
})

# ---- .chunk_fetch ----------------------------------------------------------

test_that(".chunk_fetch batches into correct number of chunks", {
  vars <- data.frame(
    variable   = paste0("v", 1:12), percentile = "p90p100",
    age = "992", pop = "j", country = "US",
    api_code   = paste0("v", 1:12, "_p90p100_992_j"),
    stringsAsFactors = FALSE)
  calls <- 0L
  widr:::.chunk_fetch(vars, size = 5L, function(countries, codes) {
    calls <<- calls + 1L
    data.frame(country="US", indicator=codes[1L], year="2020", value=0.1,
               stringsAsFactors=FALSE)
  })
  expect_true(calls >= 2L)
})

test_that(".chunk_fetch returns combined data.frame from all chunks", {
  vars <- data.frame(
    variable = "sptinc992j", percentile = "p90p100",
    age = "992", pop = "j", country = "US",
    api_code = "sptinc_p90p100_992_j", stringsAsFactors = FALSE)
  out <- widr:::.chunk_fetch(vars, size = 10L, function(countries, codes)
    data.frame(country="US", indicator=codes[1L], year="2020", value=0.4,
               stringsAsFactors=FALSE))
  expect_s3_class(out, "data.frame")
  expect_equal(out$value, 0.4)
})

# ---- .auto_title -----------------------------------------------------------

test_that(".auto_title builds label with countries appended (<= 3)", {
  t <- widr:::.auto_title("sptinc992j", c("US","FR"))
  expect_type(t, "character")
  expect_match(t, "US")
})

test_that(".auto_title omits countries when > 3", {
  t <- widr:::.auto_title("sptinc992j", c("US","FR","DE","CN"))
  expect_false(grepl("US", t, fixed = TRUE))
})

test_that(".auto_title falls back to code on decode error or unknown concept", {
  with_mocked_bindings(
    wid_decode = function(...) stop("bad"),
    { expect_match(widr:::.auto_title("sptinc992j", "US"), "sptinc992j") },
    .package = "widr"
  )
  with_mocked_bindings(
    wid_decode = function(...) list(series_type="s", concept="ZZZZZ", age="992", pop="j"),
    { expect_match(widr:::.auto_title("sptinc992j", "US"), "sptinc992j") },
    .package = "widr"
  )
})

# ---- .common_year ----------------------------------------------------------

test_that(".common_year returns max shared year and errors when none", {
  d <- data.frame(country=c("US","US","FR","FR"), year=c("2019","2020","2020","2021"),
                  stringsAsFactors=FALSE)
  expect_equal(widr:::.common_year(d), "2020")
  d2 <- data.frame(country=c("US","FR"), year=c("2019","2020"), stringsAsFactors=FALSE)
  expect_error(widr:::.common_year(d2), "No common year")
})

# wid_convert, wid_metadata, wid_tidy

.XR <- c(lcu = "xlcusx999i", usd = "xlcusx999i", eur = "xleusx999i",
         gbp = "xlgbsx999i", ppp = "xlpppx999i", yppp = "xlpppx999i")

#' Convert a wid_df to another currency
#' @param data      A \code{wid_df}.
#' @param target    One of \code{"lcu"}, \code{"usd"}, \code{"eur"}, \code{"gbp"}, \code{"ppp"}, \code{"yppp"}.
#' @param base_year Pin to a specific year's exchange rate; \code{NULL} uses contemporaneous rates.
#' @return The same \code{wid_df} with values converted. Dimensionless series
#'   (shares, ratios) are returned unchanged with a message.
#' @examples
#' \donttest{
#'   if (nzchar(Sys.getenv("WID_API_KEY"))) {
#'     d <- download_wid("aptinc992j", areas = "US", perc = "p99p100",
#'                       years = 2010:2020)
#'     wid_convert(d, target = "ppp")
#'   }
#' }
#' @export
wid_convert <- function(data, target = "usd", base_year = NULL) {
  .check(data)
  target   <- match.arg(target, names(.XR))
  is_ratio <- substr(data$variable, 1L, 1L) %in% c("s","g","b","w","r","p","i","y")
  if (all(is_ratio)) {
    message("All series are dimensionless; no conversion applied.")
    return(data)
  }
  xr <- download_wid(.XR[target], areas = unique(data$country),
                     years = base_year %||% "all", cache = TRUE)
  if (inherits(xr, "wid_data")) xr <- xr$data
  if (is.null(xr) || !nrow(xr))
    stop("No exchange rates retrieved for target=", target, call. = FALSE)
  if (!is.null(base_year)) xr <- xr[as.integer(xr$year) == as.integer(base_year), ]
  for (i in which(!is_ratio)) {
    rate <- xr$value[xr$country == data$country[i] &
                       (if (!is.null(base_year)) TRUE else as.integer(xr$year) == as.integer(data$year[i]))]
    data$value[i] <- if (length(rate) && !is.na(rate[1L])) data$value[i] / rate[1L] else NA_real_
  }
  data
}

#' Fetch metadata for variables in a wid_df
#' @param data A \code{wid_df}.
#' @return A \code{data.frame} with columns \code{variable}, \code{country},
#'   \code{source}, \code{method}, \code{quality}, and \code{imputation}.
#' @examples
#' \donttest{
#'   if (nzchar(Sys.getenv("WID_API_KEY"))) {
#'     d <- download_wid("sptinc992j", areas = "US", metadata = TRUE)
#'     wid_metadata(d)
#'   }
#' }
#' @export
wid_metadata <- function(data) {
  .check(data)
  dec  <- lapply(unique(data$variable), function(v)
    tryCatch(wid_decode(v, strict = FALSE), warning = function(w) NULL, error = function(e) NULL))
  vars <- unique(vapply(dec, function(d) if (!is.null(d)) d$concept else NA_character_, character(1L)))
  vars <- vars[!is.na(vars)]
  out  <- .get_metadata_variables(unique(data$country), vars)
  if (!nrow(out)) { message("No metadata returned."); return(invisible(out)) }
  cols <- intersect(c("variable","country"), names(out))
  out[!duplicated(out[, cols, drop = FALSE]), ]
}

#' Tidy a wid_df: decode variable codes and coerce column types
#' @param data          A \code{wid_df}.
#' @param decode        Logical. Add decoded columns. Default \code{TRUE}.
#' @param country_names Logical. Join country display names. Default \code{TRUE}.
#' @return A plain \code{data.frame} with \code{year} as integer, \code{value}
#'   as double, and optionally \code{indicator}, \code{series_type},
#'   \code{type_label}, and \code{country_name} columns appended.
#' @examples
#' d <- data.frame(
#'   country = "US", variable = "sptinc992j", percentile = "p99p100",
#'   year = "2020", value = "0.19", age = "992", pop = "j",
#'   stringsAsFactors = FALSE
#' )
#' class(d) <- c("wid_df", "data.frame")
#' wid_tidy(d, country_names = FALSE)
#' @export
wid_tidy <- function(data, decode = TRUE, country_names = TRUE) {
  .check(data)
  out <- as.data.frame(data)
  if (decode) {
    dec             <- lapply(out$variable, function(v)
      tryCatch(wid_decode(v, strict = FALSE), warning = function(w) NULL, error = function(e) NULL))
    out$indicator   <- vapply(dec, function(d)
      if (!is.null(d)) paste0(d$series_type, d$concept) else NA_character_, character(1L))
    out$series_type <- vapply(dec, function(d)
      if (!is.null(d)) d$series_type else NA_character_, character(1L))
    out$type_label  <- .recode(out$series_type, .CODE_TYPES)
  }
  if (country_names) {
    lkp        <- wid_countries[, c("code", "description")]
    names(lkp) <- c("country", "country_name")
    out        <- merge(out, lkp, by = "country", all.x = TRUE)
  }
  out$year  <- as.integer(out$year)
  out$value <- as.numeric(out$value)
  out
}

# ---- wid_convert -----------------------------------------------------------

test_that("wid_convert errors on non-wid_df and unsupported target", {
  expect_error(wid_convert(data.frame()), "wid_df")
  expect_error(wid_convert(.wid(), target = "xyz"))
})

test_that("wid_convert messages and returns unchanged for all-ratio series", {
  d <- .wid(type = "s")
  expect_message(out <- wid_convert(d, "usd"), "dimensionless")
  expect_equal(out$value, d$value)
})

test_that("wid_convert divides non-ratio values by exchange rate", {
  d  <- .wid(type = "a")
  xr <- .wid(type = "a"); xr$value <- rep(2, nrow(xr))
  with_mocked_bindings(download_wid = function(...) xr, {
    out <- wid_convert(d, "usd")
  }, .package = "widr")
  expect_equal(out$value, d$value / 2, tolerance = 1e-9)
})

test_that("wid_convert with base_year filters exchange rate to that year", {
  d  <- .wid(type = "a")
  xr <- rbind(.wid(type = "a"), .wid(type = "a"))
  xr$year <- c(rep("2019", nrow(xr) / 2), rep("2020", nrow(xr) / 2))
  xr$value <- 4
  with_mocked_bindings(download_wid = function(...) xr, {
    out <- wid_convert(d, "usd", base_year = "2020")
  }, .package = "widr")
  expect_equal(out$value, d$value / 4, tolerance = 1e-9)
})

test_that("wid_convert assigns NA when no matching exchange rate", {
  d  <- .wid(type = "a")
  xr <- .wid(type = "a"); xr$country <- "FR"
  with_mocked_bindings(download_wid = function(...) xr, {
    out <- wid_convert(d, "usd")
  }, .package = "widr")
  expect_true(all(is.na(out$value)))
})

test_that("wid_convert handles wid_df returned by download_wid", {
  d  <- .wid(type = "a")
  xr <- .wid(type = "a"); xr$value <- rep(4, nrow(xr))
  with_mocked_bindings(download_wid = function(...) xr, {
    out <- wid_convert(d, "usd")
  }, .package = "widr")
  expect_equal(out$value, d$value / 4, tolerance = 1e-9)
})

test_that("wid_convert stops when no exchange rates retrieved", {
  d <- .wid(type = "a")
  with_mocked_bindings(
    download_wid = function(...) widr:::.wid_df(),
    { expect_error(wid_convert(d, "usd"), "No exchange rates") },
    .package = "widr"
  )
})

# ---- wid_metadata ----------------------------------------------------------

test_that("wid_metadata errors on non-wid_df", {
  expect_error(wid_metadata(data.frame()), "wid_df")
})

test_that("wid_metadata deduplicates and returns data.frame", {
  dup <- do.call(rbind, rep(list(
    data.frame(variable="sptinc992j", country="US", source="s",
               stringsAsFactors=FALSE)), 2L))
  with_mocked_bindings(
    .get_metadata_variables = function(...) dup,
    { out <- wid_metadata(.wid()) },
    .package = "widr"
  )
  expect_equal(nrow(out), 1L)
})

test_that("wid_metadata deduplicates when result has no 'variable' column", {
  dup <- do.call(rbind, rep(list(
    data.frame(country="US", source="s", stringsAsFactors=FALSE)), 2L))
  with_mocked_bindings(
    .get_metadata_variables = function(...) dup,
    { out <- wid_metadata(.wid()) },
    .package = "widr"
  )
  expect_equal(nrow(out), 1L)
})

test_that("wid_metadata messages when empty result returned", {
  with_mocked_bindings(
    .get_metadata_variables = function(...) data.frame(),
    { expect_message(wid_metadata(.wid()), "No metadata") },
    .package = "widr"
  )
})

# ---- wid_tidy --------------------------------------------------------------

test_that("wid_tidy errors on non-wid_df", {
  expect_error(wid_tidy(data.frame()), "wid_df")
})

test_that("wid_tidy adds decoded columns with correct values", {
  out <- wid_tidy(.wid(), country_names = FALSE)
  expect_true(all(c("indicator","series_type","type_label") %in% names(out)))
  expect_equal(out$indicator[1L],   "sptinc")
  expect_equal(out$series_type[1L], "s")
  expect_equal(out$type_label[1L],  "share")
})

test_that("wid_tidy coerces year to integer and value to double", {
  out <- wid_tidy(.wid(), country_names = FALSE)
  expect_type(out$year,  "integer")
  expect_type(out$value, "double")
})

test_that("wid_tidy skips decode columns when decode=FALSE", {
  out <- wid_tidy(.wid(), decode = FALSE, country_names = FALSE)
  expect_false("indicator" %in% names(out))
})

test_that("wid_tidy joins country_name when country_names=TRUE", {
  out <- wid_tidy(.wid(), decode = FALSE, country_names = TRUE)
  expect_true("country_name" %in% names(out))
  expect_equal(out$country_name[1L],
               wid_countries$description[wid_countries$code == "US"])
})

test_that("wid_tidy sets indicator to NA for unrecognisable variable", {
  d <- .wid(); d$variable <- "ZZZZZZ999z"
  out <- wid_tidy(d, country_names = FALSE)
  expect_true(is.na(out$indicator[1L]))
})

# WID code grammar: <series_type:1><concept:5-6>[<age:3>][<pop:1>]

#' Validate WID code components
#' @param series_type One-letter series type code.
#' @param concept     5-6 letter concept code.
#' @param age         Three-digit age code.
#' @param pop         One-letter population unit code.
#' @param years       Integer vector of years.
#' @param areas       ISO-2 area codes.
#' @param perc        Percentile codes of the form \code{pXpY}.
#' @return Invisibly, a named list of normalised inputs with the same names as
#'   the parameters, with \code{age} zero-padded and \code{years} coerced to
#'   integer.
#' @examples
#' wid_validate(series_type = "s", concept = "ptinc", age = 992, pop = "j")
#' wid_validate(areas = c("US", "FR", "US-CA"))
#' @export
wid_validate <- function(series_type = NULL, concept = NULL, age = NULL,
                         pop = NULL, years = NULL, areas = NULL, perc = NULL) {
  if (!is.null(age)) {
    age <- formatC(suppressWarnings(as.integer(age)), width = 3L, flag = "0")
    if (anyNA(suppressWarnings(as.integer(age))))
      stop("'age' must be coercible to integer.", call. = FALSE)
  }
  for (ch in list(list(series_type, wid_series_types, "series_type"),
                  list(concept,     wid_concepts,     "concept"),
                  list(age,         wid_ages,         "age"),
                  list(pop,         wid_pop_types,    "pop"))) {
    val <- ch[[1L]]; tbl <- ch[[2L]]; label <- ch[[3L]]
    if (is.null(val)) next
    bad <- setdiff(val, tbl$code)
    if (length(bad))
      stop(sprintf("Unknown %s: %s", label, paste(bad, collapse = ", ")), call. = FALSE)
  }
  if (!is.null(years)) {
    years <- suppressWarnings(as.integer(years))
    if (anyNA(years)) stop("'years' must be integers.", call. = FALSE)
    if (any(years < 1800L | years > 2100L))
      warning("Some years outside [1800, 2100].", call. = FALSE)
  }
  if (!is.null(areas)) {
    bad <- areas[!grepl("^[A-Z]{2}(-[A-Z0-9]{1,5})?$", areas)]
    if (length(bad))
      warning(sprintf("Area codes may be invalid: %s", paste(bad, collapse = ", ")), call. = FALSE)
  }
  if (!is.null(perc)) {
    bad <- perc[!grepl("^p[0-9]+(\\.[0-9]+)?p[0-9]+(\\.[0-9]+)?$", perc)]
    if (length(bad))
      stop(sprintf("Invalid percentile format: %s", paste(bad, collapse = ", ")), call. = FALSE)
    bounds <- regmatches(perc, regexec("^p([0-9.]+)p([0-9.]+)$", perc))
    inv    <- vapply(bounds, function(m) as.numeric(m[2L]) >= as.numeric(m[3L]), logical(1L))
    if (any(inv))
      stop(sprintf("Lower >= upper in: %s", paste(perc[inv], collapse = ", ")), call. = FALSE)
  }
  invisible(list(series_type = series_type, concept = concept, age = age,
                 pop = pop, years = years, areas = areas, perc = perc))
}

#' Test WID code components without throwing
#' @param ... Arguments passed to \code{\link{wid_validate}}.
#' @return Scalar logical: \code{TRUE} if all supplied components are valid,
#'   \code{FALSE} if any would cause an error. Warning-level issues return
#'   \code{TRUE}.
#' @examples
#' wid_is_valid(series_type = "s", concept = "ptinc")  # TRUE
#' wid_is_valid(series_type = "Z")                      # FALSE
#' @export
wid_is_valid <- function(...) {
  tryCatch({ wid_validate(...); TRUE }, error = function(e) FALSE, warning = function(w) TRUE)
}

#' Parse a WID variable code into its components
#' @param x      Character string, e.g. \code{"sptinc992j"}.
#' @param strict \code{TRUE} (default) stops on error; \code{FALSE} warns and returns \code{NULL}.
#' @return Named list with elements \code{series_type} (character),
#'   \code{concept} (character), \code{age} (character or \code{NULL}),
#'   and \code{pop} (character or \code{NULL}).
#' @examples
#' wid_decode("sptinc992j")
#' wid_decode("mnninc")        # no age or pop
#' @export
wid_decode <- function(x, strict = TRUE) {
  x    <- trimws(as.character(x))
  .err <- function(...) {
    msg <- paste0(...)
    if (strict) stop(msg, call. = FALSE) else { warning(msg, call. = FALSE); return(NULL) }
  }
  st <- substr(x, 1L, 1L)
  if (!st %in% wid_series_types$code) return(.err("Unknown series_type: '", st, "'"))
  rest <- substr(x, 2L, nchar(x))
  if (nchar(rest) < 4L) return(.err("Unknown series_type or code too short: '", x, "'"))
  codes <- wid_concepts$code[order(-nchar(wid_concepts$code))]
  hit   <- codes[startsWith(rest, codes)][1L]
  if (is.na(hit)) return(.err("No known concept in: '", rest, "'"))
  rest <- substr(rest, nchar(hit) + 1L, nchar(rest))
  age  <- pop <- NULL
  if (nchar(rest) > 0L) {
    if (!grepl("^[0-9]{3}", rest))
      return(.err("Expected 3-digit age suffix, got: '", rest, "'"))
    age_c <- substr(rest, 1L, 3L)
    if (!age_c %in% wid_ages$code) return(.err("Unknown age: '", age_c, "'"))
    age  <- age_c
    rest <- substr(rest, 4L, nchar(rest))
  }
  if (nchar(rest) == 1L) {
    if (!rest %in% wid_pop_types$code) return(.err("Unknown pop: '", rest, "'"))
    pop <- rest
  } else if (nchar(rest) > 1L) {
    return(.err("Unexpected trailing characters: '", rest, "'"))
  }
  list(series_type = st, concept = hit, age = age, pop = pop)
}

#' Build a WID variable code from its components
#' @param series_type 1-letter series type, or a list from \code{\link{wid_decode}}.
#' @param concept     5-6 letter concept code.
#' @param age         3-digit age code or \code{NULL}.
#' @param pop         1-letter population code or \code{NULL}.
#' @return A character string of the form
#'   \code{<type><concept>[age][pop]}.
#' @examples
#' wid_encode("s", "ptinc", "992", "j")   # "sptinc992j"
#' wid_encode("m", "nninc")               # "mnninc"
#' wid_encode(wid_decode("sptinc992j"))   # round-trip
#' @export
wid_encode <- function(series_type, concept = NULL, age = NULL, pop = NULL) {
  if (is.list(series_type)) {
    x <- series_type
    return(paste0(x$series_type, x$concept, x$age %||% "", x$pop %||% ""))
  }
  wid_validate(series_type = series_type, concept = concept, age = age, pop = pop)
  paste0(series_type, concept, age %||% "", pop %||% "")
}

# Internal lookup source (mockable in tests)
.get_available_variables <- function() {
  pool <- list(
    concepts     = wid_concepts,     series_types = wid_series_types,
    ages         = wid_ages,         pop_types    = wid_pop_types,
    countries    = wid_countries,    percentiles  = wid_percentiles
  )
  do.call(rbind, lapply(names(pool), function(nm) {
    df <- pool[[nm]]
    data.frame(table = nm, variable = df$code, description = df$description,
               stringsAsFactors = FALSE)
  }))
}

#' Search WID lookup tables by regex
#' @param query  Regular expression.
#' @param tables Table names to search. Default \code{"concepts"}.
#' @param type   Optional series-type letter to filter results.
#' @return A \code{data.frame} with columns \code{table},
#'   \code{variable}, and \code{description} for each matching row,
#'   or an invisible empty \code{data.frame} if no matches are found.
#' @examples
#' wid_search("national income")
#' wid_search("wealth", type = "s")
#' wid_search("^US", tables = "countries")
#' @export
wid_search <- function(query, tables = "concepts", type = NULL) {
  all_tables <- c("concepts","series_types","ages","pop_types","countries","percentiles")
  if (!identical(tables, "all")) {
    bad <- setdiff(tables, all_tables)
    if (length(bad)) stop("Unknown table(s): ", paste(bad, collapse = ", "), call. = FALSE)
  }
  av  <- .get_available_variables()
  if (!identical(tables, "all")) av <- av[av$table %in% tables, ]
  idx <- grepl(query, av$variable, ignore.case = TRUE) |
    grepl(query, av$description, ignore.case = TRUE)
  out <- av[idx, ]
  if (!is.null(type)) out <- out[substr(out$variable, 1L, 1L) == type, ]
  rownames(out) <- NULL
  if (!nrow(out)) { message("No matches found."); return(invisible(out)) }
  out
}

# ---- wid_validate ----------------------------------------------------------

test_that("wid_validate returns invisibly on valid inputs", {
  expect_invisible(wid_validate(series_type = "s", concept = "ptinc",
                                age = 992, pop = "j"))
})

test_that("wid_validate normalises age to zero-padded 3-digit string", {
  out <- wid_validate(age = 992)
  expect_equal(out$age, "992")
})

test_that("wid_validate errors on unknown series_type", {
  expect_error(wid_validate(series_type = "Z"), "series_type")
})

test_that("wid_validate errors on unknown concept", {
  expect_error(wid_validate(concept = "zzzzz"), "concept")
})

test_that("wid_validate errors on unknown age", {
  expect_error(wid_validate(age = 0), "age")
})

test_that("wid_validate errors on unknown pop", {
  expect_error(wid_validate(pop = "z"), "pop")
})

test_that("wid_validate errors on non-integer years", {
  expect_error(wid_validate(years = "x"), "integer")
})

test_that("wid_validate warns on years outside [1800,2100]", {
  expect_warning(wid_validate(years = 1799L), "1800")
})

test_that("wid_validate warns on malformed area codes", {
  expect_warning(wid_validate(areas = "lowercase"), "invalid")
})

test_that("wid_validate accepts valid area codes including subregions", {
  expect_invisible(wid_validate(areas = c("US","US-CA","FR")))
})

test_that("wid_validate errors on invalid perc format", {
  expect_error(wid_validate(perc = "bad"), "percentile")
})

test_that("wid_validate errors when percentile lower >= upper", {
  expect_error(wid_validate(perc = "p90p10"), "Lower >= upper")
})

test_that("wid_validate accepts valid perc", {
  expect_invisible(wid_validate(perc = "p99p100"))
})

test_that("wid_validate errors on non-coercible age", {
  expect_error(wid_validate(age = "abc"), "integer")
})

# ---- wid_is_valid ----------------------------------------------------------

test_that("wid_is_valid returns TRUE for valid components", {
  expect_true(wid_is_valid(series_type = "s", concept = "ptinc"))
})

test_that("wid_is_valid returns FALSE for invalid series_type", {
  expect_false(wid_is_valid(series_type = "Z"))
})

test_that("wid_is_valid returns TRUE (not FALSE) for warning-level issues", {
  expect_true(wid_is_valid(years = 1799L))
})

# ---- wid_decode ------------------------------------------------------------

test_that("wid_decode parses full canonical code", {
  d <- wid_decode("sptinc992j")
  expect_equal(d$series_type, "s")
  expect_equal(d$concept,     "ptinc")
  expect_equal(d$age,         "992")
  expect_equal(d$pop,         "j")
})

test_that("wid_decode parses code without age/pop", {
  d <- wid_decode("mnninc")
  expect_equal(d$series_type, "m")
  expect_equal(d$concept,     "nninc")
  expect_null(d$age)
  expect_null(d$pop)
})

test_that("wid_decode errors on unknown series_type (strict)", {
  expect_error(wid_decode("Zptinc992j"), "series_type")
})

test_that("wid_decode errors on code too short (strict)", {
  expect_error(wid_decode("bad"), "series_type|too short")
})

test_that("wid_decode errors when concept not found in rest of code", {
  # 's' is a valid series_type; 'zzzzz' matches no concept in wid_concepts
  expect_error(wid_decode("szzzzz"), "No known concept")
})

test_that("wid_decode errors on unexpected trailing characters (strict)", {
  expect_error(wid_decode("sptinc992jXX"), "trailing")
})

test_that("wid_decode returns NULL and warns on bad code when strict=FALSE", {
  expect_warning(out <- wid_decode("bad", strict = FALSE))
  expect_null(out)
})

test_that("wid_decode errors on unknown age code", {
  expect_error(wid_decode("sptinc000j"), "age")
})

test_that("wid_decode errors on unknown pop code", {
  expect_error(wid_decode("sptinc992z"), "pop")
})

test_that("wid_decode errors when age suffix is not 3 digits", {
  expect_error(wid_decode("sptincXXXj"), "age suffix")
})

# ---- wid_encode ------------------------------------------------------------

test_that("wid_encode builds correct code from components", {
  expect_equal(wid_encode("s", "ptinc", "992", "j"), "sptinc992j")
})

test_that("wid_encode omits optional age and pop", {
  expect_equal(wid_encode("m", "nninc"), "mnninc")
})

test_that("wid_encode accepts list from wid_decode", {
  d <- wid_decode("sptinc992j")
  expect_equal(wid_encode(d), "sptinc992j")
})

test_that("wid_encode round-trips through wid_decode", {
  code <- "tptinc992j"
  expect_equal(wid_encode(wid_decode(code)), code)
})

test_that("wid_encode errors on invalid series_type", {
  expect_error(wid_encode("Z", "ptinc"), "series_type")
})

test_that("wid_encode errors on invalid concept", {
  expect_error(wid_encode("s", "zzzzz"), "concept")
})

# ---- wid_search ------------------------------------------------------------

test_that("wid_search returns matches from concepts table", {
  out <- wid_search("pretax national income")
  expect_s3_class(out, "data.frame")
  expect_true(nrow(out) >= 1L)
  expect_true("ptinc" %in% out$variable)
})

test_that("wid_search is case-insensitive", {
  out <- wid_search("PRETAX")
  expect_true(nrow(out) >= 1L)
})

test_that("wid_search messages and returns empty frame on no match", {
  expect_message(out <- wid_search("zzznomatch999"), "No matches")
  expect_equal(nrow(out), 0L)
})

test_that("wid_search errors on unknown table name", {
  expect_error(wid_search("x", tables = "nosuchtable"), "Unknown table")
})

test_that("wid_search type= filter restricts to codes starting with that letter", {
  out <- wid_search("ptinc", type = "s")
  expect_true(all(substr(out$variable, 1L, 1L) == "s"))
})

test_that("wid_search tables='all' searches across all tables", {
  out <- wid_search("share", tables = "all")
  tbls <- unique(out$table)
  expect_true(length(tbls) >= 1L)
})

test_that("wid_search searches the series_types table when requested", {
  out <- wid_search("threshold", tables = "series_types")
  expect_true(nrow(out) >= 1L)
})

# Lookup table documentation.
# The actual data objects live in data/*.rda, generated by
# data-raw/sync_wid_dictionary.R and committed with usethis::use_data().

#' WID series type codes
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_series_types"

#' WID concept codes (400+ indicators)
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_concepts"

#' WID age group codes (zero-padded 3-digit strings)
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_ages"

#' WID population unit codes
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_pop_types"

#' WID country and region codes
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_countries"

#' Canonical WID percentile codes
#' @format A data frame with columns \code{code} and \code{description}.
#' @source \url{https://wid.world/codes-dictionary/}
#' @export
"wid_percentiles"

# Generated by data-raw/sync_wid_dictionary.R -- do not edit by hand.
# Source: https://wid.world/codes-dictionary/  Synced: 2026-03-05
# series_types=17 ages=40 pop_types=6 countries=281 percentiles=184 concepts=464

#' @title Series type codes (\\S2.1.1)
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_series_types <- local({
  code <- c(
    "a",
    "b",
    "f",
    "g",
    "i",
    "n",
    "s",
    "t",
    "m",
    "p",
    "w",
    "y",
    "r",
    "x",
    "e",
    "k",
    "l"
  )
  description <- c(
    "average",
    "inverted Pareto-Lorenz coefficient",
    "female population",
    "Gini coefficient",
    "index",
    "population",
    "share",
    "threshold",
    "total",
    "proportion of women",
    "wealth-to-income ratio or labor/capital share",
    "wealth-to-GDP ratio",
    "Top 10/Bottom 50 ratio",
    "exchange rate (market or PPP)",
    "Total emissions",
    "Per capita emissions",
    "Average per capita group emissions"
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})


#' @title Age group codes (\\S2.1.3)
#' @description Zero-padded three-digit strings.
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_ages <- local({
  code <- c(
    "999",
    "014",
    "156",
    "997",
    "991",
    "992",
    "993",
    "994",
    "995",
    "996",
    "998",
    "201",
    "251",
    "301",
    "351",
    "401",
    "451",
    "501",
    "551",
    "601",
    "651",
    "701",
    "751",
    "801",
    "851",
    "901",
    "951",
    "111",
    "001",
    "051",
    "101",
    "151",
    "202",
    "302",
    "402",
    "502",
    "602",
    "702",
    "802",
    "902"
  )
  description <- c(
    "The population is comprised of individuals of all ages.",
    "The population is comprised of individuals in the 0 to 14 age group.",
    "The population is comprised of individuals in the 15 to 64 age group.",
    "The population is comprised of individuals over the age of 65.",
    "The population is comprised of individuals below the age of 20.",
    "The population is comprised of individuals over the age of 20.",
    "The population is comprised of individuals between the ages of 20 and 39.",
    "The population is comprised of individuals between the ages of 40 and 59.",
    "The population is comprised of individuals over the age of 60.",
    "The population is comprised of individuals between the ages of 20 and 64.",
    "The population is comprised of individuals over the age of 80.",
    "The population is comprised of individuals in the 20 to 24 age group.",
    "The population is comprised of individuals in the 25 to 29 age group.",
    "The population is comprised of individuals in the 30 to 34 age group.",
    "The population is comprised of individuals in the 35 to 39 age group.",
    "The population is comprised of individuals in the 40 to 44 age group.",
    "The population is comprised of individuals in the 45 to 49 age group.",
    "The population is comprised of individuals in the 50 to 54 age group.",
    "The population is comprised of individuals in the 55 to 59 age group.",
    "The population is comprised of individuals in the 60 to 64 age group.",
    "The population is comprised of individuals in the 65 to 69 age group.",
    "The population is comprised of individuals in the 70 to 74 age group.",
    "The population is comprised of individuals in the 75 to 79 age group.",
    "The population is comprised of individuals in the 80 to 84 age group.",
    "The population is comprised of individuals in the 85 to 89 age group.",
    "The population is comprised of individuals in the 90 to 94 age group.",
    "The population is comprised of individuals in the 95 to 99 age group.",
    "The population is comprised of individuals over the age of 99.",
    "The population is comprised of individuals in the 0 to 4 age group.",
    "The population is comprised of individuals in the 5 to 9 age group.",
    "The population is comprised of individuals in the 10 to 14 age group.",
    "The population is comprised of individuals in the 15 to 19 age group.",
    "The population is comprised of individuals in the 20 to 29 age group.",
    "The population is comprised of individuals in the 30 to 39 age group.",
    "The population is comprised of individuals in the 40 to 49 age group.",
    "The population is comprised of individuals in the 50 to 59 age group.",
    "The population is comprised of individuals in the 60 to 69 age group.",
    "The population is comprised of individuals in the 70 to 79 age group.",
    "The population is comprised of individuals in the 80 to 89 age group.",
    "The population is comprised of individuals in the 90 to 99 age group."
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})


#' @title Population unit codes (\\S2.1.4)
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_pop_types <- local({
  code <- c(
    "i",
    "j",
    "m",
    "f",
    "t",
    "e"
  )
  description <- c(
    "individuals",
    "equal-split adults",
    "male",
    "female",
    "tax unit",
    "employed"
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})


#' @title Country and region codes (\\S2.2)
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_countries <- local({
  code <- c(
    "AD",
    "AE",
    "AF",
    "AG",
    "AI",
    "AL",
    "AM",
    "AO",
    "AR",
    "AT",
    "AU",
    "AW",
    "AZ",
    "BA",
    "BB",
    "BD",
    "BE",
    "BF",
    "BG",
    "BH",
    "BI",
    "BJ",
    "BM",
    "BN",
    "BO",
    "BQ",
    "BR",
    "BS",
    "BT",
    "BW",
    "BY",
    "BZ",
    "CA",
    "CD",
    "CF",
    "CG",
    "CH",
    "CI",
    "CL",
    "CM",
    "CN",
    "CN-RU",
    "CN-UR",
    "CO",
    "CR",
    "CS",
    "CU",
    "CV",
    "CW",
    "CY",
    "CZ",
    "DD",
    "DE",
    "DE-BD",
    "DE-BY",
    "DE-HB",
    "DE-HE",
    "DE-HH",
    "DE-PR",
    "DE-SN",
    "DE-WU",
    "DJ",
    "DK",
    "DM",
    "DO",
    "DZ",
    "EC",
    "EE",
    "EG",
    "ER",
    "ES",
    "ET",
    "FI",
    "FJ",
    "FM",
    "FR",
    "GA",
    "GB",
    "GD",
    "GE",
    "GG",
    "GH",
    "GI",
    "GL",
    "GM",
    "GN",
    "GQ",
    "GR",
    "GT",
    "GW",
    "GY",
    "HK",
    "HN",
    "HR",
    "HT",
    "HU",
    "ID",
    "IE",
    "IL",
    "IM",
    "IN",
    "IQ",
    "IR",
    "IS",
    "IT",
    "JE",
    "JM",
    "JO",
    "JP",
    "KE",
    "KG",
    "KH",
    "KI",
    "KM",
    "KN",
    "KP",
    "KR",
    "KS",
    "KW",
    "KY",
    "KZ",
    "LA",
    "LB",
    "LC",
    "LI",
    "LK",
    "LR",
    "LS",
    "LT",
    "LU",
    "LV",
    "LY",
    "MA",
    "MC",
    "MD",
    "ME",
    "MG",
    "MH",
    "MK",
    "ML",
    "MM",
    "MN",
    "MO",
    "MR",
    "MS",
    "MT",
    "MU",
    "MV",
    "MW",
    "MX",
    "MY",
    "MZ",
    "NC",
    "NE",
    "NG",
    "NI",
    "NL",
    "NO",
    "NP",
    "NR",
    "NZ",
    "OM",
    "PA",
    "PE",
    "PF",
    "PG",
    "PH",
    "PK",
    "PL",
    "PR",
    "PS",
    "PT",
    "PW",
    "PY",
    "QA",
    "RO",
    "RS",
    "RU",
    "RW",
    "SA",
    "SB",
    "SC",
    "SD",
    "SE",
    "SG",
    "SI",
    "SK",
    "SL",
    "SM",
    "SN",
    "SO",
    "SR",
    "SS",
    "ST",
    "SU",
    "SV",
    "SX",
    "SY",
    "SZ",
    "TC",
    "TD",
    "TG",
    "TH",
    "TJ",
    "TL",
    "TM",
    "TN",
    "TO",
    "TR",
    "TT",
    "TV",
    "TW",
    "TZ",
    "UA",
    "UG",
    "US",
    "US-AK",
    "US-AL",
    "US-AR",
    "US-AZ",
    "US-CA",
    "US-CO",
    "US-CT",
    "US-DC",
    "US-DE",
    "US-FL",
    "US-GA",
    "US-HI",
    "US-IA",
    "US-ID",
    "US-IL",
    "US-IN",
    "US-KS",
    "US-KY",
    "US-LA",
    "US-MA",
    "US-MD",
    "US-ME",
    "US-MI",
    "US-MN",
    "US-MO",
    "US-MS",
    "US-MT",
    "US-NC",
    "US-ND",
    "US-NE",
    "US-NH",
    "US-NJ",
    "US-NM",
    "US-NV",
    "US-NY",
    "US-OH",
    "US-OK",
    "US-OR",
    "US-PA",
    "US-RI",
    "US-SC",
    "US-SD",
    "US-TN",
    "US-TX",
    "US-UT",
    "US-VA",
    "US-VT",
    "US-WA",
    "US-WI",
    "US-WV",
    "US-WY",
    "UY",
    "UZ",
    "VC",
    "VE",
    "VG",
    "VN",
    "VU",
    "WS",
    "YE",
    "YU",
    "ZA",
    "ZM",
    "ZW",
    "ZZ"
  )
  description <- c(
    "Andorra",
    "United Arab Emirates",
    "Afghanistan",
    "Antigua and Barbuda",
    "Anguilla",
    "Albania",
    "Armenia",
    "Angola",
    "Argentina",
    "Austria",
    "Australia",
    "Aruba",
    "Azerbaijan",
    "Bosnia and Herzegovina",
    "Barbados",
    "Bangladesh",
    "Belgium",
    "Burkina Faso",
    "Bulgaria",
    "Bahrain",
    "Burundi",
    "Benin",
    "Bermuda",
    "Brunei Darussalam",
    "Bolivia",
    "Bonaire",
    "Brazil",
    "Bahamas",
    "Bhutan",
    "Botswana",
    "Belarus",
    "Belize",
    "Canada",
    "DR Congo",
    "Central African Republic",
    "Congo",
    "Switzerland",
    "Cote d\u2019Ivoire",
    "Chile",
    "Cameroon",
    "China",
    "Rural China",
    "Urban China",
    "Colombia",
    "Costa Rica",
    "Czechoslovakia",
    "Cuba",
    "Cabo Verde",
    "Curacao",
    "Cyprus",
    "Czechia",
    "German Democratic Republic",
    "Germany",
    "Baden",
    "Bavaria",
    "Bremen",
    "Hesse",
    "Hamburg",
    "Prussia",
    "Saxony",
    "Wurttemberg",
    "Djibouti",
    "Denmark",
    "Dominica",
    "Dominican Republic",
    "Algeria",
    "Ecuador",
    "Estonia",
    "Egypt",
    "Eritrea",
    "Spain",
    "Ethiopia",
    "Finland",
    "Fiji",
    "Micronesia",
    "France",
    "Gabon",
    "United Kingdom",
    "Grenada",
    "Georgia",
    "Guernsey",
    "Ghana",
    "Gibraltar",
    "Greenland",
    "Gambia",
    "Guinea",
    "Equatorial Guinea",
    "Greece",
    "Guatemala",
    "Guinea-Bissau",
    "Guyana",
    "Hong Kong",
    "Honduras",
    "Croatia",
    "Haiti",
    "Hungary",
    "Indonesia",
    "Ireland",
    "Israel",
    "Isle of Man",
    "India",
    "Iraq",
    "Iran",
    "Iceland",
    "Italy",
    "Jersey",
    "Jamaica",
    "Jordan",
    "Japan",
    "Kenya",
    "Kyrgyzstan",
    "Cambodia",
    "Kiribati",
    "Comoros",
    "Saint Kitts and Nevis",
    "North Korea",
    "South Korea",
    "Kosovo",
    "Kuwait",
    "Cayman Islands",
    "Kazakhstan",
    "Lao PDR",
    "Lebanon",
    "Saint Lucia",
    "Liechtenstein",
    "Sri Lanka",
    "Liberia",
    "Lesotho",
    "Lithuania",
    "Luxembourg",
    "Latvia",
    "Libya",
    "Morocco",
    "Monaco",
    "Moldova",
    "Montenegro",
    "Madagascar",
    "Marshall Islands",
    "North Macedonia",
    "Mali",
    "Myanmar",
    "Mongolia",
    "Macao",
    "Mauritania",
    "Montserrat",
    "Malta",
    "Mauritius",
    "Maldives",
    "Malawi",
    "Mexico",
    "Malaysia",
    "Mozambique",
    "New Caledonia",
    "Niger",
    "Nigeria",
    "Nicaragua",
    "Netherlands",
    "Norway",
    "Nepal",
    "Nauru",
    "New Zealand",
    "Oman",
    "Panama",
    "Peru",
    "French Polynesia",
    "Papua New Guinea",
    "Philippines",
    "Pakistan",
    "Poland",
    "Puerto Rico",
    "Palestine",
    "Portugal",
    "Palau",
    "Paraguay",
    "Qatar",
    "Romania",
    "Serbia",
    "Russia",
    "Rwanda",
    "Saudi Arabia",
    "Solomon Islands",
    "Seychelles",
    "Sudan",
    "Sweden",
    "Singapore",
    "Slovenia",
    "Slovakia",
    "Sierra Leone",
    "San Marino",
    "Senegal",
    "Somalia",
    "Suriname",
    "South Sudan",
    "Sao Tome and Principe",
    "USSR",
    "El Salvador",
    "Sint Maarten (Dutch part)",
    "Syria",
    "Eswatini",
    "Turks and Caicos Islands",
    "Chad",
    "Togo",
    "Thailand",
    "Tajikistan",
    "Timor-Leste",
    "Turkmenistan",
    "Tunisia",
    "Tonga",
    "Turkiye",
    "Trinidad and Tobago",
    "Tuvalu",
    "Taiwan",
    "Tanzania",
    "Ukraine",
    "Uganda",
    "USA",
    "Alaska",
    "Alabama",
    "Arkansas",
    "Arizona",
    "California",
    "Colorado",
    "Connecticut",
    "District of Columbia",
    "Delaware",
    "Florida",
    "Georgia",
    "Hawaii",
    "Iowa",
    "Idaho",
    "Illinois",
    "Indiana",
    "Kansas",
    "Kentucky",
    "Louisiana",
    "Massachusetts",
    "Maryland",
    "Maine",
    "Michigan",
    "Minnesota",
    "Missouri",
    "Mississippi",
    "Montana",
    "North Carolina",
    "North Dakota",
    "Nebraska",
    "New Hampshire",
    "New Jersey",
    "New Mexico",
    "Nevada",
    "New York",
    "Ohio",
    "Oklahoma",
    "Oregon",
    "Pennsylvania",
    "Rhode Island",
    "South Carolina",
    "South Dakota",
    "Tennessee",
    "Texas",
    "Utah",
    "Virginia",
    "Vermont",
    "Washington",
    "Wisconsin",
    "West Virginia",
    "Wyoming",
    "Uruguay",
    "Uzbekistan",
    "Saint Vincent and the Grenadines",
    "Venezuela",
    "Virgin Islands, British",
    "Viet Nam",
    "Vanuatu",
    "Samoa",
    "Yemen",
    "Yugoslavia",
    "South Africa",
    "Zambia",
    "Zimbabwe",
    "Zanzibar"
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})


#' @title Percentile group codes (\\S2.3)
#' @description Enumerated from the pXpY grammar: key groups, deciles, centiles, and top-1\% fine groups.
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_percentiles <- local({
  code <- c(
    "p0p1",
    "p0p10",
    "p0p100",
    "p0p20",
    "p0p30",
    "p0p40",
    "p0p50",
    "p0p60",
    "p0p70",
    "p0p80",
    "p0p90",
    "p10p100",
    "p10p11",
    "p10p20",
    "p10p30",
    "p10p40",
    "p10p50",
    "p10p60",
    "p10p70",
    "p10p80",
    "p10p90",
    "p11p12",
    "p12p13",
    "p13p14",
    "p14p15",
    "p15p16",
    "p16p17",
    "p17p18",
    "p18p19",
    "p19p20",
    "p1p2",
    "p20p100",
    "p20p21",
    "p20p30",
    "p20p40",
    "p20p50",
    "p20p60",
    "p20p70",
    "p20p80",
    "p20p90",
    "p21p22",
    "p22p23",
    "p23p24",
    "p24p25",
    "p25p26",
    "p26p27",
    "p27p28",
    "p28p29",
    "p29p30",
    "p2p3",
    "p30p100",
    "p30p31",
    "p30p40",
    "p30p50",
    "p30p60",
    "p30p70",
    "p30p80",
    "p30p90",
    "p31p32",
    "p32p33",
    "p33p34",
    "p34p35",
    "p35p36",
    "p36p37",
    "p37p38",
    "p38p39",
    "p39p40",
    "p3p4",
    "p40p100",
    "p40p41",
    "p40p50",
    "p40p60",
    "p40p70",
    "p40p80",
    "p40p90",
    "p41p42",
    "p42p43",
    "p43p44",
    "p44p45",
    "p45p46",
    "p46p47",
    "p47p48",
    "p48p49",
    "p49p50",
    "p4p5",
    "p50p100",
    "p50p51",
    "p50p60",
    "p50p70",
    "p50p80",
    "p50p90",
    "p51p52",
    "p52p53",
    "p53p54",
    "p54p55",
    "p55p56",
    "p56p57",
    "p57p58",
    "p58p59",
    "p59p60",
    "p5p6",
    "p60p100",
    "p60p61",
    "p60p70",
    "p60p80",
    "p60p90",
    "p61p62",
    "p62p63",
    "p63p64",
    "p64p65",
    "p65p66",
    "p66p67",
    "p67p68",
    "p68p69",
    "p69p70",
    "p6p7",
    "p70p100",
    "p70p71",
    "p70p80",
    "p70p90",
    "p71p72",
    "p72p73",
    "p73p74",
    "p74p75",
    "p75p76",
    "p76p77",
    "p77p78",
    "p78p79",
    "p79p80",
    "p7p8",
    "p80p100",
    "p80p81",
    "p80p90",
    "p81p82",
    "p82p83",
    "p83p84",
    "p84p85",
    "p85p86",
    "p86p87",
    "p87p88",
    "p88p89",
    "p89p90",
    "p8p9",
    "p90p100",
    "p90p91",
    "p91p92",
    "p92p93",
    "p93p94",
    "p94p95",
    "p95p96",
    "p96p97",
    "p97p98",
    "p98p99",
    "p99.1p99.2",
    "p99.2p99.3",
    "p99.3p99.4",
    "p99.4p99.5",
    "p99.5p99.6",
    "p99.6p99.7",
    "p99.7p99.8",
    "p99.8p99.9",
    "p99.90p99.91",
    "p99.91p99.92",
    "p99.92p99.93",
    "p99.93p99.94",
    "p99.94p99.95",
    "p99.95p99.96",
    "p99.96p99.97",
    "p99.97p99.98",
    "p99.98p99.99",
    "p99.990p99.991",
    "p99.991p99.992",
    "p99.992p99.993",
    "p99.993p99.994",
    "p99.994p99.995",
    "p99.995p99.996",
    "p99.996p99.997",
    "p99.997p99.998",
    "p99.998p99.999",
    "p99.999p100",
    "p99.99p100",
    "p99.9p100",
    "p99p100",
    "p9p10"
  )
  description <- c(
    "Bottom 1%",
    "Bottom 10%",
    "Entire population",
    "Bottom 20%",
    "Bottom 30%",
    "Bottom 40%",
    "Bottom 50%",
    "Bottom 60%",
    "Bottom 70%",
    "Bottom 80%",
    "Bottom 90%",
    "Top 90%",
    "Percentiles 10\u201311",
    "Percentiles 10\u201320",
    "Percentiles 10\u201330",
    "Percentiles 10\u201340",
    "Percentiles 10\u201350",
    "Percentiles 10\u201360",
    "Percentiles 10\u201370",
    "Percentiles 10\u201380",
    "Percentiles 10\u201390",
    "Percentiles 11\u201312",
    "Percentiles 12\u201313",
    "Percentiles 13\u201314",
    "Percentiles 14\u201315",
    "Percentiles 15\u201316",
    "Percentiles 16\u201317",
    "Percentiles 17\u201318",
    "Percentiles 18\u201319",
    "Percentiles 19\u201320",
    "Percentiles 1\u20132",
    "Top 80%",
    "Percentiles 20\u201321",
    "Percentiles 20\u201330",
    "Percentiles 20\u201340",
    "Percentiles 20\u201350",
    "Percentiles 20\u201360",
    "Percentiles 20\u201370",
    "Percentiles 20\u201380",
    "Percentiles 20\u201390",
    "Percentiles 21\u201322",
    "Percentiles 22\u201323",
    "Percentiles 23\u201324",
    "Percentiles 24\u201325",
    "Percentiles 25\u201326",
    "Percentiles 26\u201327",
    "Percentiles 27\u201328",
    "Percentiles 28\u201329",
    "Percentiles 29\u201330",
    "Percentiles 2\u20133",
    "Top 70%",
    "Percentiles 30\u201331",
    "Percentiles 30\u201340",
    "Percentiles 30\u201350",
    "Percentiles 30\u201360",
    "Percentiles 30\u201370",
    "Percentiles 30\u201380",
    "Percentiles 30\u201390",
    "Percentiles 31\u201332",
    "Percentiles 32\u201333",
    "Percentiles 33\u201334",
    "Percentiles 34\u201335",
    "Percentiles 35\u201336",
    "Percentiles 36\u201337",
    "Percentiles 37\u201338",
    "Percentiles 38\u201339",
    "Percentiles 39\u201340",
    "Percentiles 3\u20134",
    "Top 60%",
    "Percentiles 40\u201341",
    "Percentiles 40\u201350",
    "Percentiles 40\u201360",
    "Percentiles 40\u201370",
    "Percentiles 40\u201380",
    "Percentiles 40\u201390",
    "Percentiles 41\u201342",
    "Percentiles 42\u201343",
    "Percentiles 43\u201344",
    "Percentiles 44\u201345",
    "Percentiles 45\u201346",
    "Percentiles 46\u201347",
    "Percentiles 47\u201348",
    "Percentiles 48\u201349",
    "Percentiles 49\u201350",
    "Percentiles 4\u20135",
    "Top 50%",
    "Percentiles 50\u201351",
    "Percentiles 50\u201360",
    "Percentiles 50\u201370",
    "Percentiles 50\u201380",
    "Percentiles 50\u201390",
    "Percentiles 51\u201352",
    "Percentiles 52\u201353",
    "Percentiles 53\u201354",
    "Percentiles 54\u201355",
    "Percentiles 55\u201356",
    "Percentiles 56\u201357",
    "Percentiles 57\u201358",
    "Percentiles 58\u201359",
    "Percentiles 59\u201360",
    "Percentiles 5\u20136",
    "Top 40%",
    "Percentiles 60\u201361",
    "Percentiles 60\u201370",
    "Percentiles 60\u201380",
    "Percentiles 60\u201390",
    "Percentiles 61\u201362",
    "Percentiles 62\u201363",
    "Percentiles 63\u201364",
    "Percentiles 64\u201365",
    "Percentiles 65\u201366",
    "Percentiles 66\u201367",
    "Percentiles 67\u201368",
    "Percentiles 68\u201369",
    "Percentiles 69\u201370",
    "Percentiles 6\u20137",
    "Top 30%",
    "Percentiles 70\u201371",
    "Percentiles 70\u201380",
    "Percentiles 70\u201390",
    "Percentiles 71\u201372",
    "Percentiles 72\u201373",
    "Percentiles 73\u201374",
    "Percentiles 74\u201375",
    "Percentiles 75\u201376",
    "Percentiles 76\u201377",
    "Percentiles 77\u201378",
    "Percentiles 78\u201379",
    "Percentiles 79\u201380",
    "Percentiles 7\u20138",
    "Top 20%",
    "Percentiles 80\u201381",
    "Percentiles 80\u201390",
    "Percentiles 81\u201382",
    "Percentiles 82\u201383",
    "Percentiles 83\u201384",
    "Percentiles 84\u201385",
    "Percentiles 85\u201386",
    "Percentiles 86\u201387",
    "Percentiles 87\u201388",
    "Percentiles 88\u201389",
    "Percentiles 89\u201390",
    "Percentiles 8\u20139",
    "Top 10%",
    "Percentiles 90\u201391",
    "Percentiles 91\u201392",
    "Percentiles 92\u201393",
    "Percentiles 93\u201394",
    "Percentiles 94\u201395",
    "Percentiles 95\u201396",
    "Percentiles 96\u201397",
    "Percentiles 97\u201398",
    "Percentiles 98\u201399",
    "Percentiles 99.1\u201399.2",
    "Percentiles 99.2\u201399.3",
    "Percentiles 99.3\u201399.4",
    "Percentiles 99.4\u201399.5",
    "Percentiles 99.5\u201399.6",
    "Percentiles 99.6\u201399.7",
    "Percentiles 99.7\u201399.8",
    "Percentiles 99.8\u201399.9",
    "Percentiles 99.9\u201399.91",
    "Percentiles 99.91\u201399.92",
    "Percentiles 99.92\u201399.93",
    "Percentiles 99.93\u201399.94",
    "Percentiles 99.94\u201399.95",
    "Percentiles 99.95\u201399.96",
    "Percentiles 99.96\u201399.97",
    "Percentiles 99.97\u201399.98",
    "Percentiles 99.98\u201399.99",
    "Percentiles 99.99\u201399.991",
    "Percentiles 99.991\u201399.992",
    "Percentiles 99.992\u201399.993",
    "Percentiles 99.993\u201399.994",
    "Percentiles 99.994\u201399.995",
    "Percentiles 99.995\u201399.996",
    "Percentiles 99.996\u201399.997",
    "Percentiles 99.997\u201399.998",
    "Percentiles 99.998\u201399.999",
    "Top 0.001%",
    "Top 0.01%",
    "Top 0.1%",
    "Top 1%",
    "Percentiles 9\u201310"
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})


#' @title Concept codes (\\S3-9)
#' @description Letters 2-6 of a WID variable code. Scraped from \\S3-6, 8-9 tables; \\S7 hardcoded from prose.
#' @source \url{https://wid.world/codes-dictionary/}
#' @format data.frame(code, description)
#' @export

wid_concepts <- local({
  code <- c(
    "cainc",
    "ccmhn",
    "ccmho",
    "ccshn",
    "ccsho",
    "ceuco",
    "ceugo",
    "ceuhn",
    "cfcco",
    "cfcfc",
    "cfcgo",
    "cfchn",
    "cfcho",
    "cfcnf",
    "cfcnp",
    "cfghg",
    "citgr",
    "colgo",
    "comhn",
    "comho",
    "comnp",
    "comnx",
    "compx",
    "comrx",
    "confc",
    "congo",
    "conhn",
    "conho",
    "connp",
    "cwagr",
    "cwbol",
    "cwboo",
    "cwbus",
    "cwcud",
    "cwdeb",
    "cwdeq",
    "cwdwe",
    "cweqi",
    "cwfie",
    "cwfin",
    "cwfiw",
    "cwhou",
    "cwlan",
    "cwnfa",
    "cwodk",
    "cwpen",
    "cwres",
    "defge",
    "defgo",
    "diinc",
    "ecoge",
    "ecogo",
    "edpge",
    "edsge",
    "edtge",
    "eduge",
    "edugo",
    "envge",
    "envgo",
    "expgo",
    "fainc",
    "fdinx",
    "fdipx",
    "fdirx",
    "fdixa",
    "fdixd",
    "fdixn",
    "ficap",
    "fidiv",
    "fiinc",
    "fiint",
    "fikgi",
    "filin",
    "fimik",
    "fimil",
    "fimix",
    "finpx",
    "finrx",
    "firen",
    "fiwag",
    "fkanx",
    "fkapx",
    "fkarx",
    "fkpin",
    "flcin",
    "flcip",
    "flcir",
    "flinc",
    "fosub",
    "fotax",
    "fpsub",
    "fptax",
    "fsubx",
    "ftaxx",
    "gdpro",
    "gfcar",
    "gfghg",
    "gfgho",
    "gicar",
    "gighg",
    "gigho",
    "gmxhn",
    "gmxho",
    "gpsge",
    "gpsgo",
    "gsmhn",
    "gsmho",
    "gsrco",
    "gsrfc",
    "gsrgo",
    "gsrhn",
    "gsrho",
    "gsrnf",
    "gsrnp",
    "gvaco",
    "gvago",
    "gvahn",
    "gvato",
    "gwagr",
    "gwbol",
    "gwbus",
    "gwcud",
    "gwdeb",
    "gwdec",
    "gwdwe",
    "gweal",
    "gweqi",
    "gwfie",
    "gwfin",
    "gwfiw",
    "gwhou",
    "gwlan",
    "gwnfa",
    "gwodk",
    "gwpen",
    "heage",
    "heago",
    "hfcar",
    "hfghd",
    "hfghg",
    "hfghn",
    "hfgho",
    "hicar",
    "highg",
    "higho",
    "houge",
    "hougo",
    "hwagr",
    "hwbol",
    "hwbus",
    "hwcud",
    "hwdeb",
    "hwdwe",
    "hweal",
    "hweqi",
    "hwfie",
    "hwfin",
    "hwfiw",
    "hwhou",
    "hwlan",
    "hwnfa",
    "hwodk",
    "hwpen",
    "icwtoq",
    "ifcar",
    "ifghg",
    "ifgho",
    "iicar",
    "iighg",
    "iigho",
    "indgo",
    "inpgo",
    "intgr",
    "intlcu",
    "inyixx",
    "iquali",
    "iwagr",
    "iwbol",
    "iwbus",
    "iwcud",
    "iwdeb",
    "iwdwe",
    "iweal",
    "iweqi",
    "iwfie",
    "iwfin",
    "iwfiw",
    "iwhou",
    "iwlan",
    "iwnfa",
    "iwodk",
    "iwpen",
    "ncanx",
    "ndpro",
    "necar",
    "neghg",
    "negho",
    "nfcar",
    "nfghg",
    "nfgho",
    "nicar",
    "nighg",
    "nigho",
    "nmxhn",
    "nmxho",
    "nncar",
    "nnfin",
    "nnghg",
    "nngho",
    "nninc",
    "npopem",
    "npopul",
    "nsmhn",
    "nsmho",
    "nsrco",
    "nsrfc",
    "nsrgo",
    "nsrhn",
    "nsrho",
    "nsrnf",
    "nsrnp",
    "ntaxad",
    "ntaxma",
    "ntaxre",
    "ntaxto",
    "ntcar",
    "ntcna",
    "ntghg",
    "ntgho",
    "ntgna",
    "ntona",
    "ntrgr",
    "nwagr",
    "nwboo",
    "nwbus",
    "nwdka",
    "nwdwe",
    "nweal",
    "nwgxa",
    "nwgxd",
    "nwhou",
    "nwlan",
    "nwnfa",
    "nwnxa",
    "nwodk",
    "ofcar",
    "ofghg",
    "ofgho",
    "oicar",
    "oighg",
    "oigho",
    "optxn",
    "ospgo",
    "othgo",
    "otpgp",
    "ottgr",
    "pfcar",
    "pfghg",
    "pinnx",
    "pinpx",
    "pinrx",
    "pitgr",
    "pkkin",
    "pllin",
    "polge",
    "polgo",
    "prgco",
    "prgfc",
    "prggo",
    "prghn",
    "prgho",
    "prgnf",
    "prgnp",
    "prico",
    "prifc",
    "prigo",
    "prihn",
    "priho",
    "prinf",
    "prinp",
    "prpco",
    "prpfc",
    "prpgo",
    "prphn",
    "prpho",
    "prpnf",
    "prpnp",
    "prtxn",
    "psugo",
    "ptdnx",
    "ptdpx",
    "ptdrx",
    "ptdxa",
    "ptdxd",
    "ptenx",
    "ptepx",
    "pterx",
    "ptexa",
    "ptexd",
    "ptfnx",
    "ptfpx",
    "ptfrn",
    "ptfrp",
    "ptfrr",
    "ptfrx",
    "ptfxa",
    "ptfxd",
    "ptfxn",
    "ptinc",
    "ptkin",
    "ptlin",
    "ptrrx",
    "ptrxa",
    "ptxgo",
    "pwagr",
    "pwbol",
    "pwbus",
    "pwcud",
    "pwdeb",
    "pwdwe",
    "pweal",
    "pweqi",
    "pwfie",
    "pwfin",
    "pwfiw",
    "pwhou",
    "pwlan",
    "pwnfa",
    "pwodk",
    "pwoff",
    "pwpen",
    "pwtgr",
    "recge",
    "recgo",
    "retgo",
    "revgo",
    "sacge",
    "sagco",
    "saggo",
    "saghn",
    "sagho",
    "sagnp",
    "sakge",
    "savco",
    "savgo",
    "savhn",
    "savho",
    "savig",
    "savin",
    "savnp",
    "scgnx",
    "scgpx",
    "scgrx",
    "scinx",
    "scipx",
    "scirx",
    "scogr",
    "sconx",
    "scopx",
    "scorx",
    "scrnx",
    "scrpx",
    "scrrx",
    "secco",
    "secfc",
    "secgo",
    "sechn",
    "secho",
    "secnf",
    "secnp",
    "segco",
    "segfc",
    "seggo",
    "seghn",
    "segho",
    "segnf",
    "segnp",
    "sopge",
    "sopgo",
    "spige",
    "spigo",
    "sprgo",
    "ssbco",
    "ssbfc",
    "ssbgo",
    "ssbhn",
    "ssbho",
    "ssbnf",
    "ssbnp",
    "sscco",
    "sscfc",
    "sscgo",
    "sschn",
    "sscho",
    "sscnf",
    "sscnp",
    "ssugo",
    "taxco",
    "taxfc",
    "taxgo",
    "taxhn",
    "taxho",
    "taxnf",
    "taxnp",
    "taxnx",
    "tbmpx",
    "tbnnx",
    "tbxrx",
    "tgmcx",
    "tgmmx",
    "tgmpx",
    "tgncx",
    "tgnmx",
    "tgnnx",
    "tgxcx",
    "tgxmx",
    "tgxrx",
    "tiwgo",
    "tiwhn",
    "tiwho",
    "tiwnp",
    "tpigo",
    "tprgo",
    "tsmpx",
    "tsnnx",
    "tsonx",
    "tsopx",
    "tsorx",
    "tstnx",
    "tstpx",
    "tstrx",
    "tsvnx",
    "tsvpx",
    "tsvrx",
    "tsxrx",
    "wcapsh",
    "wcsgni",
    "wcsnni",
    "wlabsh",
    "wlsgni",
    "wlsnni",
    "wwealc",
    "wwealg",
    "wwealh",
    "wweali",
    "wwealn",
    "wwealp",
    "xlceup",
    "xlceux",
    "xlcusp",
    "xlcusx",
    "xlcyup",
    "xlcyux",
    "xrereu",
    "xrerus",
    "xreryu",
    "ycscgv",
    "ycscnv",
    "ycsgdp",
    "ycsndp",
    "ylscgv",
    "ylscnv",
    "ylsgdp",
    "ylsndp"
  )
  description <- c(
    "(=) post-tax disposable income",
    "(+) CFC of mixed income of household sector",
    "(+) consumption of fixed capital attributable to mixed income",
    "(+) CFC of operating surplus of household sector",
    "(+) consumption of fixed capital attributable to operating surplus",
    "(+) compensation of employees of the corporate sector",
    "(+) compensation of employees of the government",
    "(+) compensation of employees\u00a0of household sector",
    "(+) CFC income of corporations",
    "(+) CFC of financial corporations",
    "(+) CFC of the general government",
    "(+) CFC of households and NPISH",
    "(+) CFC of households",
    "(+) CFC of non-financial corporations",
    "(+) CFC of NPISH",
    "(+) Personal carbon footprint (consumption only)",
    "(+) corporate income tax",
    "(+) collective consumption expenditure",
    "(+) compensation of employees",
    "(+) compensation of employees",
    "(+) compensation of employees",
    "(+) net foreign labor income",
    "(+) compensation of employees paid to the rest of the world",
    "(+) compensation of employees received from the rest of the world",
    "(-) consumption of fixed capital",
    "(-) final consumption expenditures",
    "(-) private expenditures of households and NPISH",
    "(-) private expenditures of households",
    "(-) private expenditures of NPISH",
    "(+) agricultural land",
    "(+) bonds and loans",
    "(=) book value of corporations",
    "(+) corporate business and other non-financial assets",
    "(+) currency and deposits",
    "(-) corporate debt (non-equity liability)",
    "(=) market value of corporations (equity liability)",
    "(+) dwellings",
    "(+) equities and fund shares",
    "(=) financial assets except for currency and deposits",
    "(+) financial assets",
    "(+) currency, deposits, bonds, and loans",
    "(+) corporate housing assets",
    "(+) land underlying dwellings",
    "(+) corporate non-financial assets",
    "(+) other domestic capital",
    "(+) pension funds and life insurance",
    "(+) residual corporate wealth",
    "(+) defense",
    "(+) defense",
    "(=) post-tax national income",
    "(+) economic affairs",
    "(+) economic affairs",
    "(+) education: Primary",
    "(+) education: Secondary",
    "(+) education: Tertiary",
    "(+) education",
    "(+) education",
    "(+) environment protection",
    "(+) environment protection",
    "(=) total public spending (excl. interest payment)",
    "(=) factor national income",
    "(+) net foreign direct investment income",
    "(+) foreign direct investment income",
    "(+) foreign direct investment income",
    "(+) Foreign direct investments assets",
    "(-) Foreign direct investments liabilities",
    "(+) Net foreign direct investment",
    "(+) fiscal capital income",
    "(+) dividends",
    "(=) fiscal income",
    "(+) interests",
    "(+) capital gains",
    "(+) fiscal labor income",
    "(+) capital component of mixed-income",
    "(+) labor component of mixed-income",
    "(=) mixed-income",
    "(-) foreign income paid to the rest of the world",
    "(+) foreign income received from the rest of the world",
    "(+) rents",
    "(+) wage and pensions",
    "(=) Capital Account",
    "(-) Capital transfers paid to the rest of the world",
    "(+) Capital transfers received from the rest of the world",
    "(+) net capital income",
    "(+) net foreign labor and capital income",
    "(+) labor and capital income paid to the rest of the world",
    "(+) labor and capital income from the rest of the world",
    "(=) labor factor income",
    "(+) other subsidies on production",
    "(+) other taxes on production",
    "(+) subsidies on products",
    "(+) taxes on products",
    "(+) subsidies on prod. received from the rest of the world",
    "(+) taxes on prod. paid to the rest of the world",
    "(+) gross domestic product",
    "(+) Government CO2 footprint",
    "(+) Government carbon footprint",
    "(+) Government footprint of other gases",
    "(+) Government imported CO2 emissions",
    "(+) Government imported emissions",
    "(+) Government imported emissions of other gases",
    "(+) gross mixed income of household sector",
    "(+) gross mixed income",
    "(+) general public services (excl. interest payment)",
    "(+) general public services",
    "(+) gross operating surplus and mixed income",
    "(+) gross operating surplus and mixed income",
    "(+) gross operating surplus of the corporate sector",
    "(+) gross operating surplus",
    "(+) gross operating surplus of the government",
    "(+) gross operating surplus of household sector",
    "(+) gross operating surplus",
    "(+) gross operating surplus",
    "(+) gross operating surplus and mixed income",
    "(+) gross value added of the corporate sector",
    "(+) gross value added of the government",
    "(+) gross value added of household sector",
    "(+) gross domestic product at factor-price",
    "(+) agricultural land",
    "(+) bonds and loans",
    "(+) government business and other non-financial assets",
    "(+) currency and deposits",
    "(-) liabilities",
    "(=) consolidated government debt",
    "(+) dwellings",
    "(=) net wealth of the general government",
    "(+) equities and fund shares",
    "(=) financial assets except for currency and deposits",
    "(+) financial assets",
    "(+) currency, deposits, bonds, and loans",
    "(+) government housing assets",
    "(+) land underlying dwellings",
    "(+) government non-financial assets",
    "(+) other domestic capital",
    "(+) pension funds and life insurance",
    "(+) health",
    "(+) health",
    "(+) Household indirect CO2 footprint",
    "(+) Household direct emissions",
    "(+) Household carbon footprint",
    "(+) Household indirect carbon footprint",
    "(+) Household indirect footprint of other gases",
    "(+) Household imported CO2 emissions",
    "(+) Household imported carbon emissions",
    "(+) Household imported emissions of other gases",
    "(+) housing and community amenities",
    "(+) housing and community amenities",
    "(+) agricultural land",
    "(+) bonds and loans",
    "(+) household business and other non-financial assets",
    "(+) currency and deposits",
    "(-) liabilities",
    "(+) dwellings",
    "(=) household net wealth",
    "(+) equities and fund shares",
    "(=) financial assets except for currency and deposits",
    "(+) financial assets",
    "(+) currency, deposits, bonds, and loans",
    "(+) household housing assets",
    "(+) land underlying dwellings",
    "(+) household non-financial assets",
    "(+) other domestic capital",
    "(+) pension funds and life insurance",
    "(*) Tobin Q (market value of corporations/book value of corporations)",
    "(+) Investment CO2 footprint",
    "(+) Investment carbon footprint",
    "(+) Investment footprint of other gases",
    "(+) Investment imported CO2 emissions",
    "(+) Investment imported carbon emissions",
    "(+) Investment imported emissions of other gases",
    "(+) individual consumption expenditures",
    "(-) interest paid by the government",
    "(+) indirect taxes",
    "Local currency unit change indicator",
    "Consumer price index / national income deflator",
    "Inequality transparency index",
    "(+) agricultural land",
    "(+) bonds and loans",
    "(+) non-profit institutions\u2019 business and other non-financial assets",
    "(+) currency and deposits",
    "(-) liabilities",
    "(+) dwellings",
    "(=) non-profit institutions\u2019 net wealth",
    "(+) equities and fund shares",
    "(=) financial assets except for currency and deposits",
    "(+) financial assets",
    "(+) currency, deposits, bonds, and loans",
    "(+) non-profit institutions\u2019 housing assets",
    "(+) land underlying dwellings",
    "(+) non-profit institutions\u2019 non-financial assets",
    "(+) other domestic capital",
    "(+) pension funds and life insurance",
    "(=) Current Account = pinnx + comnx + tbnnx + taxnx + scinx",
    "(+) net domestic product",
    "(+) National exported CO2 emissions",
    "(=) National exported carbon emissions",
    "(+) National exported emissions of other gases",
    "(+) National CO2 footprint",
    "(=) National carbon footprint",
    "(+) National footprint of other gases",
    "(+) National imported CO2 emissions",
    "(=) National imported carbon emissions",
    "(+) National imported emissions of other gases",
    "(+) net mixed income of household sector",
    "(+) net mixed income of households",
    "(+) National net imports of CO2 emissions",
    "(+) foreign income",
    "(=) National net imports of carbon emissions",
    "(+) National net imports of other gases",
    "(=) net national income",
    "employed population",
    "population",
    "(+) net operating surplus and mixed income",
    "(+) net operating surplus and mixed income",
    "(+) net operating surplus of the corporate sector",
    "(+) net operating surplus",
    "(+) Net operating surplus of the government (=0)",
    "(+) net operating surplus of the households and NPISH",
    "(+) net operating surplus",
    "(+) net operating surplus",
    "(+) net operating surplus",
    "number of tax units \u201a adults",
    "number of tax units \u201a married couples & single adults",
    "number of tax returns",
    "total tax population",
    "(+) National territorial CO2 emissions",
    "(+) Territorial emissions of the national productive sector and other emissions not attributed to households (CO2)",
    "(=) National territorial carbon emissions",
    "(+) National territorial emissions of other gases",
    "(+) Territorial emissions of the national productive sector and other emissions not attributed to households",
    "(+) Territorial emissions of the national productive sector and other emissions not attributed to households (Other gases)",
    "(+) non-tax revenue",
    "(+) agricultural land",
    "(=) book-value national wealth",
    "(+) national business and other non-financial assets",
    "(=) domestic capital",
    "(+) dwellings",
    "(=) net market-value national wealth",
    "(+) gross foreign assets",
    "(-) gross foreign liabilities",
    "(+) national housing assets",
    "(+) land underlying dwellings",
    "(+) national non-financial assets",
    "(=) Net foreign assets",
    "(+) other domestic capital",
    "(+) NGO CO2 footprint",
    "(+) NGO carbon footprint",
    "(+) NGO footprint of other gases",
    "(+) NGO imported CO2 emissions",
    "(+) NGO imported carbon emissions",
    "(+) NGO imported emissions of other gases",
    "(+) other subsidies less taxes on production",
    "(+) other subsidies on production",
    "(+) other government spending",
    "(+) other taxes on production",
    "(+) other taxes",
    "(=) Personal CO2 footprint",
    "(=) Personal carbon footprint",
    "(+) net foreign capital income",
    "(+) property income paid to the rest of the world",
    "(+) property income received from the rest of the world",
    "(+) personal income tax",
    "(+) pretax capital income",
    "(+) pretax labor income",
    "(+) public order and safety",
    "(+) public order and safety",
    "(=) gross primary income of corporations",
    "(=) gross primary income of financial corporations",
    "(=) gross primary income of the general government",
    "(=) gross primary income of households and NPISH",
    "(=) gross primary income of households",
    "(=) gross primary income of non-financial corporations",
    "(=) gross primary income of NPISH",
    "(+) net primary income of corporations",
    "(+) net primary income of financial corporations",
    "(+) net primary income of the general government",
    "(+) net primary income of households and non-profits",
    "(+) net primary income of households",
    "(+) net primary income of non-financial corporations",
    "(+) net primary income of non-profits",
    "(+) property income of corporations",
    "(+) property income (net)",
    "(+) property income of the government",
    "(+) property income of households and NPISH",
    "(+) property income (net)",
    "(+) property income (net)",
    "(+) property income (net)",
    "(+) subsidies less taxes on products",
    "(=) primary surplus of the government",
    "(+) net debt income",
    "(+) debt income",
    "(+) debt income",
    "(+) Portfolio debt assets",
    "(+) Portfolio debt liabilities",
    "(+) net equity income",
    "(+) equity income",
    "(+) equity income",
    "(+) Portfolio equity assets",
    "(+) Portfolio equity liabilities",
    "(+) net portfolio income",
    "(+) portfolio and other income",
    "(+) net reinvested earnings on foreign portfolio investment",
    "(+) reinvested earnings on foreign portfolio investment",
    "(+) reinvested earnings on foreign portfolio investment",
    "(+) portfolio and other income",
    "(+) Portfolio assets",
    "(-) Portfolio liabilities",
    "(+) Net foreign portfolio",
    "(=) pretax national income",
    "(+) pretax capital income",
    "(+) pretax labor income",
    "(+) reserves income",
    "(+) Foreign reserve exchange assets",
    "(+) taxes on products and production",
    "(+) agricultural land",
    "(+) bonds and loans",
    "(+) private business and other non-financial assets",
    "(+) currency and deposits",
    "(-) liabilities",
    "(+) dwellings",
    "(=) private net wealth",
    "(+) equities and fund shares",
    "(=) financial assets except for currency and deposits",
    "(+) financial assets",
    "(+) currency, deposits, bonds, and loans",
    "(+) private housing assets",
    "(+) land underlying dwellings",
    "(+) private non-financial assets",
    "(+) other domestic capital",
    "(+) offshore wealth",
    "(+) pension funds and life insurance",
    "(+) property and wealth taxes",
    "(+) recreation, culture and religion",
    "(+) recreation, culture and religion",
    "(=) total public revenue (excl. non-tax revenue)",
    "(=) total public revenue",
    "(+) social protection: social assistance in cash",
    "(+) gross savings/secondary income of corporations",
    "(+) gross savings of the general government",
    "(+) gross savings of households and NPISH",
    "(+) gross savings of households",
    "(+) gross savings of NPISH",
    "(+) social protection: social assistance in kind",
    "(+) net savings/secondary income of corporations",
    "(+) net savings of the general government",
    "(+) net savings of households and NPISH",
    "(+) net savings of households",
    "(=) gross savings of the total economy",
    "(=) net savings of the total economy",
    "(+) net savings of NPISH",
    "(+) Net public foreign transfers",
    "(-) Public foreign transfers paid to the rest of the world",
    "(+) Public foreign transfers received from the rest of the world",
    "(+) Net foreign transfers",
    "(-) Foreign transfers paid to the rest of the world",
    "(+) Foreign transfers received from the rest of the world",
    "(+) social contributions",
    "(+) Net other foreign transfers",
    "(-) Other foreign transfers paid to the rest of the world",
    "(+) Other foreign transfers received from the rest of the world",
    "(+) Net remittances",
    "(-) Remittances paid to the rest of the world",
    "(+) Remittances received from the rest of the world",
    "(+) net secondary income of corporations",
    "(+) net secondary income of financial corporations",
    "(+) net secondary income of the general government",
    "(+) net secondary income of households and non-profits",
    "(+) net secondary income of households",
    "(+) net secondary income of non-financial corporations",
    "(+) net secondary income of non-profits",
    "(=) gross secondary income/gross saving of corporations",
    "(+) gross savings of financial corporations",
    "(=) gross secondary income of the general government",
    "(=) gross secondary income of households and NPISH",
    "(=) gross secondary income of households",
    "(+) gross savings of non-financial corporations",
    "(=) gross secondary income of NPISH",
    "(+) social protection",
    "(+) social protection",
    "(+) social protection: social insurance",
    "(-) subsidies on production and imports",
    "(+) subsidies on products",
    "(+) social benefits from private employer social insurance",
    "(-) social benefits from private employer social insurance",
    "(+) social benefits paid by the government",
    "(=) social benefits received by households and NPISH",
    "(+) social benefits other than social transfers in kind",
    "(-) social benefits from private employer social insurance",
    "(+) social benefits other than social transfers in kind",
    "(+) social contributions to private employer social insurance",
    "(+) social contributions to private employer social insurance",
    "(+) social contributions received by the government",
    "(=) social contributions paid by households and NPISH",
    "(+) social contributions",
    "(+) social contributions to private employer social insurance",
    "(+) social contributions",
    "(=) secondary surplus of the government",
    "(+) corporate tax",
    "(-) corporate tax",
    "(=) direct taxes received by the government",
    "(-) direct taxes",
    "(-) direct taxes",
    "(-) corporate tax",
    "(-) direct taxes",
    "(+) subsidies less taxes on production and imports",
    "(-) Imports of goods and services",
    "(+) Trade balance (exports-imports)",
    "(+) Exports of goods and services",
    "(-) Imports of primary commodities",
    "(-) Imports of manufacturing goods",
    "(-) Imports of goods",
    "(+) Trade balance of primary commodities (exports-imports)",
    "(+) Trade balance of manufacturing goods (exports-imports)",
    "(+) Trade balance of goods (exports-imports)",
    "(+) Exports of primary commodities",
    "(+) Exports of manufacturing goods",
    "(+) Exports of goods",
    "(+) direct taxes on income and wealth",
    "(+) taxes on income and wealth paid by households",
    "(+) personal taxes on income and wealth",
    "(+) personal taxes on income and wealth",
    "(+) taxes on production and imports",
    "(+) taxes on products",
    "(-) Imports of services",
    "(+) Trade balance of services (exports-imports)",
    "(+) Trade balance\u00a0 of other services (exports-imports)",
    "(-) Imports of other services",
    "(+) Exports of other services",
    "(+) Trade balance of transport services (exports-imports)",
    "(-) Imports of transport services",
    "(+) Exports of transport services",
    "(+) Trade balance of travel services (exports-imports)",
    "(-) Imports of travel services",
    "(+) Exports of travel services",
    "(+) Exports of services",
    "Capital share of national income",
    "Capital share of total gross national income at factor-price",
    "Capital share of total national income at factor-price",
    "Labor share of national income",
    "Labor share of total gross national income at factor-price",
    "Labor share of total national income at factor-price",
    "Corporate wealth-to-income ratio",
    "Government wealth-to-income ratio",
    "Household wealth-to-income ratio",
    "Institutional wealth-to-income ratio",
    "Net national wealth-to-income ratio",
    "Private wealth-to-income ratio",
    "PPP exchange rate LCU/EUR",
    "Market exchange rate LCU/EUR",
    "PPP exchange rate LCU/USD",
    "Market exchange rate LCU/USD",
    "PPP exchange rate LCU/CNY",
    "Market exchange rate LCU/CNY",
    "Real exchange rate (EUR base)",
    "Real exchange rate (USD base)",
    "Real exchange rate (CNY base)",
    "Capital share in corporate gross value added at factor-price",
    "Capital share in corporate net\u00a0 value added at factor-price",
    "Capital share of total gross domesic product at factor-price",
    "Capital share of total net domestic product at factor-price",
    "Labor share in corporate gross value added at factor-price",
    "Labor share in corporate net\u00a0 value added at factor-price",
    "Labor share of total gross domestic product at factor-price",
    "Labor share of total net domestic product at factor-price"
  )
  stopifnot(length(code) == length(description))
  data.frame(code=code, description=description, stringsAsFactors=FALSE)
})

# Visualisation helpers for wid_df objects

.PAL <- c("#58a6ff", "#3fb950", "#d2a8ff", "#ffa657", "#ff7b72",
          "#79c0ff", "#56d364", "#e3b341")

.theme_wid <- function() theme(
  plot.background   = element_rect(fill = "#0d1117", color = NA),
  panel.background  = element_rect(fill = "#0d1117", color = NA),
  panel.grid.major  = element_line(color = "#21262d"),
  panel.grid.minor  = element_blank(),
  axis.text         = element_text(color = "#8b949e"),
  axis.title        = element_text(color = "#e6edf3"),
  plot.title        = element_text(color = "#e6edf3", face = "bold"),
  plot.subtitle     = element_text(color = "#8b949e"),
  legend.background = element_rect(fill = "#161b22", color = NA),
  legend.text       = element_text(color = "#e6edf3"),
  legend.title      = element_text(color = "#8b949e"),
  strip.background  = element_rect(fill = "#161b22"),
  strip.text        = element_text(color = "#e6edf3")
)

.prep <- function(data, variable, country, country_labels) {
  .check(data)
  variable <- .resolve_var(data, variable)
  d <- data[data$variable == variable, ]
  if (!is.null(country)) d <- d[d$country %in% country, ]
  if (!nrow(d)) stop("No data after filtering.", call. = FALSE)
  if (!is.null(country_labels)) d$country <- .recode(d$country, country_labels)
  list(d = d, v = variable)
}

#' Time-series line chart for WID data
#'
#' @param data           A \code{wid_df}.
#' @param variable       Variable code to plot.
#' @param country        Country codes to include.
#' @param country_labels Named character vector for renaming countries.
#' @param facet          Logical. Facet by country. Default \code{FALSE}.
#' @return A \code{ggplot} object.
#' @examples
#' \donttest{
#'   d <- data.frame(
#'     country = rep(c("US", "FR"), each = 3L),
#'     year    = rep(c("2018", "2019", "2020"), 2L),
#'     variable = "sptinc992j", percentile = "p99p100",
#'     value = c(0.20, 0.19, 0.19, 0.11, 0.11, 0.12),
#'     age = "992", pop = "j", stringsAsFactors = FALSE
#'   )
#'   class(d) <- c("wid_df", "data.frame")
#'   wid_plot_timeseries(d)
#' }
#' @export
wid_plot_timeseries <- function(data, variable = NULL, country = NULL,
                                country_labels = NULL, facet = FALSE) {
  p <- .prep(data, variable, country, country_labels)
  g <- ggplot(p$d, aes(year, value, color = country)) +
    geom_line(linewidth = 0.8) +
    geom_point(size = 1.5) +
    scale_color_manual(values = .PAL) +
    labs(title = .auto_title(p$v, unique(p$d$country)),
         x = "Year", y = "Value", color = NULL) +
    .theme_wid()
  if (facet) g + facet_wrap(~country) else g
}

#' Cross-country comparison bar/point chart
#'
#' @param data           A \code{wid_df}.
#' @param year           Integer year (defaults to most recent common year).
#' @param variable       Variable code to plot.
#' @param country        Country codes to include.
#' @param country_labels Named character vector for renaming countries.
#' @param type           \code{"bar"} (default) or \code{"point"}.
#' @return A \code{ggplot} object.
#' @examples
#' \donttest{
#'   d <- data.frame(
#'     country = rep(c("US", "FR"), each = 3L),
#'     year    = rep(c("2018", "2019", "2020"), 2L),
#'     variable = "sptinc992j", percentile = "p99p100",
#'     value = c(0.20, 0.19, 0.19, 0.11, 0.11, 0.12),
#'     age = "992", pop = "j", stringsAsFactors = FALSE
#'   )
#'   class(d) <- c("wid_df", "data.frame")
#'   wid_plot_compare(d, year = 2020)
#' }
#' @export
wid_plot_compare <- function(data, year = NULL, variable = NULL, country = NULL,
                             country_labels = NULL, type = "bar") {
  p   <- .prep(data, variable, country, country_labels)
  yr  <- year %||% .common_year(p$d)
  d   <- p$d[p$d$year == yr, ]
  if (!nrow(d)) stop("No data for year ", yr, ".", call. = FALSE)
  agg <- tapply(d$value, d$country, mean, na.rm = TRUE)
  d$country <- factor(d$country, levels = names(sort(agg, decreasing = TRUE)))
  g <- ggplot(d, aes(country, value)) +
    labs(title = paste0(.auto_title(p$v, levels(d$country)), " (", yr, ")"),
         x = NULL, y = "Value") +
    .theme_wid() +
    theme(legend.position = "none")
  if (type == "bar")
    g + geom_col(aes(fill = country)) + scale_fill_manual(values = .PAL)
  else
    g + geom_point(size = 3, color = .PAL[1L])
}

#' Lorenz curve
#'
#' @param data           A \code{wid_df} with share (\code{"s"}) series.
#' @param variable       Variable code to plot.
#' @param country        Country codes to include.
#' @param country_labels Named character vector for renaming countries.
#' @return A \code{ggplot} object.
#' @examples
#' \donttest{
#'   d <- data.frame(
#'     country = rep("US", 3L), year = rep("2020", 3L),
#'     variable = "sptinc992j",
#'     percentile = c("p0p50", "p50p90", "p90p100"),
#'     value = c(0.20, 0.40, 0.40), age = "992", pop = "j",
#'     stringsAsFactors = FALSE
#'   )
#'   class(d) <- c("wid_df", "data.frame")
#'   wid_plot_lorenz(d)
#' }
#' @export
wid_plot_lorenz <- function(data, variable = NULL, country = NULL,
                            country_labels = NULL) {
  .check(data, series_type = "s")
  p <- .prep(data, variable, country, country_labels)
  d <- p$d
  
  bounds <- lapply(d$percentile, .parse_perc)
  ok     <- !vapply(bounds, is.null, logical(1L))
  if (any(!ok))
    warning(sum(!ok), " row(s) dropped: unparseable percentile codes.", call. = FALSE)
  d     <- d[ok, ]
  d$hi  <- vapply(bounds[ok], `[[`, numeric(1L), "hi")
  d     <- do.call(rbind, lapply(split(d, d$country), function(g) {
    g <- g[order(g$hi), ]
    g$cum_share <- cumsum(g$value)
    g
  }))
  
  ggplot(d, aes(hi, cum_share, color = country)) +
    geom_abline(slope = 1, intercept = 0, linetype = "dashed",
                color = "#8b949e", linewidth = 0.5) +
    geom_line(linewidth = 0.9) +
    scale_color_manual(values = .PAL) +
    scale_x_continuous(labels = percent_format()) +
    scale_y_continuous(labels = percent_format()) +
    labs(title = "Lorenz Curve", x = "Population share",
         y = "Income/wealth share", color = NULL) +
    .theme_wid()
}

# ---- .prep -----------------------------------------------------------------

test_that(".prep errors on non-wid_df", {
  expect_error(widr:::.prep(data.frame(), NULL, NULL, NULL), "wid_df")
})

test_that(".prep errors when no rows remain after filtering", {
  expect_error(widr:::.prep(.wid(), NULL, "XX", NULL), "No data")
})

test_that(".prep applies country_labels recode", {
  p <- widr:::.prep(.wid(), NULL, NULL, c(US = "United States"))
  expect_true(any(p$d$country == "United States"))
})

test_that(".prep restricts to requested country", {
  d <- .wid(countries = c("US","FR"))
  p <- widr:::.prep(d, NULL, "US", NULL)
  expect_true(all(p$d$country == "US"))
})

# ---- .theme_wid ------------------------------------------------------------

test_that(".theme_wid returns a ggplot theme with dark background", {
  t <- widr:::.theme_wid()
  expect_s3_class(t, "theme")
  expect_equal(t$plot.background$fill, "#0d1117")
})

# ---- wid_plot_timeseries ---------------------------------------------------

test_that("wid_plot_timeseries returns ggplot", {
  d <- .wid(5); d$year <- 2016:2020
  expect_s3_class(wid_plot_timeseries(d), "ggplot")
})

test_that("wid_plot_timeseries applies dark theme", {
  d <- .wid(3); d$year <- 2018:2020
  p <- wid_plot_timeseries(d)
  expect_equal(p$theme$plot.background$fill, "#0d1117")
})

test_that("wid_plot_timeseries applies country_labels", {
  d <- .wid(3); d$year <- 2018:2020
  p <- wid_plot_timeseries(d, country_labels = c(US = "United States"))
  expect_true(any(p$data$country == "United States"))
})

test_that("wid_plot_timeseries facet=TRUE adds FacetWrap", {
  d <- .wid(5, countries = c("US","FR")); d$year <- rep(2016:2020, 2L)
  p <- wid_plot_timeseries(d, facet = TRUE)
  expect_true(inherits(p$facet, "FacetWrap"))
})

test_that("wid_plot_timeseries facet=FALSE has no FacetWrap", {
  d <- .wid(5); d$year <- 2016:2020
  p <- wid_plot_timeseries(d, facet = FALSE)
  expect_false(inherits(p$facet, "FacetWrap"))
})

# ---- wid_plot_compare ------------------------------------------------------

test_that("wid_plot_compare returns ggplot", {
  d <- .wid(2, countries = c("US","FR"))
  expect_s3_class(wid_plot_compare(d, year = 2020), "ggplot")
})

test_that("wid_plot_compare applies dark theme", {
  d <- .wid(2, countries = c("US","FR"))
  p <- wid_plot_compare(d, year = 2020)
  expect_equal(p$theme$plot.background$fill, "#0d1117")
})

test_that("wid_plot_compare type='point' returns ggplot", {
  d <- .wid(2, countries = c("US","FR"))
  expect_s3_class(wid_plot_compare(d, year = 2020, type = "point"), "ggplot")
})

test_that("wid_plot_compare errors when year not in data", {
  d <- .wid(2, countries = c("US","FR"))
  expect_error(wid_plot_compare(d, year = 1800), "No data for year")
})

test_that("wid_plot_compare uses most recent common year when year=NULL", {
  d <- .wid(2, countries = c("US","FR"))
  p <- wid_plot_compare(d)
  expect_s3_class(p, "ggplot")
})

# ---- wid_plot_lorenz -------------------------------------------------------

test_that("wid_plot_lorenz returns ggplot", {
  expect_s3_class(wid_plot_lorenz(.wid_dist()), "ggplot")
})

test_that("wid_plot_lorenz errors on non-wid_df", {
  expect_error(wid_plot_lorenz(data.frame()), "wid_df")
})

test_that("wid_plot_lorenz errors on non-share series", {
  expect_error(wid_plot_lorenz(.wid(type = "t")), "share")
})

test_that("wid_plot_lorenz errors on empty wid_df", {
  expect_error(wid_plot_lorenz(widr:::.wid_df()), "empty")
})

test_that("wid_plot_lorenz warns and drops unparseable percentile rows", {
  d              <- .wid_dist()
  d$percentile[1L] <- "bad"
  expect_warning(out <- wid_plot_lorenz(d), "dropped")
  expect_s3_class(out, "ggplot")
})

test_that("wid_plot_lorenz applies dark theme", {
  p <- wid_plot_lorenz(.wid_dist())
  expect_equal(p$theme$plot.background$fill, "#0d1117")
})

test_that("wid_plot_lorenz applies country_labels", {
  d <- rbind(.wid_dist("US"), .wid_dist("FR"))
  p <- wid_plot_lorenz(d, country_labels = c(US = "United States"))
  expect_true(any(p$data$country == "United States"))
})

library(widr)

# ── .wid() ────────────────────────────────────────────────────────────────────
# Returns a wid_df with n rows PER country.
#   n         : rows per country (default 2; ensures resolve_var multi-var test works)
#   type      : series-type letter   (default "s")
#   countries : country codes        (default "US")
#
# Year sequence starts at 2020 and increments by 1 for each row within a country,
# so all countries share the same set of years — required by wid_plot_compare.
#
# Examples:
#   .wid()                       → 2 rows, US, years 2020–2021
#   .wid(5)                      → 5 rows, US, years 2020–2024
#   .wid(3, type = "a")          → 3 rows, US, type "a"
#   .wid(5, countries = c("US","FR"))  → 10 rows, 5 per country, years 2020–2024
.wid <- function(n = 2L, type = "s", countries = "US") {
  yrs <- as.character(seq(2020L, length.out = n))
  df  <- data.frame(
    country    = rep(countries, each = n),
    variable   = paste0(type, "ptinc992j"),
    percentile = "p99p100",
    year       = rep(yrs, times = length(countries)),
    value      = seq(0.2, length.out = n * length(countries), by = 0.01),
    age        = "992",
    pop        = "j",
    stringsAsFactors = FALSE
  )
  widr:::new_wid_df(df)
}

# ── .wid_dist() ───────────────────────────────────────────────────────────────
# share-series wid_df with three percentile bands for one country and one year.
# Defaults: country="US", years=2020L  →  exactly 3 rows.
#
# Value layout:  p0p50 → 0.50 | p50p90 → 0.10 | p90p100 → 0.40
# wid_top_share(.wid_dist(), top=0.1)  →  share = 0.40, top = 0.1  (scalar)
#
# Build multi-country/year data by calling repeatedly:
#   rbind(.wid_dist("US"), .wid_dist("FR"))
#   rbind(.wid_dist("US", 2019L), .wid_dist("US", 2020L))
.wid_dist <- function(country = "US", years = 2020L) {
  bands <- list(c("p0p50", 0.50), c("p50p90", 0.10), c("p90p100", 0.40))
  rows  <- do.call(rbind, lapply(years, function(yr)
    do.call(rbind, lapply(bands, function(b)
      data.frame(
        country    = country,
        variable   = "sptinc992j",
        percentile = b[[1L]],
        year       = as.character(yr),
        value      = as.numeric(b[[2L]]),
        age        = "992",
        pop        = "j",
        stringsAsFactors = FALSE
      )))))
  widr:::new_wid_df(rows)
}

# ── .wid_t() ──────────────────────────────────────────────────────────────────
# threshold-series wid_df with p10 / p50 / p90 rows.
# P90/P10 = 200000/10000 = 20;   P90/P50 = 200000/50000 = 4.
.wid_t <- function(country = "US", year = "2020") {
  widr:::new_wid_df(data.frame(
    country    = country,
    variable   = "tptinc992j",
    percentile = c("p10", "p50", "p90"),
    year       = year,
    value      = c(10000, 50000, 200000),
    age        = "992",
    pop        = "j",
    stringsAsFactors = FALSE
  ))
}

---
title: "WID Codes Dictionary"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{WID Codes Dictionary}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
library(widr)
```

This vignette mirrors the official [WID Codes Dictionary](https://wid.world/codes-dictionary/) section by section, with `widr` function calls woven in wherever a function applies. For methodology and estimation, see the [DINA Guidelines](https://wid.world/document/distributional-national-accounts-dina-guidelines-2025-methods-and-concepts-used-in-the-world-inequality-database/).

---

## 1. General Presentation

`widr` provides direct R access to the WID API. The primary interface is `download_wid()`, which returns a `wid_df` -- a classed `data.frame` compatible with dplyr, ggplot2, and base R.

```{r download-sig}
download_wid(
  indicators             = "all",
  areas                  = "all",
  years                  = "all",
  perc                   = "all",
  ages                   = "992",   # default: adults 20+
  pop                    = "j",     # default: equal-split
  metadata               = FALSE,
  include_extrapolations = TRUE,
  verbose                = FALSE,
  cache                  = TRUE)
```

`metadata = TRUE` attaches source documentation as `attr(result, "wid_meta")`. Cache responses are managed with `wid_cache()`, `wid_cache_list()`, and `wid_cache_clear()`. Set an API key with `wid_set_key()`.

---

## 2. General Structure

Every series has a WID code (concept), a percentile code, a country/region code, and a year. Codes are built by combining series type, concept, age group, and population unit.

### 2.1 WID Code Examples

The three canonical examples from the official dictionary:

| Type | Concept | Age | Pop | Percentile | Country | Year | Meaning |
|------|---------|-----|-----|-----------|---------|------|---------|
| `s` | `ptinc` | `992` | `j` | `p99p100` | `US` | `1970` | share of pretax national income, top 1%, equal-split adults, United States |
| `l` | `pfghg` | `999` | `i` | `p90p100` | `CH` | `2014` | avg. per capita group emissions, personal carbon footprint, top 10%, Switzerland |
| `s` | `pllin` | `992` | `f` | `p0p100` | `UY` | `2023` | share of pretax labour income, female, entire adult population, Uruguay |

```{r decode-encode}
wid_decode("sptinc992j")
#> $series_type  "s"     $concept  "ptinc"     $age  "992"     $pop  "j"

wid_encode("s", "ptinc", "992", "j")       #> [1] "sptinc992j"
wid_encode("m", "nninc")                   #> [1] "mnninc"
wid_encode(wid_decode("sptinc992j"))       # round-trip: identical to input

# Validate (throws on failure) or check silently
wid_validate(series_type = "s", concept = "ptinc", age = 992, pop = "j")
wid_is_valid(series_type = "s", concept = "ptinc")   #> [1] TRUE
wid_is_valid(series_type = "Z")                       #> [1] FALSE
```

### 2.1.1 Series Type (one-letter code)

There are `r nrow(wid_series_types)` types.

```{r series-types}
wid_series_types
wid_search("share", tables = "series_types")
```

| Code | Meaning | Unit |
|------|---------|------|
| `a` | average | local currency, last year's prices |
| `b` | inverted Pareto-Lorenz coefficient | no unit |
| `e` | total emissions | tonnes CO2-equivalent |
| `f` | female population | fraction (0 to 1) |
| `g` | Gini coefficient | 0 to 1 |
| `i` | index | no unit |
| `k` | per capita emissions | tonnes CO2-equivalent |
| `l` | average per capita group emissions | tonnes CO2-equivalent |
| `m` | total | local currency, last year's prices |
| `n` | population | people |
| `p` | proportion of women | fraction (0 to 1) |
| `r` | top 10 / bottom 50 ratio | no unit |
| `s` | share | fraction (0 to 1) |
| `t` | threshold | local currency, last year's prices |
| `w` | wealth-to-income ratio or labor/capital share | fraction of national income |
| `x` | exchange rate (market or PPP) | local currency per foreign currency |
| `y` | wealth-to-GDP ratio | fraction of GDP |

All monetary amounts are in local currency at constant prices. For world regions: `XX-MER` (USD MER) or `XX-PPP` (USD PPP). Shares and ratios are fractions: a top-1% share of 20% = `0.2`; a 300% wealth/income ratio = `3`. Series types `s`, `g`, `b`, `w`, `r`, `p`, `i`, `y` are dimensionless; `wid_convert()` passes them through unchanged.

### 2.1.2 Concept (five-letter code)

Characters 2-6 (or 2-7 for six-letter concepts). There are `r nrow(wid_concepts)` concepts.

```{r concepts}
nrow(wid_concepts)
head(wid_concepts)
wid_search("wealth")
wid_search("^ptinc$", tables = "concepts")
wid_search("wealth", type = "s")
wid_search("income", tables = "all")
```

### 2.1.3 Age Group (three-digit code)

There are `r nrow(wid_ages)` defined age groups. `anninc999i` = national income per capita; `anninc992i` = national income per adult.

```{r ages}
wid_ages
wid_validate(age = 992)
```

| Code | Population |
|------|-----------|
| `999` | All ages |
| `992` | Adults 20+ *(default)* |
| `996` | Working-age, 20-64 |
| `995` | Adults 60+ |
| `994` | Adults 40-59 |
| `993` | Adults 20-39 |
| `997` | Elderly 65+ |
| `998` | Aged 80+ |
| `991` | Below 20 |
| `014` | Ages 0-14 |
| `156` | Ages 15-64 |
| `001` | Ages 0-4 |
| `051` | Ages 5-9 |
| `101` | Ages 10-14 |
| `151` | Ages 15-19 |
| `201` | Ages 20-24 |
| `251` | Ages 25-29 |
| `301` | Ages 30-34 |
| `351` | Ages 35-39 |
| `401` | Ages 40-44 |
| `451` | Ages 45-49 |
| `501` | Ages 50-54 |
| `551` | Ages 55-59 |
| `601` | Ages 60-64 |
| `651` | Ages 65-69 |
| `701` | Ages 70-74 |
| `751` | Ages 75-79 |
| `801` | Ages 80-84 |
| `851` | Ages 85-89 |
| `901` | Ages 90-94 |
| `951` | Ages 95-99 |
| `111` | Ages 99+ |
| `202` | Ages 20-29 |
| `302` | Ages 30-39 |
| `402` | Ages 40-49 |
| `502` | Ages 50-59 |
| `602` | Ages 60-69 |
| `702` | Ages 70-79 |
| `802` | Ages 80-89 |
| `902` | Ages 90-99 |

### 2.1.4 Population Unit (one-letter code)

There are `r nrow(wid_pop_types)` population unit codes. Only affects distributional series; aggregate series default to `i`.

```{r pop-types}
wid_pop_types
```

| Code | Description |
|------|------------|
| `i` | individuals |
| `j` | equal-split adults *(default)* |
| `m` | male |
| `f` | female |
| `t` | tax unit |
| `e` | employed |

### 2.1.5 Series Type x Population Unit Availability

```{r is-valid}
wid_is_valid(series_type = "s", concept = "ptinc")   # TRUE
wid_is_valid(series_type = "b", pop = "f")            # FALSE
```

| Series type | `f: female` | `i: individuals` | `j: equal-split adults` | `m: male` | `t: tax unit` |
|-------------|:---:|:---:|:---:|:---:|:---:|
| `a` average | + | + | + | + | + |
| `b` Pareto coef. | | + | + | | |
| `f` female pop. | | + | | | |
| `i` index | | + | | | |
| `m` total | | + | | | + |
| `n` population | + | + | | + | + |
| `p` prop. women | | + | | | |
| `s` share | + | + | + | + | + |
| `t` threshold | + | + | + | + | + |
| `w` wealth/income ratio | | + | | | + |
| `y` wealth-to-GDP ratio | | + | | | + |
| `x` exchange rate | | + | | | |
| `e`/`k`/`l` emissions | | + | | | |

---

## 2.2 Country Codes

There are `r nrow(wid_countries)` entries in `wid_countries`.

```{r countries}
head(wid_countries)
wid_search("^US", tables = "countries")
wid_validate(areas = c("US", "FR", "US-CA"))
```

### Country Codes

| WID code | Country | World region | Sub-division | Historical series | Sub-division of historical |
|----------|---------|-------------|-------------|------------------|---------------------------|
| AD | Andorra | Europe | | Other Western Europe | |
| AE | United Arab Emirates | MENA | | Main Country | |
| AF | Afghanistan | South & Southeast Asia | | Other South & Southeast Asia | |
| AG | Antigua and Barbuda | Latin America | | Other Latin America | |
| AI | Anguilla | Latin America | | Other Latin America | |
| AL | Albania | Europe | | Eastern Europe | |
| AM | Armenia | Russia & Central Asia | | Other Russia & Central Asia | |
| AO | Angola | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| AR | Argentina | Latin America | | Main Country | |
| AT | Austria | Europe | | Other Western Europe | |
| AU | Australia | North America & Oceania | Oceania | Main Country | |
| AW | Aruba | Latin America | | Other Latin America | |
| AZ | Azerbaijan | Russia & Central Asia | | Other Russia & Central Asia | |
| BA | Bosnia and Herzegovina | Europe | | Eastern Europe | |
| BB | Barbados | Latin America | | Other Latin America | |
| BD | Bangladesh | South & Southeast Asia | | Main Country | |
| BE | Belgium | Europe | | Other Western Europe | |
| BF | Burkina Faso | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| BG | Bulgaria | Europe | | Eastern Europe | |
| BH | Bahrain | MENA | | Other MENA | |
| BI | Burundi | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| BJ | Benin | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| BM | Bermuda | North America & Oceania | North America | Other North America & Oceania | Other North America |
| BN | Brunei Darussalam | South & Southeast Asia | | Other South & Southeast Asia | |
| BO | Bolivia | Latin America | | Other Latin America | |
| BQ | Bonaire | Latin America | | Other Latin America | |
| BR | Brazil | Latin America | | Main Country | |
| BS | Bahamas | Latin America | | Other Latin America | |
| BT | Bhutan | South & Southeast Asia | | Other South & Southeast Asia | |
| BW | Botswana | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| BY | Belarus | Russia & Central Asia | | Other Russia & Central Asia | |
| BZ | Belize | Latin America | | Other Latin America | |
| CA | Canada | North America & Oceania | North America | Main Country | |
| CD | DR Congo | Sub-Saharan Africa | | Main Country | |
| CF | Central African Republic | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| CG | Congo | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| CH | Switzerland | Europe | | Other Western Europe | |
| CI | Cote d'Ivoire | Sub-Saharan Africa | | Main Country | |
| CL | Chile | Latin America | | Main Country | |
| CM | Cameroon | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| CN | China | East Asia | | Main Country | |
| CO | Colombia | Latin America | | Main Country | |
| CR | Costa Rica | Latin America | | Other Latin America | |
| CU | Cuba | Latin America | | Other Latin America | |
| CV | Cabo Verde | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| CW | Curacao | Latin America | | Other Latin America | |
| CY | Cyprus | Europe | | Eastern Europe | |
| CZ | Czechia | Europe | | Eastern Europe | |
| DE | Germany | Europe | | Main Country | |
| DJ | Djibouti | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| DK | Denmark | Europe | | Main Country | |
| DM | Dominica | Latin America | | Other Latin America | |
| DO | Dominican Republic | Latin America | | Other Latin America | |
| DZ | Algeria | MENA | | Main Country | |
| EC | Ecuador | Latin America | | Other Latin America | |
| EE | Estonia | Europe | | Eastern Europe | |
| EG | Egypt | MENA | | Main Country | |
| ER | Eritrea | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| ES | Spain | Europe | | Main Country | |
| ET | Ethiopia | Sub-Saharan Africa | | Main Country | |
| FI | Finland | Europe | | Other Western Europe | |
| FJ | Fiji | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| FM | Micronesia | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| FR | France | Europe | | Main Country | |
| GA | Gabon | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GB | United Kingdom | Europe | | Main Country | |
| GD | Grenada | Latin America | | Other Latin America | |
| GE | Georgia | Russia & Central Asia | | Other Russia & Central Asia | |
| GG | Guernsey | Europe | | Other Western Europe | |
| GH | Ghana | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GI | Gibraltar | Europe | | Other Western Europe | |
| GL | Greenland | North America & Oceania | North America | Other North America & Oceania | Other North America |
| GM | Gambia | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GN | Guinea | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GQ | Equatorial Guinea | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GR | Greece | Europe | | Other Western Europe | |
| GT | Guatemala | Latin America | | Other Latin America | |
| GW | Guinea-Bissau | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| GY | Guyana | Latin America | | Other Latin America | |
| HK | Hong Kong | East Asia | | Other East Asia | |
| HN | Honduras | Latin America | | Other Latin America | |
| HR | Croatia | Europe | | Eastern Europe | |
| HT | Haiti | Latin America | | Other Latin America | |
| HU | Hungary | Europe | | Eastern Europe | |
| ID | Indonesia | South & Southeast Asia | | Main Country | |
| IE | Ireland | Europe | | Other Western Europe | |
| IL | Israel | MENA | | Other MENA | |
| IM | Isle of Man | Europe | | Other Western Europe | |
| IN | India | South & Southeast Asia | | Main Country | |
| IQ | Iraq | MENA | | Other MENA | |
| IR | Iran | MENA | | Main Country | |
| IS | Iceland | Europe | | Other Western Europe | |
| IT | Italy | Europe | | Main Country | |
| JE | Jersey | Europe | | Other Western Europe | |
| JM | Jamaica | Latin America | | Other Latin America | |
| JO | Jordan | MENA | | Other MENA | |
| JP | Japan | East Asia | | Main Country | |
| KE | Kenya | Sub-Saharan Africa | | Main Country | |
| KG | Kyrgyzstan | Russia & Central Asia | | Other Russia & Central Asia | |
| KH | Cambodia | South & Southeast Asia | | Other South & Southeast Asia | |
| KI | Kiribati | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| KM | Comoros | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| KN | Saint Kitts and Nevis | Latin America | | Other Latin America | |
| KP | North Korea | East Asia | | Other East Asia | |
| KR | South Korea | East Asia | | Main Country | |
| KS | Kosovo | Europe | | Eastern Europe | |
| KW | Kuwait | MENA | | Other MENA | |
| KY | Cayman Islands | Latin America | | Other Latin America | |
| KZ | Kazakhstan | Russia & Central Asia | | Other Russia & Central Asia | |
| LA | Lao PDR | South & Southeast Asia | | Other South & Southeast Asia | |
| LB | Lebanon | MENA | | Other MENA | |
| LC | Saint Lucia | Latin America | | Other Latin America | |
| LI | Liechtenstein | Europe | | Other Western Europe | |
| LK | Sri Lanka | South & Southeast Asia | | Other South & Southeast Asia | |
| LR | Liberia | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| LS | Lesotho | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| LT | Lithuania | Europe | | Eastern Europe | |
| LU | Luxembourg | Europe | | Other Western Europe | |
| LV | Latvia | Europe | | Eastern Europe | |
| LY | Libya | MENA | | Other MENA | |
| MA | Morocco | MENA | | Main Country | |
| MC | Monaco | Europe | | Other Western Europe | |
| MD | Moldova | Europe | | Eastern Europe | |
| ME | Montenegro | Europe | | Eastern Europe | |
| MG | Madagascar | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| MH | Marshall Islands | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| MK | North Macedonia | Europe | | Eastern Europe | |
| ML | Mali | Sub-Saharan Africa | | Main Country | |
| MM | Myanmar | South & Southeast Asia | | Main Country | |
| MN | Mongolia | East Asia | | Other East Asia | |
| MO | Macao | East Asia | | Other East Asia | |
| MR | Mauritania | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| MS | Montserrat | Latin America | | Other Latin America | |
| MT | Malta | Europe | | Other Western Europe | |
| MU | Mauritius | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| MV | Maldives | South & Southeast Asia | | Other South & Southeast Asia | |
| MW | Malawi | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| MX | Mexico | Latin America | | Main Country | |
| MY | Malaysia | South & Southeast Asia | | Other South & Southeast Asia | |
| MZ | Mozambique | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| NA | Namibia | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| NC | New Caledonia | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| NE | Niger | Sub-Saharan Africa | | Main Country | |
| NG | Nigeria | Sub-Saharan Africa | | Main Country | |
| NI | Nicaragua | Latin America | | Other Latin America | |
| NL | Netherlands | Europe | | Main Country | |
| NO | Norway | Europe | | Main Country | |
| NP | Nepal | South & Southeast Asia | | Other South & Southeast Asia | |
| NR | Nauru | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| NZ | New Zealand | North America & Oceania | Oceania | Main Country | |
| OM | Oman | MENA | | Other MENA | |
| PA | Panama | Latin America | | Other Latin America | |
| PE | Peru | Latin America | | Other Latin America | |
| PF | French Polynesia | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| PG | Papua New Guinea | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| PH | Philippines | South & Southeast Asia | | Main Country | |
| PK | Pakistan | South & Southeast Asia | | Main Country | |
| PL | Poland | Europe | | Eastern Europe | |
| PR | Puerto Rico | Latin America | | Other Latin America | |
| PS | Palestine | MENA | | Other MENA | |
| PT | Portugal | Europe | | Other Western Europe | |
| PW | Palau | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| PY | Paraguay | Latin America | | Other Latin America | |
| QA | Qatar | MENA | | Other MENA | |
| RO | Romania | Europe | | Eastern Europe | |
| RS | Serbia | Europe | | Eastern Europe | |
| RU | Russia | Russia & Central Asia | | Main Country | |
| RW | Rwanda | Sub-Saharan Africa | | Main Country | |
| SA | Saudi Arabia | MENA | | Main Country | |
| SB | Solomon Islands | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| SC | Seychelles | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| SD | Sudan | Sub-Saharan Africa | | Main Country | |
| SE | Sweden | Europe | | Main Country | |
| SG | Singapore | South & Southeast Asia | | Other South & Southeast Asia | |
| SI | Slovenia | Europe | | Eastern Europe | |
| SK | Slovakia | Europe | | Eastern Europe | |
| SL | Sierra Leone | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| SM | San Marino | Europe | | Other Western Europe | |
| SN | Senegal | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| SO | Somalia | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| SR | Suriname | Latin America | | Other Latin America | |
| SS | South Sudan | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| ST | Sao Tome and Principe | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| SV | El Salvador | Latin America | | Other Latin America | |
| SX | Sint Maarten (Dutch part) | Latin America | | Other Latin America | |
| SY | Syria | MENA | | Other MENA | |
| SZ | Eswatini | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| TC | Turks and Caicos Islands | Latin America | | Other Latin America | |
| TD | Chad | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| TG | Togo | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| TH | Thailand | South & Southeast Asia | | Main Country | |
| TJ | Tajikistan | Russia & Central Asia | | Other Russia & Central Asia | |
| TL | Timor-Leste | South & Southeast Asia | | Other South & Southeast Asia | |
| TM | Turkmenistan | Russia & Central Asia | | Other Russia & Central Asia | |
| TN | Tunisia | MENA | | Other MENA | |
| TO | Tonga | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| TR | Turkiye | MENA | | Main Country | |
| TT | Trinidad and Tobago | Latin America | | Other Latin America | |
| TV | Tuvalu | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| TW | Taiwan | East Asia | | Main Country | |
| TZ | Tanzania | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| UA | Ukraine | Russia & Central Asia | | Other Russia & Central Asia | |
| UG | Uganda | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| US | USA | North America & Oceania | North America | Main Country | |
| UY | Uruguay | Latin America | | Other Latin America | |
| UZ | Uzbekistan | Russia & Central Asia | | Other Russia & Central Asia | |
| VC | Saint Vincent and the Grenadines | Latin America | | Other Latin America | |
| VE | Venezuela | Latin America | | Other Latin America | |
| VG | Virgin Islands, British | Latin America | | Other Latin America | |
| VN | Viet Nam | South & Southeast Asia | | Main Country | |
| VU | Vanuatu | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| WS | Samoa | North America & Oceania | Oceania | Other North America & Oceania | Other Oceania |
| YE | Yemen | MENA | | Other MENA | |
| ZA | South Africa | Sub-Saharan Africa | | Main Country | |
| ZM | Zambia | Sub-Saharan Africa | | Other Sub-Saharan Africa | |
| ZW | Zimbabwe | Sub-Saharan Africa | | Other Sub-Saharan Africa | |

### World Region Codes

Regions available in two currency variants. The decomposition `(=) WO = (+) QE + (+) QL + (+) XB + (+) XF + (+) XL + (+) XN + (+) XR + (+) XS`.

| PPP code | MER code | Region |
|----------|----------|--------|
| WO-PPP | WO-MER | World |
| QE-PPP | QE-MER | Europe |
| OC-PPP | OC-MER | Other Western Europe |
| QM-PPP | QM-MER | Eastern Europe |
| QL-PPP | QL-MER | East Asia |
| OB-PPP | OB-MER | Other East Asia |
| XB-PPP | XB-MER | North America and Oceania |
| QF-PPP | QF-MER | Oceania |
| OL-PPP | OL-MER | Other Oceania |
| QP-PPP | QP-MER | North America |
| OK-PPP | OK-MER | Other North America |
| OH-PPP | OH-MER | Other North America and Oceania |
| XF-PPP | XF-MER | Sub-Saharan Africa |
| OJ-PPP | OJ-MER | Other Sub-Saharan Africa |
| XL-PPP | XL-MER | Latin America |
| OD-PPP | OD-MER | Other Latin America |
| XN-PPP | XN-MER | MENA (Middle East and North Africa) |
| OE-PPP | OE-MER | Other MENA |
| XR-PPP | XR-MER | Russia and Central Asia |
| OA-PPP | OA-MER | Other Russia and Central Asia |
| XS-PPP | XS-MER | South and Southeast Asia |
| OI-PPP | OI-MER | Other South and Southeast Asia |

### Historical Country Codes

| WID code | Country |
|----------|---------|
| CS | Czechoslovakia |
| DD | German Democratic Republic |
| SU | USSR |
| YU | Yugoslavia |
| ZZ | Zanzibar |

### Country Subregion Codes

| WID code | Region |
|----------|--------|
| CN-RU | Rural China |
| CN-UR | Urban China |
| DE-BD | Baden |
| DE-BY | Bavaria |
| DE-HB | Bremen |
| DE-HE | Hesse |
| DE-HH | Hamburg |
| DE-PR | Prussia |
| DE-SN | Saxony |
| DE-WU | Wurttemberg |
| US-AK | Alaska |
| US-AL | Alabama |
| US-AR | Arkansas |
| US-AZ | Arizona |
| US-CA | California |
| US-CO | Colorado |
| US-CT | Connecticut |
| US-DC | District of Columbia |
| US-DE | Delaware |
| US-FL | Florida |
| US-GA | Georgia |
| US-HI | Hawaii |
| US-IA | Iowa |
| US-ID | Idaho |
| US-IL | Illinois |
| US-IN | Indiana |
| US-KS | Kansas |
| US-KY | Kentucky |
| US-LA | Louisiana |
| US-MA | Massachusetts |
| US-MD | Maryland |
| US-ME | Maine |
| US-MI | Michigan |
| US-MN | Minnesota |
| US-MO | Missouri |
| US-MS | Mississippi |
| US-MT | Montana |
| US-NC | North Carolina |
| US-ND | North Dakota |
| US-NE | Nebraska |
| US-NH | New Hampshire |
| US-NJ | New Jersey |
| US-NM | New Mexico |
| US-NV | Nevada |
| US-NY | New York |
| US-OH | Ohio |
| US-OK | Oklahoma |
| US-OR | Oregon |
| US-PA | Pennsylvania |
| US-RI | Rhode Island |
| US-SC | South Carolina |
| US-SD | South Dakota |
| US-TN | Tennessee |
| US-TX | Texas |
| US-UT | Utah |
| US-VA | Virginia |
| US-VT | Vermont |
| US-WA | Washington |
| US-WI | Wisconsin |
| US-WV | West Virginia |
| US-WY | Wyoming |

---

## 2.3 Percentile Codes

There are `r nrow(wid_percentiles)` enumerated codes of the form `pXpY` (X < Y, both 0-100).

```{r percentiles}
head(wid_percentiles)
wid_search("top 1", tables = "percentiles")
wid_validate(perc = "p99p100")
wid_validate(perc = "p90p10")    # error: invalid order
```

**Key groups** (most widely used): `p0p100`, `p0p50`, `p50p90`, `p90p100`, `p99p100`, `p99.9p100`, `p99.99p100`, and all deciles `p0p10`, `p10p20`, ..., `p90p100`.

**Detailed percentile groups:** all centiles (`p0p1` through `p99p100`), tenths within the top 1% (`p99.1p99.2`, ..., `p99.9p100`), hundredths within the top 0.1%, and thousandths within the top 0.01%.

**Detailed top groups:** population above a given percentile -- `p1p100`, `p2p100`, ...

**Detailed bottom groups:** population below a given percentile -- `p0p99`, `p0p98`, ...

For each group, WID provides: **share** (`s`), **average** (`a`), **threshold** (`t`), and for top groups the **inverted Pareto-Lorenz coefficient** (`b`) -- the ratio of group average to group threshold, measuring fatness of the distributional tail.

Note: individuals are not always ranked by the quantity being measured. Use `wid_decode()` or `wid_metadata()` to confirm the ranking variable.

---

## 3. Aggregate Income Variables

### 3.1.1 GDP and Net National Income

```{r nni}
download_wid(indicators = "anninc992i", areas = "FR", years = 1900:2023)
```

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nninc` | (=) net national income | B5n | S1 |
| `ndpro` | (+) net domestic product | | |
| `gdpro` | (+) gross domestic product | B1g | S14+S15 |
| `confc` | (-) consumption of fixed capital | P51c | S1 |
| `nnfin` | (+) foreign income | (-) B5n | S2 |

### 3.1.2 Decomposition of Foreign Income

**Paid/Received**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nnfin` | (=) net foreign income | | |
| `finrx` | (+) foreign income received from the rest of the world | | |
| `flcir` | (+) labor and capital income from the rest of the world | D1+D4 | S2 |
| `comrx` | (+) compensation of employees received | D1 | S2 |
| `pinrx` | (+) property income received | D4 | S2 |
| `fdirx` | (+) foreign direct investment income | | |
| `ptfrx` | (+) portfolio and other income | | |
| `ptdrx` | (+) debt income | | |
| `pterx` | (+) equity income | | |
| `ptrrx` | (+) reserves income | | |
| `ptfrr` | (+) reinvested earnings on foreign portfolio investment | | |
| `fsubx` | (+) subsidies on prod. received from rest of world | D3 | S2 |
| `fpsub` | (+) subsidies on products | D31 | S2 |
| `fosub` | (+) other subsidies on production | D39 | S2 |
| `finpx` | (-) foreign income paid to the rest of the world | | |
| `flcip` | (+) labor and capital income paid to rest of world | | |
| `compx` | (+) compensation of employees paid | D1 | S2 |
| `pinpx` | (+) property income paid | D4 | S2 |
| `fdipx` | (+) foreign direct investment income paid | | |
| `ptfpx` | (+) portfolio and other income paid | | |
| `ptdpx` | (+) debt income paid | | |
| `ptepx` | (+) equity income paid | | |
| `ptfrp` | (+) reinvested earnings on foreign portfolio investment paid | | |
| `ftaxx` | (+) taxes on prod. paid to rest of world | D2 | S2 |
| `fptax` | (+) taxes on products | D21 | S2 |
| `fotax` | (+) other taxes on production | D29 | S2 |

**Labor/Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nnfin` | (=) net foreign income | | |
| `flcin` | (+) net foreign labor and capital income | | |
| `pinnx` | (+) net foreign capital income | D4 | S2 |
| `fdinx` | (+) net foreign direct investment income | | |
| `ptfnx` | (+) net portfolio income | | |
| `ptdnx` | (+) net debt income | | |
| `ptenx` | (+) net equity income | | |
| `ptfrn` | (+) net reinvested earnings on foreign portfolio investment | | |
| `ptrrx` | (+) reserves income | | |
| `comnx` | (+) net foreign labor income | D1 | S2 |
| `taxnx` | (+) subsidies less taxes on production and imports | D3-D2 | S2 |
| `prtxn` | (+) subsidies less taxes on products | D31-D21 | S2 |
| `optxn` | (+) other subsidies less taxes on production | D39-D29 | S2 |

### 3.1.3 Decomposition of Foreign Wealth

| WID code | Description |
|----------|-------------|
| `nwnxa` | (=) net foreign assets |
| `fdixn` | (+) net foreign direct investment |
| `fdixa` | (+) foreign direct investment assets |
| `fdixd` | (-) foreign direct investment liabilities |
| `ptfxn` | (+) net foreign portfolio |
| `ptfxa` | (+) portfolio assets |
| `ptexa` | (+) portfolio equity assets |
| `ptdxa` | (+) portfolio debt assets |
| `ptrxa` | (+) foreign reserve exchange assets |
| `ptfxd` | (-) portfolio liabilities |
| `ptexd` | (+) portfolio equity liabilities |
| `ptdxd` | (+) portfolio debt liabilities |

### 3.1.4 Decomposition of National Income between Sectors

**Primary Income Decomposition**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nninc` | (=) net national income | B5n | S1 |
| `prigo` | (+) net primary income of the general government | B5n | S13 |
| `prihn` | (+) net primary income of households and non-profits | B5n | S14+S15 |
| `priho` | (+) net primary income of households | B5n | S14 |
| `prinp` | (+) net primary income of non-profits | B5n | S15 |
| `prico` | (+) net primary income of corporations | B5n | S11+S12 |
| `prinf` | (+) net primary income of non-financial corporations | B5n | S11 |
| `prifc` | (+) net primary income of financial corporations | B5n | S12 |

**Secondary Income Decomposition**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nninc` | (=) net national income | B5n | S1 |
| `secgo` | (+) net secondary income of the general government | B6n+D7 | S13 |
| `sechn` | (+) net secondary income of households and non-profits | B6n+D7 | S14+S15 |
| `secho` | (+) net secondary income of households | B6n+D7 | S14 |
| `secnp` | (+) net secondary income of non-profits | B6n+D7 | S15 |
| `secco` | (+) net secondary income of corporations | B6n+D7 | S11+S12 |
| `secnf` | (+) net secondary income of non-financial corporations | B6n+D7 | S11 |
| `secfc` | (+) net secondary income of financial corporations | B6n+D7 | S12 |

**Decomposition of Property Income between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `pinnx` | (=) net foreign property income | (-) D4 | S2 |
| `prphn` | (+) property income of households and NPISH | D4 | S14+S15 |
| `prpco` | (+) property income of corporations | D4 | S11+S12 |
| `prpgo` | (+) property income of the government | D4 | S13 |

**Decomposition of Direct Taxes on Income and Wealth between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `taxgo` | (=) direct taxes received by the government | D5 | S13 |
| `tiwhn` | (+) taxes on income and wealth paid by households | D5 | S14+S15 |
| `taxco` | (+) corporate tax | D5 | S11+S12 |

**Decomposition of Social Security Contributions between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `sschn` | (=) social contributions paid by households and NPISH | D61 | S14+S15 |
| `sscgo` | (+) social contributions received by the government | D61 | S13 |
| `sscco` | (+) social contributions to private employer social insurance | D61 | S11+S12 |

**Decomposition of Social Security Benefits between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `ssbhn` | (=) social benefits received by households and NPISH | D61 | S14+S15 |
| `ssbgo` | (+) social benefits paid by the government | D61 | S13 |
| `ssbco` | (+) social benefits from private employer social insurance | D61 | S11+S12 |

**Decomposition of Net Savings between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savin` | (=) net savings of the total economy | B8n+D7 | S1 |
| `savhn` | (+) net savings of households and NPISH | B8n+D7 | S14+S15 |
| `savho` | (+) net savings of households | B8n+D7 | S14 |
| `savnp` | (+) net savings of NPISH | B8n+D7 | S15 |
| `savco` | (+) net savings/secondary income of corporations | B8n+D7 | S11+S12 |
| `secnf` | (+) net savings of non-financial corporations | B8n+D7 | S11 |
| `secfc` | (+) net savings of financial corporations | B8n+D7 | S12 |
| `savgo` | (+) net savings of the general government | B8n+D7 | S13 |

**Decomposition of Gross Savings between Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savig` | (=) gross savings of the total economy | B8g+D7 | S1 |
| `saghn` | (+) gross savings of households and NPISH | B8g+D7 | S14+S15 |
| `sagho` | (+) gross savings of households | B8g+D7 | S14 |
| `sagnp` | (+) gross savings of NPISH | B8g+D7 | S15 |
| `sagco` | (+) gross savings/secondary income of corporations | B8g+D7 | S11+S12 |
| `segnf` | (+) gross savings of non-financial corporations | B8g+D7 | S11 |
| `segfc` | (+) gross savings of financial corporations | B8g+D7 | S12 |
| `saggo` | (+) gross savings of the general government | B8g+D7 | S13 |

**Net and Gross Variables across Sectors**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `confc` | (=) consumption of fixed capital of the total economy | P51c | S1 |
| `cfchn` | (+) CFC of households and NPISH | P51c | S14+S15 |
| `cfcho` | (+) CFC of households | P51c | S14 |
| `cfcnp` | (+) CFC of NPISH | P51c | S15 |
| `cfcco` | (+) CFC of corporations | P51c | S11+S12 |
| `cfcnf` | (+) CFC of non-financial corporations | P51c | S11 |
| `cfcfc` | (+) CFC of financial corporations | P51c | S12 |
| `cfcgo` | (+) CFC of the general government | P51c | S13 |
| `savin` | (=) net savings of the total economy | B8n+D7 | S1 |
| `savig` | (+) gross savings of the total economy | B8g+D7 | S1 |
| `confc` | (-) consumption of fixed capital of the total economy | P51c | S1 |

### 3.1.5 Labor and Capital Share of National Income

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `nninc` | (=) net national income | B5n | S1 |
| `comhn` | (+) compensation of employees | D1 | S14+S15 |
| `fkpin` | (+) net capital income | | |
| `prphn` | (+) property income distributed to households and NPISH | D4 | S14+S15 |
| `prpgo` | (+) property income distributed to the government | D4 | S13 |
| `nsrhn` | (+) net operating surplus of households and NPISH | B2n | S14+S15 |
| `prico` | (+) net primary income of corporations | B5n | S11+S12 |
| `nmxho` | (+) net mixed income of households | B3n | S14 |
| `ptxgo` | (+) taxes on products and production | D2-D3 | S13 |

Mixed income is conventionally split 70% labor / 30% capital for computing factor shares. The full four-way decomposition (pure labor, pure capital, mixed income, production taxes) is available when the treatment of mixed income matters.

### 3.1.6 Income Decomposition by Institutional Sector

**Factor-Price GDP Decomposition**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `gdpro` | (=) gross domestic product at market prices | B1g | S1 |
| `ptxgo` | (+) net taxes on products and production | D2-D3 | S1 |
| `gvato` | (+) gross domestic product at factor-price | B1g | S1 |
| `gvago` | (+) gross value added of the government | B1g | S13 |
| `ceugo` | (+) compensation of employees of the government | D1 | S13 |
| `gsrgo` | (+) gross operating surplus of the government | B2g | S13 |
| `nsrgo` | (+) net operating surplus of the government (=0) | B2n | S13 |
| `cfcgo` | (+) CFC of the government | P51c | S13 |
| `gvaco` | (+) gross value added of the corporate sector | B1g | S11+S12 |
| `ceuco` | (+) compensation of employees of the corporate sector | D1 | S11+S12 |
| `gsrco` | (+) gross operating surplus of the corporate sector | B2g | S11+S12 |
| `nsrco` | (+) net operating surplus of the corporate sector | B2n | S11+S12 |
| `cfcco` | (+) CFC of the corporate sector | P51c | S11+S12 |
| `gvahn` | (+) gross value added of household sector | B1g | S14+S15 |
| `ceuhn` | (+) compensation of employees of household sector | D1 | S14+S15 |
| `gmxhn` | (+) gross mixed income of household sector | B3g | S14+S15 |
| `nmxhn` | (+) net mixed income of household sector | B3n | S14+S15 |
| `ccmhn` | (+) CFC of mixed income of household sector | P51c | S14+S15 |
| `gsrhn` | (+) gross operating surplus of household sector | B2g | S14+S15 |
| `nsrhn` | (+) net operating surplus of household sector | B2n | S14+S15 |
| `ccshn` | (+) CFC of operating surplus of household sector | P51c | S14+S15 |

**Factor Shares**

| WID code | Description | SNA sector |
|----------|-------------|------------|
| `ylsgdp` | labor share of gross domestic product at factor-price | S1 |
| `ylsndp` | labor share of net domestic product at factor-price | S1 |
| `ycsgdp` | capital share of gross domestic product at factor-price | S1 |
| `ycsndp` | capital share of net domestic product at factor-price | S1 |
| `wlsgni` | labor share of gross national income at factor-price | S1 |
| `wlsnni` | labor share of national income at factor-price | S1 |
| `wcsgni` | capital share of gross national income at factor-price | S1 |
| `wcsnni` | capital share of national income at factor-price | S1 |
| `ylscgv` | labor share in corporate gross value added at factor-price | S11+S12 |
| `ylscnv` | labor share in corporate net value added at factor-price | S11+S12 |
| `ycscgv` | capital share in corporate gross value added at factor-price | S11+S12 |
| `ycscnv` | capital share in corporate net value added at factor-price | S11+S12 |

---

## 3.2 Income of Households and NPISH

Variables with suffix `hn` = households and NPISH combined; `ho` = households only; `np` = NPISH only.

### 3.2.1 Income of the Sectors Combined

#### 3.2.1.1 Primary Incomes of Households and NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prihn` | (=) net primary income of households and NPISH | B5n | S14+S15 |
| `comhn` | (+) compensation of employees | D1 | S14+S15 |
| `prphn` | (+) property income (net) | D4 | S14+S15 |
| `nsmhn` | (+) net operating surplus and mixed income | B2n+B3n | S14+S15 |
| `nsrhn` | (+) net operating surplus | B2n | S14+S15 |
| `nmxhn` | (+) net mixed income | B3n | S14+S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prghn` | (=) gross primary income of households and NPISH | B5n | S14+S15 |
| `comhn` | (+) compensation of employees | D1 | S14+S15 |
| `prphn` | (+) property income (net) | D4 | S14+S15 |
| `gsmhn` | (+) gross operating surplus and mixed income | B2g+B3g | S14+S15 |
| `gsrhn` | (+) gross operating surplus | B2g | S14+S15 |
| `gmxhn` | (+) gross mixed income | B3g | S14+S15 |

#### 3.2.1.2 Secondary Incomes of Households and NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `sechn` | (=) net secondary income of households and NPISH | B6n+D7 | S14+S15 |
| `prihn` | (+) net primary income of households and NPISH | D1 | S14+S15 |
| `taxhn` | (-) direct taxes | D5+D61 | S14+S15 |
| `sschn` | (+) social contributions | D61 | S14+S15 |
| `tiwhn` | (+) personal taxes on income and wealth | D5 | S14+S15 |
| `ssbhn` | (+) social benefits other than social transfers in kind | D62 | S14+S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `seghn` | (=) gross secondary income of households and NPISH | B6n+D7 | S14+S15 |
| `prghn` | (+) gross primary income of households and NPISH | D1 | S14+S15 |
| `taxhn` | (-) direct taxes | D5+D61 | S14+S15 |
| `sschn` | (+) social contributions | D61 | S14+S15 |
| `tiwhn` | (+) personal taxes on income and wealth | D5 | S14+S15 |
| `ssbhn` | (+) social benefits other than social transfers in kind | D62 | S14+S15 |

#### 3.2.1.3 Consumption and Savings of Households and NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savhn` | (=) net savings of households and NPISH | B8n+D7 | S14+S15 |
| `sechn` | (+) net secondary income of households and NPISH | B6n+D7 | S14+S15 |
| `conhn` | (-) private expenditures of households and NPISH | P3 | S14+S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `saghn` | (=) gross savings of households and NPISH | B8n+D7 | S14+S15 |
| `seghn` | (+) gross secondary income of households and NPISH | B6n+D7 | S14+S15 |
| `conhn` | (-) private expenditures of households and NPISH | P3 | S14+S15 |

#### 3.2.1.4 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prghn` | (=) gross primary income of households and NPISH | B5g | S14+S15 |
| `prihn` | (+) net primary income of households and NPISH | B5n | S14+S15 |
| `cfchn` | (+) consumption of fixed capital | P51c | S14+S15 |
| `seghn` | (=) gross secondary income of households and NPISH | B6g+D7 | S14+S15 |
| `sechn` | (+) net secondary income of households and NPISH | B6n+D7 | S14+S15 |
| `cfchn` | (+) consumption of fixed capital | P51c | S14+S15 |
| `saghn` | (=) gross savings of households and NPISH | B8g+D7 | S14+S15 |
| `savhn` | (+) net savings of households and NPISH | B8n+D7 | S14+S15 |
| `cfchn` | (+) consumption of fixed capital | P51c | S14+S15 |
| `gsmhn` | (=) gross surplus and mixed income of households and NPISH | B2g+B3g | S14+S15 |
| `nsmhn` | (+) net surplus and mixed income of households and NPISH | B2n+B3n | S14+S15 |
| `cfchn` | (+) consumption of fixed capital | P51c | S14+S15 |
| `gsrhn` | (=) gross surplus of households and NPISH | B2g | S14+S15 |
| `nsrhn` | (+) net surplus of households and NPISH | B2n | S14+S15 |
| `ccshn` | (+) CFC attributable to operating surplus | | |
| `gmxhn` | (=) gross mixed income of households and NPISH | B3g | S14+S15 |
| `nmxhn` | (+) net mixed income of households and NPISH | B3n | S14+S15 |
| `ccmhn` | (+) CFC attributable to mixed income | | |
| `cfchn` | (=) consumption of fixed capital | P51c | S14+S15 |
| `ccshn` | (+) CFC attributable to operating surplus | | |
| `ccmhn` | (+) CFC attributable to mixed income | | |

### 3.2.2 Income of Households

#### 3.2.2.1 Primary Incomes of the Household Sector

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `priho` | (=) net primary income of households | B5n | S14 |
| `comho` | (+) compensation of employees | D1 | S14 |
| `prpho` | (+) property income (net) | D4 | S14 |
| `nsmho` | (+) net operating surplus and mixed income | B2n+B3n | S14 |
| `nsrho` | (+) net operating surplus | B2n | S14 |
| `nmxho` | (+) net mixed income | B3n | S14 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgho` | (=) gross primary income of households | B5n | S14 |
| `comho` | (+) compensation of employees | D1 | S14 |
| `prpho` | (+) property income (net) | D4 | S14 |
| `gsmho` | (+) gross operating surplus and mixed income | B2g+B3g | S14 |
| `gsrho` | (+) gross operating surplus | B2g | S14 |
| `gmxho` | (+) gross mixed income | B3g | S14 |

#### 3.2.2.2 Secondary Incomes of the Household Sector

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secho` | (=) net secondary income of households | B6n+D7 | S14 |
| `priho` | (+) net primary income of households | D1 | S14 |
| `taxho` | (-) direct taxes | D5+D61 | S14 |
| `sscho` | (+) social contributions | D61 | S14 |
| `tiwho` | (+) personal taxes on income and wealth | D5 | S14 |
| `ssbho` | (+) social benefits other than social transfers in kind | D62 | S14 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `segho` | (=) gross secondary income of households | B6n+D7 | S14 |
| `prgho` | (+) gross primary income of households | D1 | S14 |
| `taxho` | (-) direct taxes | D5+D61 | S14 |
| `sscho` | (+) social contributions | D61 | S14 |
| `tiwho` | (+) personal taxes on income and wealth | D5 | S14 |
| `ssbho` | (+) social benefits other than social transfers in kind | D62 | S14 |

#### 3.2.2.3 Consumption and Savings of the Household Sector

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savho` | (=) net savings of households | B8n+D7 | S14 |
| `secho` | (+) net secondary income of households | B6n+D7 | S14 |
| `conho` | (-) private expenditures of households | P3 | S14 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `sagho` | (=) gross savings of households | B8n+D7 | S14 |
| `segho` | (+) gross secondary income of households | B6n+D7 | S14 |
| `conho` | (-) private expenditures of households | P3 | S14 |

#### 3.2.2.4 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgho` | (=) gross primary income of households | B5g | S14 |
| `priho` | (+) net primary income of households | B5n | S14 |
| `cfcho` | (+) consumption of fixed capital | P51c | S14 |
| `segho` | (=) gross secondary income of households | B6g+D7 | S14 |
| `secho` | (+) net secondary income of households | B6n+D7 | S14 |
| `cfcho` | (+) consumption of fixed capital | P51c | S14 |
| `sagho` | (=) gross savings of households | B8g+D7 | S14 |
| `savho` | (+) net savings of households | B8n+D7 | S14 |
| `cfcho` | (+) consumption of fixed capital | P51c | S14 |
| `gsmho` | (=) gross surplus and mixed income of households | B2g+B3g | S14 |
| `nsmho` | (+) net surplus and mixed income of households | B2n+B3n | S14 |
| `cfcho` | (+) consumption of fixed capital | P51c | S14 |
| `gsrho` | (=) gross surplus of households | B2g | S14 |
| `nsrho` | (+) net surplus of households | B2n | S14 |
| `ccsho` | (+) CFC attributable to operating surplus | | |
| `gmxho` | (=) gross mixed income of households | B3g | S14 |
| `nmxho` | (+) net mixed income of households | B3n | S14 |
| `ccmho` | (+) CFC attributable to mixed income | | |
| `cfcho` | (=) consumption of fixed capital | P51c | S14 |
| `ccsho` | (+) CFC attributable to operating surplus | | |
| `ccmho` | (+) CFC attributable to mixed income | | |

### 3.2.3 Income of NPISH

#### 3.2.3.1 Primary Incomes of NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prinp` | (=) net primary income of NPISH | B5n | S15 |
| `comnp` | (+) compensation of employees | D1 | S15 |
| `prpnp` | (+) property income (net) | D4 | S15 |
| `nsrnp` | (+) net operating surplus | B2n | S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgnp` | (=) gross primary income of NPISH | B5n | S15 |
| `comnp` | (+) compensation of employees | D1 | S15 |
| `prpnp` | (+) property income (net) | D4 | S15 |
| `gsrnp` | (+) gross operating surplus | B2g | S15 |

#### 3.2.3.2 Secondary Incomes of NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secnp` | (=) net secondary income of NPISH | B6n+D7 | S15 |
| `prinp` | (+) net primary income of NPISH | D1 | S15 |
| `taxnp` | (-) direct taxes | D5+D61 | S15 |
| `sscnp` | (+) social contributions | D61 | S15 |
| `tiwnp` | (+) personal taxes on income and wealth | D5 | S15 |
| `ssbnp` | (+) social benefits other than social transfers in kind | D62 | S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `segnp` | (=) gross secondary income of NPISH | B6n+D7 | S15 |
| `prgnp` | (+) gross primary income of NPISH | D1 | S15 |
| `taxnp` | (-) direct taxes | D5+D61 | S15 |
| `sscnp` | (+) social contributions | D61 | S15 |
| `tiwnp` | (+) personal taxes on income and wealth | D5 | S15 |
| `ssbnp` | (+) social benefits other than social transfers in kind | D62 | S15 |

#### 3.2.3.3 Consumption and Savings of NPISH

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savnp` | (=) net savings of NPISH | B8n+D7 | S15 |
| `secnp` | (+) net secondary income of NPISH | B6n+D7 | S15 |
| `connp` | (-) private expenditures of NPISH | P3 | S15 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `sagnp` | (=) gross savings of NPISH | B8n+D7 | S15 |
| `segnp` | (+) gross secondary income of NPISH | B6n+D7 | S15 |
| `connp` | (-) private expenditures of NPISH | P3 | S15 |

#### 3.2.3.4 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgnp` | (=) gross primary income of NPISH | B5g | S15 |
| `prinp` | (+) net primary income of NPISH | B5n | S15 |
| `cfcnp` | (+) consumption of fixed capital | P51c | S15 |
| `segnp` | (=) gross secondary income of NPISH | B6g+D7 | S15 |
| `secnp` | (+) net secondary income of NPISH | B6n+D7 | S15 |
| `cfcnp` | (+) consumption of fixed capital | P51c | S15 |
| `sagnp` | (=) gross savings of NPISH | B8g+D7 | S15 |
| `savnp` | (+) net savings of NPISH | B8n+D7 | S15 |
| `cfcnp` | (+) consumption of fixed capital | P51c | S15 |
| `gsrnp` | (=) gross surplus of NPISH | B2g | S15 |
| `nsrnp` | (+) net surplus of NPISH | B2n | S15 |
| `cfcnp` | (+) CFC attributable to operating surplus | | |

---

## 3.3 Income of the Corporate Sector

### 3.3.1 Income of the Sectors Combined

#### 3.3.1.1 Primary Income of Corporations

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prico` | (=) net primary income of corporations | B5n | S11+S12 |
| `prpco` | (+) property income (net) | D4 | S11+S12 |
| `nsrco` | (+) net operating surplus | B2n | S11+S12 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgco` | (=) gross primary income of corporations | B5g | S11+S12 |
| `prpco` | (+) property income (net) | D4 | S11+S12 |
| `gsrco` | (+) gross operating surplus | B2g | S11+S12 |

#### 3.3.1.2 Secondary Income of Corporations

For corporations, secondary income equals net saving (corporations have no consumption expenditure).

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secco` | (=) net secondary income / net saving of corporations | B6n+D7 | S11+S12 |
| `prico` | (+) net primary income of corporations | B5n | S11+S12 |
| `taxco` | (-) corporate tax | D5 | S11+S12 |
| `sscco` | (+) social contributions to private employer social insurance | D61 | S11+S12 |
| `ssbco` | (-) social benefits from private employer social insurance | D62 | S11+S12 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `segco` | (=) gross secondary income / gross saving of corporations | B6g+D7 | S11+S12 |
| `prgco` | (+) gross primary income of corporations | B5g | S11+S12 |
| `taxco` | (-) corporate tax | D5 | S11+S12 |
| `sscco` | (+) social contributions to private employer social insurance | D61 | S11+S12 |
| `ssbco` | (-) social benefits from private employer social insurance | D62 | S11+S12 |

#### 3.3.1.3 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgco` | (=) gross primary income of corporations | B5g | S11+S12 |
| `prico` | (+) net primary income of corporations | B5n | S11+S12 |
| `cfcco` | (+) consumption of fixed capital | P51c | S11+S12 |
| `segco` | (=) gross secondary income / gross saving of corporations | B6g+D7 | S11+S12 |
| `secco` | (+) net secondary income / net saving of corporations | B6n+D7 | S11+S12 |
| `cfcco` | (+) consumption of fixed capital | P51c | S11+S12 |

### 3.3.2 Income of Non-financial Corporations

#### 3.3.2.1 Primary Income

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prinf` | (=) net primary income of non-financial corporations | B5n | S11 |
| `prpnf` | (+) property income (net) | D4 | S11 |
| `nsrnf` | (+) net operating surplus | B2n | S11 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgnf` | (=) gross primary income of non-financial corporations | B5g | S11 |
| `prpnf` | (+) property income (net) | D4 | S11 |
| `gsrnf` | (+) gross operating surplus | B2g | S11 |

#### 3.3.2.2 Secondary Income

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secnf` | (=) net secondary income / net saving of non-financial corporations | B6n+D7 | S11 |
| `prinf` | (+) net primary income of non-financial corporations | B5n | S11 |
| `taxnf` | (-) corporate tax | D5 | S11 |
| `sscnf` | (+) social contributions to private employer social insurance | D61 | S11 |
| `ssbnf` | (-) social benefits from private employer social insurance | D62 | S11 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `segnf` | (=) gross secondary income / gross saving of non-financial corporations | B6g+D7 | S11 |
| `prgnf` | (+) gross primary income of non-financial corporations | B5g | S11 |
| `taxnf` | (-) corporate tax | D5 | S11 |
| `sscnf` | (+) social contributions to private employer social insurance | D61 | S11 |
| `ssbnf` | (-) social benefits from private employer social insurance | D62 | S11 |

#### 3.3.2.3 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgnf` | (=) gross primary income of non-financial corporations | B5g | S11 |
| `prinf` | (+) net primary income of non-financial corporations | B5n | S11 |
| `cfcnf` | (+) consumption of fixed capital | P51c | S11 |
| `segnf` | (=) gross secondary income / gross saving of non-financial corporations | B6g+D7 | S11 |
| `secnf` | (+) net secondary income / net saving of non-financial corporations | B6n+D7 | S11 |
| `cfcnf` | (+) consumption of fixed capital | P51c | S11 |

### 3.3.3 Income of Financial Corporations

#### 3.3.3.1 Primary Income

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prifc` | (=) net primary income of financial corporations | B5n | S12 |
| `prpfc` | (+) property income (net) | D4 | S12 |
| `nsrfc` | (+) net operating surplus | B2n | S12 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgfc` | (=) gross primary income of financial corporations | B5g | S12 |
| `prpfc` | (+) property income (net) | D4 | S12 |
| `gsrfc` | (+) gross operating surplus | B2g | S12 |

#### 3.3.3.2 Secondary Income

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secfc` | (=) net secondary income / net saving of financial corporations | B6n+D7 | S12 |
| `prifc` | (+) net primary income of financial corporations | B5n | S12 |
| `taxfc` | (-) corporate tax | D5 | S12 |
| `sscfc` | (+) social contributions to private employer social insurance | D61 | S12 |
| `ssbfc` | (-) social benefits from private employer social insurance | D62 | S12 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `segfc` | (=) gross secondary income / gross saving of financial corporations | B6g+D7 | S12 |
| `prgfc` | (+) gross primary income of financial corporations | B5g | S12 |
| `taxfc` | (-) corporate tax | D5 | S12 |
| `sscfc` | (+) social contributions to private employer social insurance | D61 | S12 |
| `ssbfc` | (-) social benefits from private employer social insurance | D62 | S12 |

#### 3.3.3.3 Relation between Net and Gross Variables

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prgfc` | (=) gross primary income of financial corporations | B5g | S12 |
| `prifc` | (+) net primary income of financial corporations | B5n | S12 |
| `cfcfc` | (+) consumption of fixed capital | P51c | S12 |
| `segfc` | (=) gross secondary income / gross saving of financial corporations | B6g+D7 | S12 |
| `secfc` | (+) net secondary income / net saving of financial corporations | B6n+D7 | S12 |
| `cfcfc` | (+) consumption of fixed capital | P51c | S12 |

---

## 3.4 Income of the Government Sector

### 3.4.1 Primary Income of the Government

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prigo` | (=) net primary income of the general government | B5n | S13 |
| `ptxgo` | (+) taxes on production and imports, less subsidies | D2-D3 | S13 |
| `tpigo` | (+) taxes on production and imports | D2 | S13 |
| `tprgo` | (+) taxes on products | D21 | S13 |
| `otpgp` | (+) other taxes on production | D29 | S13 |
| `spigo` | (-) subsidies on production and imports | D3 | S13 |
| `sprgo` | (+) subsidies on products | D31 | S13 |
| `ospgo` | (+) other subsidies on production | D39 | S13 |
| `prpgo` | (+) property income (net) | D4 | S13 |
| `nsrgo` | (+) net operating surplus and miscellaneous | B2n | S13 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `prggo` | (=) gross primary income of the general government | B5n | S13 |
| `ptxgo` | (+) taxes on production and imports, less subsidies | D2-D3 | S13 |
| `tpigo` | (+) taxes on production and imports | D2 | S13 |
| `tprgo` | (+) taxes on products | D21 | S13 |
| `otpgp` | (+) other taxes on production | D29 | S13 |
| `spigo` | (-) subsidies on production and imports | D3 | S13 |
| `sprgo` | (+) subsidies on products | D31 | S13 |
| `ospgo` | (+) other subsidies on production | D39 | S13 |
| `prpgo` | (+) property income (net) | D4 | S13 |
| `gsrgo` | (+) gross operating surplus and miscellaneous | B2n | S13 |

### 3.4.2 Secondary Income of the Government

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `secgo` | (=) net secondary income of the general government | B6n+D7 | S13 |
| `prigo` | (+) net primary income of the general government | B5n | S13 |
| `taxgo` | (+) direct taxes | D5+D61 | S13 |
| `tiwgo` | (+) direct taxes on income and wealth | D5 | S13 |
| `sscgo` | (+) social contributions | D61 | S13 |
| `ssbgo` | (-) social benefits other than social transfers in kind | D62 | S13 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `seggo` | (=) gross secondary income of the general government | B6n+D7 | S13 |
| `prggo` | (+) gross primary income of the general government | B5n | S13 |
| `taxgo` | (+) direct taxes | D5+D61 | S13 |
| `tiwgo` | (+) direct taxes on income and wealth | D5 | S13 |
| `sscgo` | (+) social contributions | D61 | S13 |
| `ssbgo` | (-) social benefits other than social transfers in kind | D62 | S13 |

### 3.4.3 Consumption and Savings of the Government

**Net of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `savgo` | (=) net savings of the general government | B8n+D7 | S13 |
| `secgo` | (+) net secondary income of the general government | B6n+D7 | S13 |
| `congo` | (-) final consumption expenditures | P3 | S13 |
| `indgo` | (+) individual consumption expenditures | P31 | S13 |
| `colgo` | (+) collective consumption expenditure | P32 | S13 |

**Gross of Consumption of Fixed Capital**

| WID code | Description | SNA code | SNA sector |
|----------|-------------|----------|------------|
| `saggo` | (=) gross savings of the general government | B8n+D7 | S13 |
| `seggo` | (+) gross secondary income of the general government | B6n+D7 | S13 |
| `congo` | (-) final consumption expenditures | P3 | S13 |
| `indgo` | (+) individual consumption expenditures | P31 | S13 |
| `colgo` | (+) collective consumption expenditure | P32 | S13 |

**Government Spending by Function**

| WID code | Description | SNA sector |
|----------|-------------|------------|
| `congo` | (=) final consumption expenditures | S13 |
| `gpsgo` | (+) general public services | S13 |
| `defgo` | (+) defense | S13 |
| `polgo` | (+) public order and safety | S13 |
| `ecogo` | (+) economic affairs | S13 |
| `envgo` | (+) environment protection | S13 |
| `hougo` | (+) housing and community amenities | S13 |
| `heago` | (+) health | S13 |
| `recgo` | (+) recreation, culture and religion | S13 |
| `edugo` | (+) education | S13 |
| `sopgo` | (+) social protection | S13 |
| `othgo` | (+) other government spending | S13 |

**Government Expenditure by Function**

| WID code | Description | SNA sector |
|----------|-------------|------------|
| `expgo` | (=) total public spending (excl. interest payment) | S13 |
| `gpsge` | (+) general public services (excl. interest payment) | S13 |
| `defge` | (+) defense | S13 |
| `polge` | (+) public order and safety | S13 |
| `ecoge` | (+) economic affairs | S13 |
| `envge` | (+) environment protection | S13 |
| `houge` | (+) housing and community amenities | S13 |
| `heage` | (+) health | S13 |
| `recge` | (+) recreation, culture and religion | S13 |
| `eduge` | (+) education | S13 |
| `edpge` | (+) education: primary | S13 |
| `edsge` | (+) education: secondary | S13 |
| `edtge` | (+) education: tertiary | S13 |
| `sopge` | (+) social protection | S13 |
| `spige` | (+) social protection: social insurance | S13 |
| `sacge` | (+) social protection: social assistance in cash | S13 |
| `sakge` | (+) social protection: social assistance in kind | S13 |

**Government Revenue**

| WID code | Description | SNA sector |
|----------|-------------|------------|
| `revgo` | (=) total public revenue | S13 |
| `pitgr` | (+) personal income tax | S13 |
| `citgr` | (+) corporate income tax | S13 |
| `scogr` | (+) social contributions | S13 |
| `pwtgr` | (+) property and wealth taxes | S13 |
| `intgr` | (+) indirect taxes | S13 |
| `ottgr` | (+) other taxes | S13 |
| `ntrgr` | (+) non-tax revenue | S13 |
| `retgo` | (=) total public revenue (excl. non-tax revenue) | S13 |
| `psugo` | (=) primary surplus of the government | S13 |
| `ssugo` | (=) secondary surplus of the government | S13 |
| `inpgo` | (-) interest paid by the government | S13 |

---

## 3.5 Current Account

| WID code | Description |
|----------|-------------|
| `ncanx` | (=) Current Account = pinnx + comnx + tbnnx + taxnx + scinx |
| `pinnx` | (+) net foreign capital income |
| `pinrx` | (+) property income received from the rest of the world |
| `pinpx` | (-) property income paid to the rest of the world |
| `comnx` | (+) net foreign labor income |
| `comrx` | (+) compensation of employees received |
| `compx` | (-) compensation of employees paid |
| `tbnnx` | (+) trade balance (exports - imports) |
| `tbxrx` | (+) exports of goods and services |
| `tbmpx` | (-) imports of goods and services |
| `taxnx` | (+) foreign subsidies less taxes on production and imports |
| `fsubx` | (+) subsidies on prod. received from rest of world |
| `ftaxx` | (-) taxes on prod. paid to rest of world |
| `scinx` | (+) net foreign transfers |
| `scirx` | (+) foreign transfers received from rest of world |
| `scipx` | (-) foreign transfers paid to rest of world |

### 3.5.1 Trade Balance

| WID code | Description |
|----------|-------------|
| `tbnnx` | (=) trade balance (exports - imports) |
| `tgnnx` | (+) trade balance of goods (exports - imports) |
| `tgxrx` | (+) exports of goods |
| `tgmpx` | (-) imports of goods |
| `tsnnx` | (+) trade balance of services (exports - imports) |
| `tsxrx` | (+) exports of services |
| `tsmpx` | (-) imports of services |

### 3.5.2 Trade Balance of Services

| WID code | Description |
|----------|-------------|
| `tsnnx` | (=) trade balance of services (exports - imports) |
| `tsvnx` | (+) trade balance of travel services (exports - imports) |
| `tsvrx` | (+) exports of travel services |
| `tsvpx` | (-) imports of travel services |
| `tstnx` | (+) trade balance of transport services (exports - imports) |
| `tstrx` | (+) exports of transport services |
| `tstpx` | (-) imports of transport services |
| `tsonx` | (+) trade balance of other services (exports - imports) |
| `tsorx` | (+) exports of other services |
| `tsopx` | (-) imports of other services |

### 3.5.3 Trade Balance of Goods

| WID code | Description |
|----------|-------------|
| `tgnnx` | (=) trade balance of goods (exports - imports) |
| `tgncx` | (+) trade balance of primary commodities (exports - imports) |
| `tgxcx` | (+) exports of primary commodities |
| `tgmcx` | (-) imports of primary commodities |
| `tgnmx` | (+) trade balance of manufacturing goods (exports - imports) |
| `tgxmx` | (+) exports of manufacturing goods |
| `tgmmx` | (-) imports of manufacturing goods |

### 3.5.4 Capital Account

| WID code | Description |
|----------|-------------|
| `fkanx` | (=) capital account |
| `fkarx` | (+) capital transfers received from the rest of the world |
| `fkapx` | (-) capital transfers paid to the rest of the world |

### 3.5.5 Net Foreign Transfers

| WID code | Description |
|----------|-------------|
| `scinx` | (+) net foreign transfers |
| `scrnx` | (+) net remittances |
| `scrrx` | (+) remittances received from rest of world |
| `scrpx` | (-) remittances paid to rest of world |
| `scgnx` | (+) net public foreign transfers |
| `scgrx` | (+) public foreign transfers received from rest of world |
| `scgpx` | (-) public foreign transfers paid to rest of world |
| `sconx` | (+) net other foreign transfers |
| `scorx` | (+) other foreign transfers received from rest of world |
| `scopx` | (-) other foreign transfers paid to rest of world |

---

## 4. Distributed Income Variables

```{r dist-income}
# Top 1% pretax income share
download_wid(
  indicators = "sptinc992j",
  areas      = c("US", "FR", "CN", "ZA"),
  perc       = "p99p100",
  years      = 1990:2023
)

# Gini from full distribution
dist <- download_wid("sptinc992j", areas = "US", perc = "all", years = 2022)
wid_gini(dist)
wid_top_share(dist, top = 0.01)

# Percentile ratio from threshold series
thresh <- download_wid("tptinc992j", areas = "US", perc = "all", years = 2022)
wid_percentile_ratio(thresh)

# Plot Lorenz curve
wid_plot_lorenz(dist)
```

### 4.1 DINA Income

DINA income concepts are consistent with national accounts aggregates. Each five-letter code is associated with a concept (income attributed) and a ranking variable (income used to rank individuals -- sometimes identical, sometimes not).

#### 4.1.1 Pretax Income

Pretax income includes social insurance benefits (removing corresponding contributions) but excludes other redistribution.

| WID code | Description | Ranking variable |
|----------|-------------|-----------------|
| `ptinc` | (=) pretax national income | pretax national income |
| `ptlin` | (+) pretax labor income | |
| `ptkin` | (+) pretax capital income | |
| `pllin` | (+) pretax labor income | pretax labor income |
| `pkkin` | (+) pretax capital income | pretax capital income |

#### 4.1.2 Post-tax Income

| WID code | Description | Ranking variable |
|----------|-------------|-----------------|
| `diinc` | (=) post-tax national income | post-tax national income |
| `cainc` | (=) post-tax disposable income | post-tax disposable income |

#### 4.1.3 Factor Income

Factor income measures inequality before any redistribution (before social insurance or social assistance). Sensitive to age structure -- use with care for international comparisons.

| WID code | Description | Ranking variable |
|----------|-------------|-----------------|
| `fainc` | (=) factor national income | factor national income |
| `flinc` | (=) labor factor income | labor factor income |

### 4.2 "Fiscal" Income

Fiscal income is a legacy concept from the World Top Income Database (WTID). It is close to taxable income; its exact definition varies by country and over time. In most cases, fiscal income is only available for the top 10% of the distribution.

```{r fiscal}
download_wid(
  indicators = "sfiinc992j",
  areas      = "US",
  perc       = "p99p100",
  years      = 1913:2023
)
```

| WID code | Description | Ranking variable |
|----------|-------------|-----------------|
| `fiinc` | (=) fiscal income | total fiscal income |
| `filin` | (+) fiscal labor income | |
| `fiwag` | (+) wages and pensions | |
| `fimil` | (+) labor component of mixed-income | |
| `ficap` | (+) fiscal capital income | |
| `firen` | (+) rents | |
| `fiint` | (+) interests | |
| `fidiv` | (+) dividends | |
| `fikgi` | (+) capital gains | |
| `fimik` | (+) capital component of mixed-income | |
| `fimix` | (=) mixed-income | total fiscal income |

---

## 5. Aggregate Wealth Variables

### 5.1 National Economy

```{r wealth-national}
download_wid(indicators = "mnweal999i", areas = "FR", years = 1970:2023)
```

| WID code | Description |
|----------|-------------|
| `nweal` | (=) net market-value national wealth |
| `nwnfa` | (+) national non-financial assets |
| `nwhou` | (+) national housing assets |
| `nwdwe` | (+) dwellings |
| `nwlan` | (+) land underlying dwellings |
| `nwbus` | (+) national business and other non-financial assets |
| `nwagr` | (+) agricultural land |
| `nwodk` | (+) other domestic capital |
| `nwnxa` | (+) net foreign assets |
| `nwgxa` | (+) gross foreign assets |
| `nwgxd` | (-) gross foreign liabilities |
| `nwboo` | (=) book-value national wealth |
| `nweal` | (+) net national wealth |
| `cwres` | (+) residual corporate wealth |
| `nwdka` | (=) domestic capital |
| `nweal` | (+) net national wealth |
| `nwnxa` | (-) net foreign assets |

### 5.2 Households and NPISH

#### 5.2.1 Combined Sector (Private Wealth)

| WID code | Description |
|----------|-------------|
| `pweal` | (=) private net wealth |
| `pwnfa` | (+) private non-financial assets |
| `pwhou` | (+) private housing assets |
| `pwdwe` | (+) dwellings |
| `pwlan` | (+) land underlying dwellings |
| `pwbus` | (+) private business and other non-financial assets |
| `pwagr` | (+) agricultural land |
| `pwodk` | (+) other domestic capital |
| `pwfin` | (+) financial assets |
| `pwfiw` | (+) currency, deposits, bonds, and loans |
| `pwcud` | (+) currency and deposits |
| `pwbol` | (+) bonds and loans |
| `pweqi` | (+) equities and fund shares |
| `pwoff` | (+) offshore wealth |
| `pwpen` | (+) pension funds and life insurance |
| `pwdeb` | (-) liabilities |
| `pwfie` | (=) financial assets except for currency and deposits |

#### 5.2.2 Household Sector

| WID code | Description |
|----------|-------------|
| `hweal` | (=) household net wealth |
| `hwnfa` | (+) household non-financial assets |
| `hwhou` | (+) household housing assets |
| `hwdwe` | (+) dwellings |
| `hwlan` | (+) land underlying dwellings |
| `hwbus` | (+) household business and other non-financial assets |
| `hwagr` | (+) agricultural land |
| `hwodk` | (+) other domestic capital |
| `hwfin` | (+) financial assets |
| `hwfiw` | (+) currency, deposits, bonds, and loans |
| `hwcud` | (+) currency and deposits |
| `hwbol` | (+) bonds and loans |
| `hweqi` | (+) equities and fund shares |
| `hwpen` | (+) pension funds and life insurance |
| `hwdeb` | (-) liabilities |
| `hwfie` | (=) financial assets except for currency and deposits |

#### 5.2.3 NPISH

| WID code | Description |
|----------|-------------|
| `iweal` | (=) non-profit institutions' net wealth |
| `iwnfa` | (+) non-profit institutions' non-financial assets |
| `iwhou` | (+) non-profit institutions' housing assets |
| `iwdwe` | (+) dwellings |
| `iwlan` | (+) land underlying dwellings |
| `iwbus` | (+) non-profit institutions' business and other non-financial assets |
| `iwagr` | (+) agricultural land |
| `iwodk` | (+) other domestic capital |
| `iwfin` | (+) financial assets |
| `iwfiw` | (+) currency, deposits, bonds, and loans |
| `iwcud` | (+) currency and deposits |
| `iwbol` | (+) bonds and loans |
| `iweqi` | (+) equities and fund shares |
| `iwpen` | (+) pension funds and life insurance |
| `iwdeb` | (-) liabilities |
| `iwfie` | (=) financial assets except for currency and deposits |

### 5.3 Corporate Sector

Corporate sector wealth (assets minus debt) = book value. The ratio of market value to book value = Tobin's Q.

| WID code | Description |
|----------|-------------|
| `cwboo` | (=) book value of corporations |
| `cwnfa` | (+) corporate non-financial assets |
| `cwhou` | (+) corporate housing assets |
| `cwdwe` | (+) dwellings |
| `cwlan` | (+) land underlying dwellings |
| `cwbus` | (+) corporate business and other non-financial assets |
| `cwagr` | (+) agricultural land |
| `cwodk` | (+) other domestic capital |
| `cwfin` | (+) financial assets |
| `cwfiw` | (+) currency, deposits, bonds, and loans |
| `cwcud` | (+) currency and deposits |
| `cwbol` | (+) bonds and loans |
| `cweqi` | (+) equities and fund shares |
| `cwpen` | (+) pension funds and life insurance |
| `cwdeb` | (-) corporate debt (non-equity liability) |
| `cwdeq` | (=) market value of corporations (equity liability) |
| `cwboo` | (+) book value of corporations |
| `cwres` | (+) residual corporate wealth |
| `icwtoq` | (*) Tobin Q (market value / book value) |
| `cwfie` | (=) financial assets except for currency and deposits |

### 5.4 General Government Sector

| WID code | Description |
|----------|-------------|
| `gweal` | (=) net wealth of the general government |
| `gwnfa` | (+) government non-financial assets |
| `gwhou` | (+) government housing assets |
| `gwdwe` | (+) dwellings |
| `gwlan` | (+) land underlying dwellings |
| `gwbus` | (+) government business and other non-financial assets |
| `gwagr` | (+) agricultural land |
| `gwodk` | (+) other domestic capital |
| `gwfin` | (+) financial assets |
| `gwfiw` | (+) currency, deposits, bonds, and loans |
| `gwcud` | (+) currency and deposits |
| `gwbol` | (+) bonds and loans |
| `gweqi` | (+) equities and fund shares |
| `gwpen` | (+) pension funds and life insurance |
| `gwdeb` | (-) liabilities |
| `gwdec` | (=) consolidated government debt |
| `gwfie` | (=) financial assets except for currency and deposits |

To compute national liabilities, sum liabilities across all sectors: `mpwdeb + mcwdeb + mcwdeq + mgwdeb`.

---

## 6. Distributed Wealth Variables

The distributed wealth variables are identical to the aggregate wealth variables of the household sector. Individuals are ranked by net personal wealth.

```{r wealth-dist}
download_wid(
  indicators = "shweal992j",
  areas      = "GB",
  perc       = c("p90p100", "p99p100"),
  years      = 1995:2023)

wid_plot_lorenz(download_wid("shweal992j", areas = "US", perc = "all", years = 2019))
```

| WID code | Description |
|----------|-------------|
| `hweal` | (=) household net wealth |
| `hwnfa` | (+) household non-financial assets |
| `hwhou` | (+) household housing assets |
| `hwdwe` | (+) dwellings |
| `hwlan` | (+) land underlying dwellings |
| `hwbus` | (+) household business and other non-financial assets |
| `hwagr` | (+) agricultural land |
| `hwodk` | (+) other domestic capital |
| `hwfin` | (+) financial assets |
| `hwfiw` | (+) currency, deposits, bonds, and loans |
| `hwcud` | (+) currency and deposits |
| `hwbol` | (+) bonds and loans |
| `hweqi` | (+) equities and fund shares |
| `hwpen` | (+) pension funds and life insurance |
| `hwdeb` | (-) liabilities |
| `hwfie` | (=) financial assets except for currency and deposits |
| `hwfin` | (+) financial assets |
| `hwcud` | (-) currency and deposits |

---

## 7. Price Index, Exchange Rates, Populations, etc.

### 7.1 Price Index

The WID uses a single price index (`inyixx`) -- the national income deflator, typically the GDP deflator (CPI when unavailable). It equals 1 in the data's base year.

```{r price}
download_wid(indicators = "inyixx999i", areas = "US", years = 1950:2023)
```

| Full WID code | Description |
|--------------|-------------|
| `inyixx999i` | national income price index |
| `intlcu999i` | local currency unit change indicator |

### 7.2 Exchange Rates

The WID provides PPP conversion factors and market exchange rates for USD, EUR, and CNY. To convert between currencies in the benchmark approach, use only the last value of each series. `wid_convert()` handles this automatically.

```{r fx}
# Retrieve PPP exchange rate directly
download_wid(indicators = "xlcusp999i", areas = "FR", years = 2023)

# Convert a monetary series to 2022 USD PPP in one step
download_wid("aptinc992j", areas = c("US", "FR", "CN", "IN"), perc = "p0p50") |>
  wid_convert(target = "ppp", base_year = "2022")
```

Supported `wid_convert()` targets: `"lcu"` (no conversion), `"usd"`, `"eur"`, `"gbp"`, `"ppp"`, `"yppp"`. Dimensionless series (types `s`, `g`, `b`, `w`, `r`, `p`, `i`, `y`) are returned unchanged.

**PPP Conversion Factors**

| Full WID code | Description |
|--------------|-------------|
| `xlcusp999i` | LCU per USD |
| `xlceup999i` | LCU per EUR |
| `xlcyup999i` | LCU per CNY |

**Market Exchange Rates**

| Full WID code | Description |
|--------------|-------------|
| `xlcusx999i` | LCU per USD |
| `xlceux999i` | LCU per EUR |
| `xlcyux999i` | LCU per CNY |

**Real Exchange Rates** (PPP / market rate)

| Full WID code | Description |
|--------------|-------------|
| `xrerus999i` | real exchange rate between LCU and USD |
| `xrereu999i` | real exchange rate between LCU and EUR |
| `xreryu999i` | real exchange rate between LCU and CNY |

### 7.3 Population

```{r pop}
download_wid(indicators = "npopul999i", areas = "US", years = 1800:2023)
```

| Six-letter code | Description |
|----------------|-------------|
| `npopul` | population |
| `npopem` | employed population |
| `ntaxto` | total tax population |
| `ntaxma` | number of tax units -- married couples and single adults |
| `ntaxad` | number of tax units -- adults |
| `ntaxre` | number of tax returns |

Combined with age and population unit codes (e.g., `npopul992i` for adult population).

### 7.4 Wealth/Income Ratios and Labor/Capital Shares

| Full WID code | Description |
|--------------|-------------|
| `wlabsh999i` | labor share of factor-price national income |
| `wcapsh999i` | capital share of factor-price national income |
| `wwealn999i` | net national wealth to net national income ratio |
| `wwealp999i` | net private wealth to net national income ratio |
| `wwealh999i` | net personal wealth to net national income ratio |
| `wweali999i` | net non-profit wealth to net national income ratio |
| `wwealc999i` | net corporate wealth to net national income ratio |
| `wwealg999i` | net public wealth to net national income ratio |

### 7.5 Inequality Transparency

WID produces a single inequality transparency index per country, ranging from 0 to 20, weighting income and wealth data equally. Points are awarded for: income/consumption survey tabulations (1 pt), survey microdata (up to 3 pts), wealth survey tabulations (1 pt), wealth survey microdata (up to 3 pts), income tax tabulations for labor (1 pt) and capital (1 pt), income tax microdata for labor (2 pts) and capital (2 pts), wealth/estate tax tabulations (2 pts), and wealth/estate tax microdata (4 pts).

| Full WID code | Description |
|--------------|-------------|
| `iquali999i` | inequality transparency index (0-20) |

---

## 8. Aggregate Carbon Variables

### 8.1 National Territorial Emissions

| WID code | Description |
|----------|-------------|
| `ntghg` | (=) national territorial carbon emissions |
| `ntcar` | (+) national territorial CO2 emissions |
| `ntgho` | (+) national territorial emissions of other gases |
| `hfghd` | (+) household direct emissions |
| `ntgna` | (+) territorial emissions of national productive sector and other emissions not attributed to households |
| `ntcna` | (+) territorial emissions of national productive sector (CO2) |
| `ntona` | (+) territorial emissions of national productive sector (other gases) |

### 8.2 National Imported Emissions

| WID code | Description |
|----------|-------------|
| `nighg` | (=) national imported carbon emissions |
| `nicar` | (+) national imported CO2 emissions |
| `nigho` | (+) national imported emissions of other gases |
| `highg` | (+) household imported carbon emissions |
| `hicar` | (+) household imported CO2 emissions |
| `higho` | (+) household imported emissions of other gases |
| `gighg` | (+) government imported emissions |
| `gicar` | (+) government imported CO2 emissions |
| `gigho` | (+) government imported emissions of other gases |
| `iighg` | (+) investment imported carbon emissions |
| `iicar` | (+) investment imported CO2 emissions |
| `iigho` | (+) investment imported emissions of other gases |
| `oighg` | (+) NGO imported carbon emissions |
| `oicar` | (+) NGO imported CO2 emissions |
| `oigho` | (+) NGO imported emissions of other gases |

### 8.3 National Exported Emissions

| WID code | Description |
|----------|-------------|
| `neghg` | (=) national exported carbon emissions |
| `necar` | (+) national exported CO2 emissions |
| `negho` | (+) national exported emissions of other gases |

### 8.4 National Net Imports of Emissions

| WID code | Description |
|----------|-------------|
| `nnghg` | (=) national net imports of carbon emissions |
| `nncar` | (+) national net imports of CO2 emissions |
| `nngho` | (+) national net imports of other gases |

### 8.5 National Carbon Footprint

| WID code | Description |
|----------|-------------|
| `nfghg` | (=) national carbon footprint |
| `nfcar` | (+) national CO2 footprint |
| `nfgho` | (+) national footprint of other gases |
| `hfghg` | (+) household carbon footprint |
| `hfghd` | (+) household direct emissions |
| `hfghn` | (+) household indirect carbon footprint |
| `hfcar` | (+) household indirect CO2 footprint |
| `hfgho` | (+) household indirect footprint of other gases |
| `gfghg` | (+) government carbon footprint |
| `gfcar` | (+) government CO2 footprint |
| `gfgho` | (+) government footprint of other gases |
| `ifghg` | (+) investment carbon footprint |
| `ifcar` | (+) investment CO2 footprint |
| `ifgho` | (+) investment footprint of other gases |
| `ofghg` | (+) NGO carbon footprint |
| `ofcar` | (+) NGO CO2 footprint |
| `ofgho` | (+) NGO footprint of other gases |

```{r carbon-agg}
download_wid(indicators = "entghg999i", areas = "US", years = 1990:2023)
download_wid(indicators = "enfghg999i", areas = "US", years = 1990:2023)
```

---

## 9. Distributed Carbon Variables

Personal carbon footprints assign national carbon and CO2 footprints to income groups within countries, distinguishing between consumption-related and investment-related emissions.

```{r carbon-dist}
# WID canonical example: top 10% average per capita footprint, Switzerland 2014
download_wid(
  indicators = "lpfghg999i",
  areas      = "CH",
  perc       = "p90p100",
  years      = 2014)

# Bottom 50% share of emissions
download_wid(
  indicators = "spfghg999i",
  areas      = c("US", "FR", "IN"),
  perc       = "p0p50",
  years      = 2019)
```

| WID code | Description |
|----------|-------------|
| `pfcar` | (=) personal CO2 footprint |
| `pfghg` | (=) personal carbon footprint |
| `cfghg` | (+) personal carbon footprint (consumption only) |
| `ifghg` | (+) personal carbon footprint (investments only) |

Series types applicable to distributed carbon:

| Type | Example | Description |
|------|---------|-------------|
| `l` | `lpfghg999i` | average per capita group emissions |
| `e` | `epfghg999i` | total group emissions |
| `s` | `spfghg999i` | share of total emissions |
| `t` | `tpfghg999i` | threshold (minimum footprint to enter the group) |
| `k` | `kpfghg999i` | per capita emissions |

---

## 10. Quick Reference

`print.wid_df` and `print.wid_query` are S3 print methods; see `vignette("widr")` for display examples and full workflow demonstrations.

| Function | Purpose |
|----------|---------|
| `download_wid()` | Download data; returns a `wid_df` |
| `wid_decode()` | Parse a variable code string into components |
| `wid_encode()` | Build a variable code string from components |
| `wid_validate()` | Validate components; stops on failure |
| `wid_is_valid()` | Non-throwing validation; returns `TRUE`/`FALSE` |
| `wid_search()` | Keyword/regex search across reference tables |
| `wid_tidy()` | Coerce types; append decoded and country name columns |
| `wid_convert()` | Currency conversion (`"lcu"`, `"usd"`, `"eur"`, `"gbp"`, `"ppp"`, `"yppp"`) |
| `wid_metadata()` | Retrieve source and methodological documentation |
| `wid_gini()` | Gini coefficient from a share (`s`) series |
| `wid_top_share()` | Top fractile income/wealth share |
| `wid_percentile_ratio()` | Percentile threshold ratio (default P90/P10) |
| `wid_plot_timeseries()` | Time-series line chart; returns a `ggplot` object |
| `wid_plot_compare()` | Cross-country bar/point chart; returns a `ggplot` object |
| `wid_plot_lorenz()` | Lorenz curve; returns a `ggplot` object |
| `wid_query()` | Construct a reusable query object |
| `wid_filter()` | Update fields of a `wid_query` |
| `wid_fetch()` | Execute a `wid_query`; returns a `wid_df` |
| `wid_cache()` | Low-level cache access (`"get"`, `"set"`, `"list"`, `"clear"`) |
| `wid_cache_list()` | List cached query keys |
| `wid_cache_clear()` | Remove all cached responses |
| `wid_set_key()` | Set the WID API key |
| `print.wid_df` | S3 print method for `wid_df` objects |
| `print.wid_query` | S3 print method for `wid_query` objects |

| Dataset | Contents |
|---------|----------|
| `wid_series_types` | `r nrow(wid_series_types)` series type codes |
| `wid_concepts` | `r nrow(wid_concepts)` concept codes |
| `wid_ages` | Age group codes |
| `wid_pop_types` | Population unit codes |
| `wid_countries` | `r nrow(wid_countries)` country and region codes |
| `wid_percentiles` | `r nrow(wid_percentiles)` percentile group codes |

---

**Disclaimer:** `widr` is an independent implementation unaffiliated with the World Inequality Lab (WIL) or the Paris School of Economics. Data are sourced from WID.world, maintained by WIL. Codes reference: [wid.world/codes-dictionary](https://wid.world/codes-dictionary/).

Full getting-started guide: `vignette("widr")`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transforms.R
\name{wid_tidy}
\alias{wid_tidy}
\title{Tidy a wid_df: decode variable codes and coerce column types}
\usage{
wid_tidy(data, decode = TRUE, country_names = TRUE)
}
\arguments{
\item{data}{A \code{wid_df}.}

\item{decode}{Logical. Add decoded columns. Default \code{TRUE}.}

\item{country_names}{Logical. Join country display names. Default \code{TRUE}.}
}
\value{
A plain \code{data.frame} with \code{year} as integer, \code{value}
as double, and optionally \code{indicator}, \code{series_type},
\code{type_label}, and \code{country_name} columns appended.
}
\description{
Tidy a wid_df: decode variable codes and coerce column types
}
\examples{
d <- data.frame(
  country = "US", variable = "sptinc992j", percentile = "p99p100",
  year = "2020", value = "0.19", age = "992", pop = "j",
  stringsAsFactors = FALSE
)
class(d) <- c("wid_df", "data.frame")
wid_tidy(d, country_names = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widr.R
\name{wid_set_key}
\alias{wid_set_key}
\title{Set WID API key}
\usage{
wid_set_key(key)
}
\arguments{
\item{key}{character API key}
}
\value{
Invisibly, the key string.
}
\description{
Set WID API key
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inequality.R
\name{wid_gini}
\alias{wid_gini}
\title{Compute Gini coefficient from a share series}
\usage{
wid_gini(data, variable = NULL, country = NULL)
}
\arguments{
\item{data}{A \code{wid_df} with share series.}

\item{variable}{Variable code filter.}

\item{country}{Country code filter.}
}
\value{
A \code{data.frame} with columns \code{country}, \code{year},
and \code{gini} (numeric, 0-1).
}
\description{
Requires an \code{"s"} (share) series with \code{pXpY} percentile codes
covering the full distribution.
}
\examples{
d <- data.frame(
  country = rep("US", 3L), year = rep("2020", 3L),
  variable = rep("sptinc992j", 3L),
  percentile = c("p0p50", "p50p90", "p90p100"),
  value = c(0.20, 0.40, 0.40), age = "992", pop = "j",
  stringsAsFactors = FALSE
)
class(d) <- c("wid_df", "data.frame")
wid_gini(d)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concepts.R, R/data.R
\docType{data}
\name{wid_series_types}
\alias{wid_series_types}
\title{Series type codes (\\S2.1.1)}
\format{
data.frame(code, description)

A data frame with columns \code{code} and \code{description}.
}
\source{
\url{https://wid.world/codes-dictionary/}

\url{https://wid.world/codes-dictionary/}
}
\usage{
wid_series_types

wid_series_types
}
\description{
Series type codes (\\S2.1.1)

WID series type codes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R
\name{wid_validate}
\alias{wid_validate}
\title{Validate WID code components}
\usage{
wid_validate(
  series_type = NULL,
  concept = NULL,
  age = NULL,
  pop = NULL,
  years = NULL,
  areas = NULL,
  perc = NULL
)
}
\arguments{
\item{series_type}{One-letter series type code.}

\item{concept}{5-6 letter concept code.}

\item{age}{Three-digit age code.}

\item{pop}{One-letter population unit code.}

\item{years}{Integer vector of years.}

\item{areas}{ISO-2 area codes.}

\item{perc}{Percentile codes of the form \code{pXpY}.}
}
\value{
Invisibly, a named list of normalised inputs with the same names as
the parameters, with \code{age} zero-padded and \code{years} coerced to
integer.
}
\description{
Validate WID code components
}
\examples{
wid_validate(series_type = "s", concept = "ptinc", age = 992, pop = "j")
wid_validate(areas = c("US", "FR", "US-CA"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inequality.R
\name{wid_percentile_ratio}
\alias{wid_percentile_ratio}
\title{Percentile threshold ratio (e.g. P90/P10)}
\usage{
wid_percentile_ratio(
  data,
  numerator = "p90",
  denominator = "p10",
  variable = NULL,
  country = NULL
)
}
\arguments{
\item{data}{A \code{wid_df} with threshold (\code{"t"}) series.}

\item{numerator}{Upper percentile code, e.g. \code{"p90"}.}

\item{denominator}{Lower percentile code, e.g. \code{"p10"}.}

\item{variable}{Variable code filter.}

\item{country}{Country code filter.}
}
\value{
A \code{data.frame} with columns \code{country}, \code{year},
and \code{ratio} (numeric: numerator threshold divided by denominator threshold).
}
\description{
Percentile threshold ratio (e.g. P90/P10)
}
\examples{
d <- data.frame(
  country = rep("US", 2L), year = rep("2020", 2L),
  variable = rep("tptinc992j", 2L),
  percentile = c("p90", "p10"), value = c(200000, 10000),
  age = "992", pop = "j", stringsAsFactors = FALSE
)
class(d) <- c("wid_df", "data.frame")
wid_percentile_ratio(d)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R
\name{wid_is_valid}
\alias{wid_is_valid}
\title{Test WID code components without throwing}
\usage{
wid_is_valid(...)
}
\arguments{
\item{...}{Arguments passed to \code{\link{wid_validate}}.}
}
\value{
Scalar logical: \code{TRUE} if all supplied components are valid,
\code{FALSE} if any would cause an error. Warning-level issues return
\code{TRUE}.
}
\description{
Test WID code components without throwing
}
\examples{
wid_is_valid(series_type = "s", concept = "ptinc")  # TRUE
wid_is_valid(series_type = "Z")                      # FALSE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R
\name{wid_decode}
\alias{wid_decode}
\title{Parse a WID variable code into its components}
\usage{
wid_decode(x, strict = TRUE)
}
\arguments{
\item{x}{Character string, e.g. \code{"sptinc992j"}.}

\item{strict}{\code{TRUE} (default) stops on error; \code{FALSE} warns and returns \code{NULL}.}
}
\value{
Named list with elements \code{series_type} (character),
\code{concept} (character), \code{age} (character or \code{NULL}),
and \code{pop} (character or \code{NULL}).
}
\description{
Parse a WID variable code into its components
}
\examples{
wid_decode("sptinc992j")
wid_decode("mnninc")        # no age or pop
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{print.wid_query}
\alias{print.wid_query}
\title{Print a wid_query object}
\usage{
\method{print}{wid_query}(x, ...)
}
\arguments{
\item{x}{A \code{wid_query} object.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the input \code{x}.
}
\description{
Displays the contents of a \code{wid_query} object.
}
\examples{
q <- wid_query(indicators = "sptinc992j", areas = "US")
print(q)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R
\name{wid_encode}
\alias{wid_encode}
\title{Build a WID variable code from its components}
\usage{
wid_encode(series_type, concept = NULL, age = NULL, pop = NULL)
}
\arguments{
\item{series_type}{1-letter series type, or a list from \code{\link{wid_decode}}.}

\item{concept}{5-6 letter concept code.}

\item{age}{3-digit age code or \code{NULL}.}

\item{pop}{1-letter population code or \code{NULL}.}
}
\value{
A character string of the form
\code{<type><concept>[age][pop]}.
}
\description{
Build a WID variable code from its components
}
\examples{
wid_encode("s", "ptinc", "992", "j")   # "sptinc992j"
wid_encode("m", "nninc")               # "mnninc"
wid_encode(wid_decode("sptinc992j"))   # round-trip
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{wid_plot_compare}
\alias{wid_plot_compare}
\title{Cross-country comparison bar/point chart}
\usage{
wid_plot_compare(
  data,
  year = NULL,
  variable = NULL,
  country = NULL,
  country_labels = NULL,
  type = "bar"
)
}
\arguments{
\item{data}{A \code{wid_df}.}

\item{year}{Integer year (defaults to most recent common year).}

\item{variable}{Variable code to plot.}

\item{country}{Country codes to include.}

\item{country_labels}{Named character vector for renaming countries.}

\item{type}{\code{"bar"} (default) or \code{"point"}.}
}
\value{
A \code{ggplot} object.
}
\description{
Cross-country comparison bar/point chart
}
\examples{
\donttest{
  d <- data.frame(
    country = rep(c("US", "FR"), each = 3L),
    year    = rep(c("2018", "2019", "2020"), 2L),
    variable = "sptinc992j", percentile = "p99p100",
    value = c(0.20, 0.19, 0.19, 0.11, 0.11, 0.12),
    age = "992", pop = "j", stringsAsFactors = FALSE
  )
  class(d) <- c("wid_df", "data.frame")
  wid_plot_compare(d, year = 2020)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{download_wid}
\alias{download_wid}
\alias{download}
\title{Download data from WID.world}
\usage{
download_wid(
  indicators = "all",
  areas = "all",
  years = "all",
  perc = "all",
  ages = "992",
  pop = "j",
  metadata = FALSE,
  include_extrapolations = TRUE,
  verbose = FALSE,
  cache = TRUE
)

download(
  indicators = "all",
  areas = "all",
  years = "all",
  perc = "all",
  ages = "992",
  pop = "j",
  metadata = FALSE,
  include_extrapolations = TRUE,
  verbose = FALSE,
  cache = TRUE
)
}
\arguments{
\item{indicators}{WID variable codes or \code{"all"}.}

\item{areas}{ISO-2 area codes or \code{"all"}.}

\item{years}{Integer vector or \code{"all"}.}

\item{perc}{Percentile codes or \code{"all"}.}

\item{ages}{Age codes or \code{"all"}. Default \code{"992"}.}

\item{pop}{Population unit codes or \code{"all"}. Default \code{"j"}.}

\item{metadata}{Logical. If \code{TRUE}, fetches metadata as a
\code{"wid_meta"} attribute on the returned object. Default \code{FALSE}.}

\item{include_extrapolations}{Logical. Default \code{TRUE}.}

\item{verbose}{Logical. Default \code{FALSE}.}

\item{cache}{Logical. Default \code{TRUE}.}
}
\value{
A \code{wid_df} (a classed \code{data.frame}) with columns
\code{country}, \code{variable}, \code{percentile}, \code{year},
\code{value}, \code{age}, \code{pop}. If \code{metadata = TRUE},
a \code{"wid_meta"} attribute is attached.
}
\description{
Returns a \code{wid_df}, a classed \code{data.frame} that works natively
with dplyr, ggplot2, and all base-R operations.
}
\examples{
\donttest{
  if (nzchar(Sys.getenv("WID_API_KEY"))) {
    d <- download_wid(
      indicators = "sptinc992j",
      areas      = "US",
      perc       = "p99p100",
      years      = 2010:2020
    )
  }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{wid_fetch}
\alias{wid_fetch}
\title{Execute a wid_query}
\usage{
wid_fetch(query, ...)
}
\arguments{
\item{query}{A \code{wid_query} object.}

\item{...}{Parameter overrides.}
}
\value{
A \code{wid_df} (a classed \code{data.frame}); see \code{\link{download_wid}}.
}
\description{
Fetch data from WID.world using a \code{wid_query} object.
}
\examples{
\donttest{
  if (nzchar(Sys.getenv("WID_API_KEY"))) {
    q <- wid_query(indicators = "sptinc992j", areas = "US", years = 2010:2020)
    wid_fetch(q)
  }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{wid_plot_lorenz}
\alias{wid_plot_lorenz}
\title{Lorenz curve}
\usage{
wid_plot_lorenz(data, variable = NULL, country = NULL, country_labels = NULL)
}
\arguments{
\item{data}{A \code{wid_df} with share (\code{"s"}) series.}

\item{variable}{Variable code to plot.}

\item{country}{Country codes to include.}

\item{country_labels}{Named character vector for renaming countries.}
}
\value{
A \code{ggplot} object.
}
\description{
Lorenz curve
}
\examples{
\donttest{
  d <- data.frame(
    country = rep("US", 3L), year = rep("2020", 3L),
    variable = "sptinc992j",
    percentile = c("p0p50", "p50p90", "p90p100"),
    value = c(0.20, 0.40, 0.40), age = "992", pop = "j",
    stringsAsFactors = FALSE
  )
  class(d) <- c("wid_df", "data.frame")
  wid_plot_lorenz(d)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widr.R
\name{wid_cache}
\alias{wid_cache}
\title{Interact with the WID response cache}
\usage{
wid_cache(action, key = NULL, value = NULL)
}
\arguments{
\item{action}{One of \code{"get"}, \code{"set"}, \code{"list"}, \code{"clear"}.}

\item{key}{Cache key string (required for \code{"get"} and \code{"set"}).}

\item{value}{Value to store (\code{"set"} only).}
}
\value{
For \code{"get"}: the cached object or \code{NULL}. For
\code{"list"}: character vector of cache keys. For \code{"set"} and
\code{"clear"}: invisibly, the stored value or file count removed.
}
\description{
Low-level access to get, set, list, or clear cached API responses.
}
\examples{
wid_cache("list")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concepts.R, R/data.R
\docType{data}
\name{wid_concepts}
\alias{wid_concepts}
\title{Concept codes (\\S3-9)}
\format{
data.frame(code, description)

A data frame with columns \code{code} and \code{description}.
}
\source{
\url{https://wid.world/codes-dictionary/}

\url{https://wid.world/codes-dictionary/}
}
\usage{
wid_concepts

wid_concepts
}
\description{
Letters 2-6 of a WID variable code. Scraped from \\S3-6, 8-9 tables; \\S7 hardcoded from prose.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concepts.R, R/data.R
\docType{data}
\name{wid_pop_types}
\alias{wid_pop_types}
\title{Population unit codes (\\S2.1.4)}
\format{
data.frame(code, description)

A data frame with columns \code{code} and \code{description}.
}
\source{
\url{https://wid.world/codes-dictionary/}

\url{https://wid.world/codes-dictionary/}
}
\usage{
wid_pop_types

wid_pop_types
}
\description{
Population unit codes (\\S2.1.4)

WID population unit codes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inequality.R
\name{wid_top_share}
\alias{wid_top_share}
\title{Top income or wealth share}
\usage{
wid_top_share(data, top = 0.1, variable = NULL, country = NULL)
}
\arguments{
\item{data}{A \code{wid_df} with share series.}

\item{top}{Top fraction, e.g. 0.1 = top 10\\\%.}

\item{variable}{Variable code filter.}

\item{country}{Country code filter.}
}
\value{
A \code{data.frame} with columns \code{country}, \code{year},
\code{top} (the requested fractile), and \code{share} (numeric).
}
\description{
Top income or wealth share
}
\examples{
d <- data.frame(
  country = "US", year = "2020", variable = "sptinc992j",
  percentile = "p99p100", value = 0.19, age = "992", pop = "j",
  stringsAsFactors = FALSE
)
class(d) <- c("wid_df", "data.frame")
wid_top_share(d, top = 0.01)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transforms.R
\name{wid_metadata}
\alias{wid_metadata}
\title{Fetch metadata for variables in a wid_df}
\usage{
wid_metadata(data)
}
\arguments{
\item{data}{A \code{wid_df}.}
}
\value{
A \code{data.frame} with columns \code{variable}, \code{country},
\code{source}, \code{method}, \code{quality}, and \code{imputation}.
}
\description{
Fetch metadata for variables in a wid_df
}
\examples{
\donttest{
  if (nzchar(Sys.getenv("WID_API_KEY"))) {
    d <- download_wid("sptinc992j", areas = "US", metadata = TRUE)
    wid_metadata(d)
  }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R
\name{wid_search}
\alias{wid_search}
\title{Search WID lookup tables by regex}
\usage{
wid_search(query, tables = "concepts", type = NULL)
}
\arguments{
\item{query}{Regular expression.}

\item{tables}{Table names to search. Default \code{"concepts"}.}

\item{type}{Optional series-type letter to filter results.}
}
\value{
A \code{data.frame} with columns \code{table},
\code{variable}, and \code{description} for each matching row,
or an invisible empty \code{data.frame} if no matches are found.
}
\description{
Search WID lookup tables by regex
}
\examples{
wid_search("national income")
wid_search("wealth", type = "s")
wid_search("^US", tables = "countries")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widr.R
\docType{package}
\name{widr-package}
\alias{widr-package}
\alias{widr}
\title{widr: Interface to the World Inequality Database (WID)}
\description{
Interface to the World Inequality Database (WID) API \url{https://wid.world}. Downloads distributional national accounts data with filters for country, year, percentile, age group, and population type. Includes code validation and reference tables. Independent implementation unaffiliated with the World Inequality Lab (WIL) or the Paris School of Economics.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/cherylisabella/widr}
  \item Report bugs at \url{https://github.com/cherylisabella/widr/issues}
}

}
\author{
\strong{Maintainer}: Cheryl Isabella Lim \email{cheryl.academic@gmail.com}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widr.R
\name{wid_cache_list}
\alias{wid_cache_list}
\title{List cached WID API responses}
\usage{
wid_cache_list()
}
\value{
Character vector of cache file names.
}
\description{
List cached WID API responses
}
\examples{
wid_cache_list()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{print.wid_df}
\alias{print.wid_df}
\title{Print a wid_df object}
\usage{
\method{print}{wid_df}(x, ...)
}
\arguments{
\item{x}{A \code{wid_df} object.}

\item{...}{Additional arguments passed to \code{print()}.}
}
\value{
Invisibly returns the input \code{x}.
}
\description{
This method prints a summary of a \code{wid_df} object,
including the number of rows, countries, and variables.
}
\examples{
\donttest{
d <- data.frame(country = "US", variable = "sptinc992j",
                percentile = "p99p100", year = "2020",
                value = 0.19, age = "992", pop = "j",
                stringsAsFactors = FALSE)
class(d) <- c("wid_df", "data.frame")
print(d)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{wid_filter}
\alias{wid_filter}
\title{Update a wid_query object}
\usage{
wid_filter(query, ...)
}
\arguments{
\item{query}{A \code{wid_query} object.}

\item{...}{Fields to update.}
}
\value{
The updated \code{wid_query} object.
}
\description{
Modify fields of a \code{wid_query} object, returning an updated query.
}
\examples{
q <- wid_query(indicators = "sptinc992j", areas = "US")
q <- wid_filter(q, years = 2010:2020)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widr.R
\name{wid_cache_clear}
\alias{wid_cache_clear}
\title{Clear all cached WID API responses}
\usage{
wid_cache_clear()
}
\value{
Invisibly, the number of files removed.
}
\description{
Clear all cached WID API responses
}
\examples{
wid_cache_clear()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download.R
\name{wid_query}
\alias{wid_query}
\title{Build a reusable WID query object}
\usage{
wid_query(...)
}
\arguments{
\item{...}{Named arguments matching \code{\link{download_wid}()} parameters.}
}
\value{
A \code{wid_query} object (a named list with class \code{"wid_query"}).
}
\description{
Creates a \code{wid_query} object that stores query parameters for WID data fetching.
}
\examples{
q <- wid_query(indicators = "sptinc992j", areas = "US", years = 2010:2020)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transforms.R
\name{wid_convert}
\alias{wid_convert}
\title{Convert a wid_df to another currency}
\usage{
wid_convert(data, target = "usd", base_year = NULL)
}
\arguments{
\item{data}{A \code{wid_df}.}

\item{target}{One of \code{"lcu"}, \code{"usd"}, \code{"eur"}, \code{"gbp"}, \code{"ppp"}, \code{"yppp"}.}

\item{base_year}{Pin to a specific year's exchange rate; \code{NULL} uses contemporaneous rates.}
}
\value{
The same \code{wid_df} with values converted. Dimensionless series
(shares, ratios) are returned unchanged with a message.
}
\description{
Convert a wid_df to another currency
}
\examples{
\donttest{
  if (nzchar(Sys.getenv("WID_API_KEY"))) {
    d <- download_wid("aptinc992j", areas = "US", perc = "p99p100",
                      years = 2010:2020)
    wid_convert(d, target = "ppp")
  }
}
}
